@@ -35,7 +35,7 @@
 WINE_DEFAULT_DEBUG_CHANNEL(amstream);
 
 typedef struct {
-    IAMMultiMediaStream lpVtbl;
+    IAMMultiMediaStream IAMMultiMediaStream_iface;
     LONG ref;
     IGraphBuilder* pFilterGraph;
     IPin* ipin;
@@ -45,6 +45,11 @@ typedef struct {
     STREAM_TYPE StreamType;
 } IAMMultiMediaStreamImpl;
 
+static inline IAMMultiMediaStreamImpl *impl_from_IAMMultiMediaStream(IAMMultiMediaStream *iface)
+{
+    return CONTAINING_RECORD(iface, IAMMultiMediaStreamImpl, IAMMultiMediaStream_iface);
+}
+
 static const struct IAMMultiMediaStreamVtbl AM_Vtbl;
 
 HRESULT AM_create(IUnknown *pUnkOuter, LPVOID *ppObj)
@@ -63,7 +68,7 @@ HRESULT AM_create(IUnknown *pUnkOuter, LPVOID *ppObj)
         return E_OUTOFMEMORY;
     }
 
-    object->lpVtbl.lpVtbl = &AM_Vtbl;
+    object->IAMMultiMediaStream_iface.lpVtbl = &AM_Vtbl;
     object->ref = 1;
 
     *ppObj = object;
@@ -74,14 +79,15 @@ HRESULT AM_create(IUnknown *pUnkOuter, LPVOID *ppObj)
 /*** IUnknown methods ***/
 static HRESULT WINAPI IAMMultiMediaStreamImpl_QueryInterface(IAMMultiMediaStream* iface, REFIID riid, void** ppvObject)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     TRACE("(%p/%p)->(%s,%p)\n", iface, This, debugstr_guid(riid), ppvObject);
 
     if (IsEqualGUID(riid, &IID_IUnknown) ||
+        IsEqualGUID(riid, &IID_IMultiMediaStream) ||
         IsEqualGUID(riid, &IID_IAMMultiMediaStream))
     {
-        IClassFactory_AddRef(iface);
+        IUnknown_AddRef(iface);
         *ppvObject = This;
         return S_OK;
     }
@@ -93,7 +99,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_QueryInterface(IAMMultiMediaStream
 
 static ULONG WINAPI IAMMultiMediaStreamImpl_AddRef(IAMMultiMediaStream* iface)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     TRACE("(%p/%p)\n", iface, This);
 
@@ -102,7 +108,7 @@ static ULONG WINAPI IAMMultiMediaStreamImpl_AddRef(IAMMultiMediaStream* iface)
 
 static ULONG WINAPI IAMMultiMediaStreamImpl_Release(IAMMultiMediaStream* iface)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
     TRACE("(%p/%p)\n", iface, This);
@@ -116,7 +122,7 @@ static ULONG WINAPI IAMMultiMediaStreamImpl_Release(IAMMultiMediaStream* iface)
 /*** IMultiMediaStream methods ***/
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetInformation(IAMMultiMediaStream* iface, DWORD* pdwFlags, STREAM_TYPE* pStreamType)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%p,%p) stub!\n", This, iface, pdwFlags, pStreamType);
 
@@ -125,11 +131,11 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetInformation(IAMMultiMediaStream
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetMediaStream(IAMMultiMediaStream* iface, REFMSPID idPurpose, IMediaStream** ppMediaStream)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
     MSPID PurposeId;
     unsigned int i;
 
-    TRACE("(%p/%p)->(%p,%p)\n", This, iface, idPurpose, ppMediaStream);
+    TRACE("(%p/%p)->(%s,%p)\n", This, iface, debugstr_guid(idPurpose), ppMediaStream);
 
     for (i = 0; i < This->nbStreams; i++)
     {
@@ -147,7 +153,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetMediaStream(IAMMultiMediaStream
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_EnumMediaStreams(IAMMultiMediaStream* iface, LONG Index, IMediaStream** ppMediaStream)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%d,%p) stub!\n", This, iface, Index, ppMediaStream);
 
@@ -156,7 +162,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_EnumMediaStreams(IAMMultiMediaStre
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetState(IAMMultiMediaStream* iface, STREAM_STATE* pCurrentState)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%p) stub!\n", This, iface, pCurrentState);
 
@@ -165,7 +171,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetState(IAMMultiMediaStream* ifac
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_SetState(IAMMultiMediaStream* iface, STREAM_STATE NewState)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->() stub!\n", This, iface);
 
@@ -174,7 +180,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_SetState(IAMMultiMediaStream* ifac
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetTime(IAMMultiMediaStream* iface, STREAM_TIME* pCurrentTime)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%p) stub!\n", This, iface, pCurrentTime);
 
@@ -183,7 +189,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetTime(IAMMultiMediaStream* iface
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetDuration(IAMMultiMediaStream* iface, STREAM_TIME* pDuration)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%p) stub!\n", This, iface, pDuration);
 
@@ -192,7 +198,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetDuration(IAMMultiMediaStream* i
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_Seek(IAMMultiMediaStream* iface, STREAM_TIME SeekTime)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->() stub!\n", This, iface);
 
@@ -201,7 +207,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_Seek(IAMMultiMediaStream* iface, S
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetEndOfStream(IAMMultiMediaStream* iface, HANDLE* phEOS)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%p) stub!\n", This, iface, phEOS);
 
@@ -211,10 +217,10 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetEndOfStream(IAMMultiMediaStream
 /*** IAMMultiMediaStream methods ***/
 static HRESULT WINAPI IAMMultiMediaStreamImpl_Initialize(IAMMultiMediaStream* iface, STREAM_TYPE StreamType, DWORD dwFlags, IGraphBuilder* pFilterGraph)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
     HRESULT hr = S_OK;
 
-    FIXME("(%p/%p)->(%x,%x,%p) partial stub!\n", This, iface, (DWORD)StreamType, dwFlags, pFilterGraph);
+    TRACE("(%p/%p)->(%x,%x,%p)\n", This, iface, (DWORD)StreamType, dwFlags, pFilterGraph);
 
     if (pFilterGraph)
     {
@@ -236,7 +242,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_Initialize(IAMMultiMediaStream* if
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilterGraph(IAMMultiMediaStream* iface, IGraphBuilder** ppGraphBuilder)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     TRACE("(%p/%p)->(%p)\n", This, iface, ppGraphBuilder);
 
@@ -253,7 +259,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilterGraph(IAMMultiMediaStream
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilter(IAMMultiMediaStream* iface, IMediaStreamFilter** ppFilter)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%p) stub!\n", This, iface, ppFilter); 
 
@@ -263,29 +269,22 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilter(IAMMultiMediaStream* ifa
 static HRESULT WINAPI IAMMultiMediaStreamImpl_AddMediaStream(IAMMultiMediaStream* iface, IUnknown* pStreamObject, const MSPID* PurposeId,
                                           DWORD dwFlags, IMediaStream** ppNewStream)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
     HRESULT hr;
     IMediaStream* pStream;
     IMediaStream** pNewStreams;
 
-    FIXME("(%p/%p)->(%p,%p,%x,%p) partial stub!\n", This, iface, pStreamObject, PurposeId, dwFlags, ppNewStream);
-
-    if (IsEqualGUID(PurposeId, &MSPID_PrimaryVideo))
-        hr = DirectDrawMediaStream_create((IMultiMediaStream*)iface, PurposeId, This->StreamType, &pStream);
-    else
-        hr = MediaStream_create((IMultiMediaStream*)iface, PurposeId, This->StreamType, &pStream);
+    FIXME("(%p/%p)->(%p,%s,%x,%p) partial stub!\n", This, iface, pStreamObject, debugstr_guid(PurposeId), dwFlags, ppNewStream);
 
+    hr = mediastream_create((IMultiMediaStream*)iface, PurposeId, This->StreamType, &pStream);
     if (SUCCEEDED(hr))
     {
-        pNewStreams = CoTaskMemAlloc((This->nbStreams+1)*sizeof(IMediaStream*));
+        pNewStreams = CoTaskMemRealloc(This->pStreams, (This->nbStreams+1) * sizeof(IMediaStream*));
         if (!pNewStreams)
         {
             IMediaStream_Release(pStream);
             return E_OUTOFMEMORY;
         }
-        if (This->nbStreams)
-            CopyMemory(pNewStreams, This->pStreams, This->nbStreams*sizeof(IMediaStream*));
-        CoTaskMemFree(This->pStreams);
         This->pStreams = pNewStreams;
         This->pStreams[This->nbStreams] = pStream;
         This->nbStreams++;
@@ -300,7 +299,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_AddMediaStream(IAMMultiMediaStream
 static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenFile(IAMMultiMediaStream* iface, LPCWSTR pszFileName, DWORD dwFlags)
 {
     HRESULT ret;
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
     IFileSourceFilter *SourceFilter;
     IBaseFilter *BaseFilter;
     IEnumPins *EnumPins;
@@ -349,6 +348,14 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenFile(IAMMultiMediaStream* ifac
         goto end;
     }
 
+    /* If Initialize was not called before, we do it here */
+    if (!This->pFilterGraph)
+    {
+        ret = IAMMultiMediaStream_Initialize(iface, STREAMTYPE_READ, 0, NULL);
+        if (FAILED(ret))
+            goto end;
+    }
+
     ret = IFilterGraph_QueryInterface(This->pFilterGraph, &IID_IGraphBuilder, (void**)&This->GraphBuilder);
     if(ret != S_OK)
     {
@@ -365,7 +372,7 @@ end:
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenMoniker(IAMMultiMediaStream* iface, IBindCtx* pCtx, IMoniker* pMoniker, DWORD dwFlags)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%p,%p,%x) stub!\n", This, iface, pCtx, pMoniker, dwFlags);
 
@@ -374,7 +381,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenMoniker(IAMMultiMediaStream* i
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_Render(IAMMultiMediaStream* iface, DWORD dwFlags)
 {
-    IAMMultiMediaStreamImpl *This = (IAMMultiMediaStreamImpl *)iface;
+    IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
     FIXME("(%p/%p)->(%x) partial stub!\n", This, iface, dwFlags);
 
