@@ -962,7 +962,7 @@ static void MONTHCAL_PaintWeeknumbers(const MONTHCAL_INFO *infoPtr, HDC hdc, con
   r = infoPtr->calendars[calIdx].weeknums;
 
   /* erase whole week numbers area */
-  FillRect(hdc, &r, infoPtr->brushes[BrushTitle]);
+  FillRect(hdc, &r, infoPtr->brushes[BrushMonth]);
   SetTextColor(hdc, infoPtr->colors[MCSC_TITLEBK]);
 
   /* reduce rectangle to one week number */
@@ -1675,6 +1675,8 @@ MONTHCAL_UpdateToday(MONTHCAL_INFO *infoPtr, const SYSTEMTIME *today)
   /* only two days need redrawing */
   InvalidateRect(infoPtr->hwndSelf, &old_r, FALSE);
   InvalidateRect(infoPtr->hwndSelf, &new_r, FALSE);
+  /* and today label */
+  InvalidateRect(infoPtr->hwndSelf, &infoPtr->todayrect, FALSE);
   return TRUE;
 }
 
@@ -1882,24 +1884,25 @@ MONTHCAL_HitTest(const MONTHCAL_INFO *infoPtr, MCHITTESTINFO *lpht)
 /* MCN_GETDAYSTATE notification helper */
 static void MONTHCAL_NotifyDayState(MONTHCAL_INFO *infoPtr)
 {
-  if(infoPtr->dwStyle & MCS_DAYSTATE) {
-    NMDAYSTATE nmds;
+  MONTHDAYSTATE *state;
+  NMDAYSTATE nmds;
 
-    nmds.nmhdr.hwndFrom = infoPtr->hwndSelf;
-    nmds.nmhdr.idFrom   = GetWindowLongPtrW(infoPtr->hwndSelf, GWLP_ID);
-    nmds.nmhdr.code     = MCN_GETDAYSTATE;
-    nmds.cDayState	= MONTHCAL_GetMonthRange(infoPtr, GMR_DAYSTATE, 0);
-    nmds.prgDayState	= Alloc(nmds.cDayState * sizeof(MONTHDAYSTATE));
+  if (!(infoPtr->dwStyle & MCS_DAYSTATE)) return;
 
-    MONTHCAL_GetMinDate(infoPtr, &nmds.stStart);
-    nmds.stStart.wDay = 1;
+  nmds.nmhdr.hwndFrom = infoPtr->hwndSelf;
+  nmds.nmhdr.idFrom   = GetWindowLongPtrW(infoPtr->hwndSelf, GWLP_ID);
+  nmds.nmhdr.code     = MCN_GETDAYSTATE;
+  nmds.cDayState      = MONTHCAL_GetMonthRange(infoPtr, GMR_DAYSTATE, 0);
+  nmds.prgDayState    = state = Alloc(nmds.cDayState * sizeof(MONTHDAYSTATE));
 
-    SendMessageW(infoPtr->hwndNotify, WM_NOTIFY, nmds.nmhdr.idFrom, (LPARAM)&nmds);
-    memcpy(infoPtr->monthdayState, nmds.prgDayState,
-        MONTHCAL_GetMonthRange(infoPtr, GMR_DAYSTATE, 0)*sizeof(MONTHDAYSTATE));
+  MONTHCAL_GetMinDate(infoPtr, &nmds.stStart);
+  nmds.stStart.wDay = 1;
 
-    Free(nmds.prgDayState);
-  }
+  SendMessageW(infoPtr->hwndNotify, WM_NOTIFY, nmds.nmhdr.idFrom, (LPARAM)&nmds);
+  memcpy(infoPtr->monthdayState, nmds.prgDayState,
+      MONTHCAL_GetMonthRange(infoPtr, GMR_DAYSTATE, 0)*sizeof(MONTHDAYSTATE));
+
+  Free(state);
 }
 
 /* no valid range check performed */
