@@ -17,7 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include <assert.h>
 #include <windows.h>
 #include <commctrl.h>
 #include <stdio.h>
@@ -954,7 +953,7 @@ static void test_ignore_selection(HWND hWndTrackbar){
 static void test_initial_state(void)
 {
     HWND hWnd;
-    DWORD ret;
+    int ret;
 
     hWnd = create_trackbar(0, hWndParent);
 
