@@ -639,6 +639,15 @@ static inline int is_vertex_shader(DWORD version)
     return (version & 0xFFFF0000) == 0xFFFE0000;
 }
 
+static DWORD calc_bytes(D3DXCONSTANT_DESC *desc)
+{
+    if (desc->RegisterSet != D3DXRS_FLOAT4 && desc->RegisterSet != D3DXRS_SAMPLER)
+        FIXME("Don't know how to calculate Bytes for constants of type %d\n",
+                desc->RegisterSet);
+
+    return 4 * desc->Elements * desc->Rows * desc->Columns;
+}
+
 /*** IUnknown methods ***/
 static HRESULT WINAPI ID3DXConstantTableImpl_QueryInterface(ID3DXConstantTable* iface, REFIID riid, void** ppvObject)
 {
@@ -1130,6 +1139,7 @@ static HRESULT parse_ctab_constant_type(const D3DXSHADER_TYPEINFO *type, ctab_co
     constant->desc.Columns = type->Columns;
     constant->desc.Elements = type->Elements;
     constant->desc.StructMembers = type->StructMembers;
+    constant->desc.Bytes = calc_bytes(&constant->desc);
 
     TRACE("class = %d, type = %d, rows = %d, columns = %d, elements = %d, struct_members = %d\n",
           constant->desc.Class, constant->desc.Type, constant->desc.Elements,
@@ -1165,7 +1175,7 @@ HRESULT WINAPI D3DXGetShaderConstantTableEx(CONST DWORD* byte_code,
     if (hr != D3D_OK)
         return D3DXERR_INVALIDDATA;
 
-    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(ID3DXConstantTableImpl));
+    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
     if (!object)
     {
         ERR("Out of memory\n");
@@ -1237,22 +1247,6 @@ HRESULT WINAPI D3DXGetShaderConstantTableEx(CONST DWORD* byte_code,
              &object->constants[i]);
         if (hr != D3D_OK)
             goto error;
-
-        if (constant_info[i].RegisterSet != D3DXRS_FLOAT4 &&
-                constant_info[i].RegisterSet != D3DXRS_SAMPLER)
-            FIXME("Don't know how to calculate Bytes for constants of type %d\n",
-                    constant_info[i].RegisterSet);
-
-        /*
-         * D3DXRS_FLOAT4 and D3DXRS_SAMPLER have a base size of 4
-         * (not taking into account dimensions and element count)
-         */
-        object->constants[i].desc.Bytes = 4;
-
-        /* Take into account dimensions and elements */
-        object->constants[i].desc.Bytes *= object->constants[i].desc.Elements;
-        object->constants[i].desc.Bytes *= object->constants[i].desc.Rows *
-                object->constants[i].desc.Columns;
     }
 
     *constant_table = &object->ID3DXConstantTable_iface;
@@ -1275,3 +1269,52 @@ HRESULT WINAPI D3DXGetShaderConstantTable(CONST DWORD* byte_code,
 
     return D3DXGetShaderConstantTableEx(byte_code, 0, constant_table);
 }
+
+HRESULT WINAPI D3DXGetShaderSamplers(CONST DWORD *byte_code, LPCSTR *samplers, UINT *count)
+{
+    HRESULT hr;
+    UINT i, sampler_count = 0;
+    UINT size;
+    LPCSTR data;
+    const D3DXSHADER_CONSTANTTABLE *ctab_header;
+    const D3DXSHADER_CONSTANTINFO *constant_info;
+
+    TRACE("byte_code %p, samplers %p, count %p\n", byte_code, samplers, count);
+
+    if (count) *count = 0;
+
+    hr = D3DXFindShaderComment(byte_code, MAKEFOURCC('C','T','A','B'), (LPCVOID *)&data, &size);
+    if (hr != D3D_OK) return D3D_OK;
+
+    if (size < sizeof(D3DXSHADER_CONSTANTTABLE)) return D3D_OK;
+
+    ctab_header = (const D3DXSHADER_CONSTANTTABLE *)data;
+    if (ctab_header->Size != sizeof(*ctab_header)) return D3D_OK;
+
+    constant_info = (D3DXSHADER_CONSTANTINFO *)(data + ctab_header->ConstantInfo);
+    for (i = 0; i < ctab_header->Constants; i++)
+    {
+        const D3DXSHADER_TYPEINFO *type;
+
+        TRACE("name = %s\n", data + constant_info[i].Name);
+
+        type = (D3DXSHADER_TYPEINFO *)(data + constant_info[i].TypeInfo);
+
+        if (type->Type == D3DXPT_SAMPLER
+                || type->Type == D3DXPT_SAMPLER1D
+                || type->Type == D3DXPT_SAMPLER2D
+                || type->Type == D3DXPT_SAMPLER3D
+                || type->Type == D3DXPT_SAMPLERCUBE)
+        {
+            if (samplers) samplers[sampler_count] = data + constant_info[i].Name;
+
+            ++sampler_count;
+        }
+    }
+
+    TRACE("Found %u samplers\n", sampler_count);
+
+    if (count) *count = sampler_count;
+
+    return D3D_OK;
+}
