@@ -275,6 +275,14 @@ DirectDrawCreate(GUID *GUID,
     EnterCriticalSection(&ddraw_cs);
     hr = DDRAW_Create(GUID, (void **) DD, UnkOuter, &IID_IDirectDraw);
     LeaveCriticalSection(&ddraw_cs);
+
+    if (SUCCEEDED(hr))
+    {
+        hr = IDirectDraw_Initialize(*DD, GUID);
+        if (FAILED(hr))
+            IDirectDraw_Release(*DD);
+    }
+
     return hr;
 }
 
@@ -304,6 +312,15 @@ DirectDrawCreateEx(GUID *GUID,
     EnterCriticalSection(&ddraw_cs);
     hr = DDRAW_Create(GUID, DD, UnkOuter, iid);
     LeaveCriticalSection(&ddraw_cs);
+
+    if (SUCCEEDED(hr))
+    {
+        IDirectDraw7 *ddraw7 = *(IDirectDraw7 **)DD;
+        hr = IDirectDraw7_Initialize(ddraw7, GUID);
+        if (FAILED(hr))
+            IDirectDraw7_Release(ddraw7);
+    }
+
     return hr;
 }
 
@@ -933,7 +950,6 @@ DllMain(HINSTANCE hInstDLL,
                 /* Add references to each interface to avoid freeing them unexpectedly */
                 IDirectDraw_AddRef(&ddraw->IDirectDraw_iface);
                 IDirectDraw2_AddRef(&ddraw->IDirectDraw2_iface);
-                IDirectDraw3_AddRef(&ddraw->IDirectDraw3_iface);
                 IDirectDraw4_AddRef(&ddraw->IDirectDraw4_iface);
                 IDirectDraw7_AddRef(&ddraw->IDirectDraw7_iface);
 
@@ -960,16 +976,14 @@ DllMain(HINSTANCE hInstDLL,
                         ERR("(%p) EnumSurfaces failed, prepare for trouble\n", ddraw);
                 }
 
-                /* Check the surface count */
-                if(ddraw->surfaces > 0)
-                    ERR("DDraw %p still has %d surfaces attached\n", ddraw, ddraw->surfaces);
+                if (!list_empty(&ddraw->surface_list))
+                    ERR("DDraw %p still has surfaces attached.\n", ddraw);
 
                 /* Release all hanging references to destroy the objects. This
                     * restores the screen mode too
                     */
                 while(IDirectDraw_Release(&ddraw->IDirectDraw_iface));
                 while(IDirectDraw2_Release(&ddraw->IDirectDraw2_iface));
-                while(IDirectDraw3_Release(&ddraw->IDirectDraw3_iface));
                 while(IDirectDraw4_Release(&ddraw->IDirectDraw4_iface));
                 while(IDirectDraw7_Release(&ddraw->IDirectDraw7_iface));
             }
