@@ -27,6 +27,7 @@
 
 #include <stdarg.h>
 
+#define COBJMACROS
 #define NONAMELESSSTRUCT
 #define NONAMELESSUNION
 #include "windef.h"
@@ -38,7 +39,6 @@
 #include "winnls.h"
 #include "wine/debug.h"
 #include "dsound.h"
-#include "dsdriver.h"
 #include "dsound_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(dsound);
@@ -89,8 +89,6 @@ static ULONG WINAPI IDirectSoundCaptureNotifyImpl_Release(LPDIRECTSOUNDNOTIFY if
     TRACE("(%p) ref was %d\n", This, ref + 1);
 
     if (!ref) {
-        if (This->dscb->hwnotify)
-            IDsDriverNotify_Release(This->dscb->hwnotify);
 	This->dscb->notify=NULL;
 	IDirectSoundCaptureBuffer_Release((LPDIRECTSOUNDCAPTUREBUFFER)This->dscb);
 	HeapFree(GetProcessHeap(),0,This);
@@ -119,13 +117,7 @@ static HRESULT WINAPI IDirectSoundCaptureNotifyImpl_SetNotificationPositions(
 	    notify[i].dwOffset,notify[i].hEventNotify);
     }
 
-    if (This->dscb->hwnotify) {
-	HRESULT hres;
-	hres = IDsDriverNotify_SetNotificationPositions(This->dscb->hwnotify, howmuch, notify);
-	if (hres != DS_OK)
-	    WARN("IDsDriverNotify_SetNotificationPositions failed\n");
-	return hres;
-    } else if (howmuch > 0) {
+    if (howmuch > 0) {
 	/* Make an internal copy of the caller-supplied array.
 	 * Replace the existing copy if one is already present. */
 	if (This->dscb->notifies)
@@ -202,6 +194,7 @@ IDirectSoundCaptureBufferImpl_QueryInterface(
 {
     IDirectSoundCaptureBufferImpl *This = (IDirectSoundCaptureBufferImpl *)iface;
     HRESULT hres;
+
     TRACE( "(%p,%s,%p)\n", This, debugstr_guid(riid), ppobj );
 
     if (ppobj == NULL) {
@@ -212,27 +205,15 @@ IDirectSoundCaptureBufferImpl_QueryInterface(
     *ppobj = NULL;
 
     if ( IsEqualGUID( &IID_IDirectSoundNotify, riid ) ) {
-	if (!This->notify)
+	if (!This->notify){
 	    hres = IDirectSoundCaptureNotifyImpl_Create(This, &This->notify);
-	if (This->notify) {
-	    IDirectSoundNotify_AddRef((LPDIRECTSOUNDNOTIFY)This->notify);
-	    if (This->device->hwbuf && !This->hwnotify) {
-		hres = IDsCaptureDriverBuffer_QueryInterface(This->device->hwbuf,
-		    &IID_IDsDriverNotify, (LPVOID*)&(This->hwnotify));
-		if (hres != DS_OK) {
-		    WARN("IDsCaptureDriverBuffer_QueryInterface failed\n");
-		    IDirectSoundNotify_Release((LPDIRECTSOUNDNOTIFY)This->notify);
-		    *ppobj = 0;
-		    return hres;
-	        }
-	    }
-
-            *ppobj = This->notify;
-	    return DS_OK;
+	    if(FAILED(hres))
+		return hres;
 	}
 
-	WARN("IID_IDirectSoundNotify\n");
-	return E_FAIL;
+        IDirectSoundNotify_AddRef((LPDIRECTSOUNDNOTIFY)This->notify);
+        *ppobj = This->notify;
+        return DS_OK;
     }
 
     if ( IsEqualGUID( &IID_IDirectSoundCaptureBuffer, riid ) ||
@@ -264,37 +245,30 @@ IDirectSoundCaptureBufferImpl_Release( LPDIRECTSOUNDCAPTUREBUFFER8 iface )
 
     if (!ref) {
         TRACE("deleting object\n");
-	if (This->device->state == STATE_CAPTURING)
-	    This->device->state = STATE_STOPPING;
+        if (This->device->state == STATE_CAPTURING)
+            This->device->state = STATE_STOPPING;
 
         HeapFree(GetProcessHeap(),0, This->pdscbd);
 
-	if (This->device->hwi) {
-	    waveInReset(This->device->hwi);
-	    waveInClose(This->device->hwi);
-            HeapFree(GetProcessHeap(),0, This->device->pwave);
-            This->device->pwave = 0;
-	    This->device->hwi = 0;
-	}
+        if (This->device->client) {
+            IAudioClient_Release(This->device->client);
+            This->device->client = NULL;
+        }
 
-	if (This->device->hwbuf)
-	    IDsCaptureDriverBuffer_Release(This->device->hwbuf);
+        if (This->device->capture) {
+            IAudioCaptureClient_Release(This->device->capture);
+            This->device->capture = NULL;
+        }
 
         /* remove from DirectSoundCaptureDevice */
         This->device->capture_buffer = NULL;
 
         if (This->notify)
-	    IDirectSoundNotify_Release((LPDIRECTSOUNDNOTIFY)This->notify);
+            IDirectSoundNotify_Release((LPDIRECTSOUNDNOTIFY)This->notify);
 
-        /* If driver manages its own buffer, IDsCaptureDriverBuffer_Release
-           should have freed the buffer. Prevent freeing it again in
-           IDirectSoundCaptureBufferImpl_Create */
-        if (!(This->device->drvdesc.dwFlags & DSDDESC_USESYSTEMMEMORY))
-	    This->device->buffer = NULL;
-
-	HeapFree(GetProcessHeap(), 0, This->notifies);
+        HeapFree(GetProcessHeap(), 0, This->notifies);
         HeapFree( GetProcessHeap(), 0, This );
-	TRACE("(%p) released\n", This);
+        TRACE("(%p) released\n", This);
     }
     return ref;
 }
@@ -338,7 +312,6 @@ IDirectSoundCaptureBufferImpl_GetCurrentPosition(
     LPDWORD lpdwReadPosition )
 {
     IDirectSoundCaptureBufferImpl *This = (IDirectSoundCaptureBufferImpl *)iface;
-    HRESULT hres = DS_OK;
     TRACE( "(%p,%p,%p)\n", This, lpdwCapturePosition, lpdwReadPosition );
 
     if (This->device == NULL) {
@@ -346,29 +319,26 @@ IDirectSoundCaptureBufferImpl_GetCurrentPosition(
         return DSERR_INVALIDPARAM;
     }
 
-    if (This->device->driver) {
-        hres = IDsCaptureDriverBuffer_GetPosition(This->device->hwbuf, lpdwCapturePosition, lpdwReadPosition );
-	if (hres != DS_OK)
-	    WARN("IDsCaptureDriverBuffer_GetPosition failed\n");
-    } else if (This->device->hwi) {
-        DWORD pos;
+    EnterCriticalSection(&This->device->lock);
 
-        EnterCriticalSection(&This->device->lock);
-        pos = (DWORD_PTR)This->device->pwave[This->device->index].lpData - (DWORD_PTR)This->device->buffer;
-        if (lpdwCapturePosition)
-            *lpdwCapturePosition = (This->device->pwave[This->device->index].dwBufferLength + pos) % This->device->buflen;
-        if (lpdwReadPosition)
-            *lpdwReadPosition = pos;
+    if (!This->device->client) {
         LeaveCriticalSection(&This->device->lock);
-
-    } else {
         WARN("no driver\n");
-        hres = DSERR_NODRIVER;
+        return DSERR_NODRIVER;
     }
 
+    if(lpdwCapturePosition)
+        *lpdwCapturePosition = This->device->write_pos_bytes;
+
+    if(lpdwReadPosition)
+        *lpdwReadPosition = This->device->write_pos_bytes;
+
+    LeaveCriticalSection(&This->device->lock);
+
     TRACE("cappos=%d readpos=%d\n", (lpdwCapturePosition?*lpdwCapturePosition:-1), (lpdwReadPosition?*lpdwReadPosition:-1));
-    TRACE("returning %08x\n", hres);
-    return hres;
+    TRACE("returning DS_OK\n");
+
+    return DS_OK;
 }
 
 static HRESULT WINAPI
@@ -493,14 +463,7 @@ IDirectSoundCaptureBufferImpl_Lock(
 
     EnterCriticalSection(&(This->device->lock));
 
-    if (This->device->driver) {
-        hres = IDsCaptureDriverBuffer_Lock(This->device->hwbuf, lplpvAudioPtr1,
-                                           lpdwAudioBytes1, lplpvAudioPtr2,
-                                           lpdwAudioBytes2, dwReadCusor,
-                                           dwReadBytes, dwFlags);
-	if (hres != DS_OK)
-	    WARN("IDsCaptureDriverBuffer_Lock failed\n");
-    } else if (This->device->hwi) {
+    if (This->device->client) {
         *lplpvAudioPtr1 = This->device->buffer + dwReadCusor;
         if ( (dwReadCusor + dwReadBytes) > This->device->buflen) {
             *lpdwAudioBytes1 = This->device->buflen - dwReadCusor;
@@ -531,8 +494,9 @@ IDirectSoundCaptureBufferImpl_Start(
     LPDIRECTSOUNDCAPTUREBUFFER8 iface,
     DWORD dwFlags )
 {
-    HRESULT hres = DS_OK;
+    HRESULT hres;
     IDirectSoundCaptureBufferImpl *This = (IDirectSoundCaptureBufferImpl *)iface;
+
     TRACE( "(%p,0x%08x)\n", This, dwFlags );
 
     if (This->device == NULL) {
@@ -540,7 +504,7 @@ IDirectSoundCaptureBufferImpl_Start(
         return DSERR_INVALIDPARAM;
     }
 
-    if ( (This->device->driver == 0) && (This->device->hwi == 0) ) {
+    if ( !This->device->client ) {
         WARN("no driver\n");
         return DSERR_NODRIVER;
     }
@@ -548,91 +512,33 @@ IDirectSoundCaptureBufferImpl_Start(
     EnterCriticalSection(&(This->device->lock));
 
     This->flags = dwFlags;
-    TRACE("old This->state=%s\n",captureStateString[This->device->state]);
+    TRACE("old This->device->state=%s\n",captureStateString[This->device->state]);
     if (This->device->state == STATE_STOPPED)
         This->device->state = STATE_STARTING;
     else if (This->device->state == STATE_STOPPING)
         This->device->state = STATE_CAPTURING;
     TRACE("new This->device->state=%s\n",captureStateString[This->device->state]);
 
-    LeaveCriticalSection(&(This->device->lock));
+    if (This->device->buffer)
+        FillMemory(This->device->buffer, This->device->buflen, (This->device->pwfx->wBitsPerSample == 8) ? 128 : 0);
 
-    if (This->device->driver) {
-        hres = IDsCaptureDriverBuffer_Start(This->device->hwbuf, dwFlags);
-	if (hres != DS_OK)
-	    WARN("IDsCaptureDriverBuffer_Start failed\n");
-    } else if (This->device->hwi) {
-        DirectSoundCaptureDevice *device = This->device;
-
-        if (device->buffer) {
-            int c;
-            DWORD blocksize = DSOUND_fraglen(device->pwfx->nSamplesPerSec, device->pwfx->nBlockAlign);
-            device->nrofpwaves = device->buflen / blocksize + !!(device->buflen % blocksize);
-            TRACE("nrofpwaves=%d\n", device->nrofpwaves);
-
-            /* prepare headers */
-            if (device->pwave)
-                device->pwave = HeapReAlloc(GetProcessHeap(), 0,device->pwave, device->nrofpwaves*sizeof(WAVEHDR));
-            else
-                device->pwave = HeapAlloc(GetProcessHeap(), 0, device->nrofpwaves*sizeof(WAVEHDR));
-
-            for (c = 0; c < device->nrofpwaves; ++c) {
-                device->pwave[c].lpData = (char *)device->buffer + c * blocksize;
-                if (c + 1 == device->nrofpwaves)
-                    device->pwave[c].dwBufferLength = device->buflen - c * blocksize;
-                else
-                    device->pwave[c].dwBufferLength = blocksize;
-                device->pwave[c].dwBytesRecorded = 0;
-                device->pwave[c].dwUser = (DWORD_PTR)device;
-                device->pwave[c].dwFlags = 0;
-                device->pwave[c].dwLoops = 0;
-                hres = mmErr(waveInPrepareHeader(device->hwi, &(device->pwave[c]),sizeof(WAVEHDR)));
-                if (hres != DS_OK) {
-                    WARN("waveInPrepareHeader failed\n");
-                    while (c--)
-                        waveInUnprepareHeader(device->hwi, &(device->pwave[c]),sizeof(WAVEHDR));
-                    break;
-                }
-
-                hres = mmErr(waveInAddBuffer(device->hwi, &(device->pwave[c]), sizeof(WAVEHDR)));
-                if (hres != DS_OK) {
-                    WARN("waveInAddBuffer failed\n");
-                    while (c--)
-                        waveInUnprepareHeader(device->hwi, &(device->pwave[c]),sizeof(WAVEHDR));
-                    break;
-                }
-            }
-
-            FillMemory(device->buffer, device->buflen, (device->pwfx->wBitsPerSample == 8) ? 128 : 0);
-        }
-
-        device->index = 0;
-
-	if (hres == DS_OK) {
-	    /* start filling the first buffer */
-	    hres = mmErr(waveInStart(device->hwi));
-            if (hres != DS_OK)
-                WARN("waveInStart failed\n");
-        }
-
-        if (hres != DS_OK) {
-            WARN("calling waveInClose because of error\n");
-            waveInClose(device->hwi);
-            device->hwi = 0;
-        }
-    } else {
-        WARN("no driver\n");
-        hres = DSERR_NODRIVER;
+    hres = IAudioClient_Start(This->device->client);
+    if(FAILED(hres)){
+        WARN("Start failed: %08x\n", hres);
+        LeaveCriticalSection(&This->device->lock);
+        return hres;
     }
 
-    TRACE("returning %08x\n", hres);
-    return hres;
+    LeaveCriticalSection(&This->device->lock);
+
+    TRACE("returning DS_OK\n");
+    return DS_OK;
 }
 
 static HRESULT WINAPI
 IDirectSoundCaptureBufferImpl_Stop( LPDIRECTSOUNDCAPTUREBUFFER8 iface )
 {
-    HRESULT hres = DS_OK;
+    HRESULT hres;
     IDirectSoundCaptureBufferImpl *This = (IDirectSoundCaptureBufferImpl *)iface;
     TRACE( "(%p)\n", This );
 
@@ -650,23 +556,18 @@ IDirectSoundCaptureBufferImpl_Stop( LPDIRECTSOUNDCAPTUREBUFFER8 iface )
 	This->device->state = STATE_STOPPED;
     TRACE("new This->device->state=%s\n",captureStateString[This->device->state]);
 
-    LeaveCriticalSection(&(This->device->lock));
-
-    if (This->device->driver) {
-        hres = IDsCaptureDriverBuffer_Stop(This->device->hwbuf);
-        if (hres != DS_OK)
-            WARN("IDsCaptureDriverBuffer_Stop() failed\n");
-    } else if (This->device->hwi) {
-        hres = mmErr(waveInReset(This->device->hwi));
-        if (hres != DS_OK)
-            WARN("waveInReset() failed\n");
-    } else {
-	WARN("no driver\n");
-        hres = DSERR_NODRIVER;
+    if(This->device->client){
+        hres = IAudioClient_Stop(This->device->client);
+        if(FAILED(hres)){
+            LeaveCriticalSection(&This->device->lock);
+            return hres;
+        }
     }
 
-    TRACE("returning %08x\n", hres);
-    return hres;
+    LeaveCriticalSection(&(This->device->lock));
+
+    TRACE("returning DS_OK\n");
+    return DS_OK;
 }
 
 static HRESULT WINAPI
@@ -687,12 +588,7 @@ IDirectSoundCaptureBufferImpl_Unlock(
         return DSERR_INVALIDPARAM;
     }
 
-    if (This->device->driver) {
-        hres = IDsCaptureDriverBuffer_Unlock(This->device->hwbuf, lpvAudioPtr1,
-                                             dwAudioBytes1, lpvAudioPtr2, dwAudioBytes2);
-	if (hres != DS_OK)
-	    WARN("IDsCaptureDriverBuffer_Unlock failed\n");
-    } else if (!This->device->hwi) {
+    if (!This->device->client) {
         WARN("invalid call\n");
         hres = DSERR_INVALIDCALL;
     }
@@ -778,49 +674,6 @@ static void capture_CheckNotify(IDirectSoundCaptureBufferImpl *This, DWORD from,
     }
 }
 
-static void CALLBACK
-DSOUND_capture_callback(HWAVEIN hwi, UINT msg, DWORD_PTR dwUser, DWORD_PTR dw1,
-                        DWORD_PTR dw2)
-{
-    DirectSoundCaptureDevice * This = (DirectSoundCaptureDevice*)dwUser;
-    IDirectSoundCaptureBufferImpl * Moi = This->capture_buffer;
-    TRACE("(%p,%08x(%s),%08lx,%08lx,%08lx) entering at %d\n",hwi,msg,
-	msg == MM_WIM_OPEN ? "MM_WIM_OPEN" : msg == MM_WIM_CLOSE ? "MM_WIM_CLOSE" :
-	msg == MM_WIM_DATA ? "MM_WIM_DATA" : "UNKNOWN",dwUser,dw1,dw2,GetTickCount());
-
-    if (msg == MM_WIM_DATA) {
-	EnterCriticalSection( &(This->lock) );
-	TRACE("DirectSoundCapture msg=MM_WIM_DATA, old This->state=%s, old This->index=%d\n",
-	    captureStateString[This->state],This->index);
-	if (This->state != STATE_STOPPED) {
-	    int index = This->index;
-	    if (This->state == STATE_STARTING)
-		This->state = STATE_CAPTURING;
-	    capture_CheckNotify(Moi, (DWORD_PTR)This->pwave[index].lpData - (DWORD_PTR)This->buffer, This->pwave[index].dwBufferLength);
-	    This->index = (This->index + 1) % This->nrofpwaves;
-	    if ( (This->index == 0) && !(This->capture_buffer->flags & DSCBSTART_LOOPING) ) {
-		TRACE("end of buffer\n");
-		This->state = STATE_STOPPED;
-		capture_CheckNotify(Moi, 0, 0);
-	    } else {
-		if (This->state == STATE_CAPTURING) {
-		    waveInUnprepareHeader(hwi, &(This->pwave[index]), sizeof(WAVEHDR));
-		    waveInPrepareHeader(hwi, &(This->pwave[index]), sizeof(WAVEHDR));
-		    waveInAddBuffer(hwi, &(This->pwave[index]), sizeof(WAVEHDR));
-	        } else if (This->state == STATE_STOPPING) {
-		    TRACE("stopping\n");
-		    This->state = STATE_STOPPED;
-		}
-	    }
-	}
-	TRACE("DirectSoundCapture new This->state=%s, new This->index=%d\n",
-	    captureStateString[This->state],This->index);
-	LeaveCriticalSection( &(This->lock) );
-    }
-
-    TRACE("completed\n");
-}
-
 static HRESULT IDirectSoundCaptureBufferImpl_Create(
     DirectSoundCaptureDevice *device,
     IDirectSoundCaptureBufferImpl ** ppobj,
@@ -877,7 +730,7 @@ static HRESULT IDirectSoundCaptureBufferImpl_Create(
 	*ppobj = NULL;
 	return DSERR_OUTOFMEMORY;
     } else {
-    	HRESULT err = DS_OK;
+        HRESULT err = DS_OK;
         LPBYTE newbuf;
         DWORD buflen;
         IDirectSoundCaptureBufferImpl *This = *ppobj;
@@ -885,9 +738,8 @@ static HRESULT IDirectSoundCaptureBufferImpl_Create(
         This->ref = 1;
         This->device = device;
         This->device->capture_buffer = This;
-	This->notify = NULL;
-	This->nrofnotifies = 0;
-	This->hwnotify = NULL;
+        This->notify = NULL;
+        This->nrofnotifies = 0;
 
         This->pdscbd = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,
             lpcDSCBufferDesc->dwSize);
@@ -903,72 +755,63 @@ static HRESULT IDirectSoundCaptureBufferImpl_Create(
 
         This->lpVtbl = &dscbvt;
 
-	if (device->driver) {
-            if (This->device->drvdesc.dwFlags & DSDDESC_DOMMSYSTEMOPEN)
-                FIXME("DSDDESC_DOMMSYSTEMOPEN not supported\n");
+        err = IMMDevice_Activate(device->mmdevice, &IID_IAudioClient,
+                CLSCTX_INPROC_SERVER, NULL, (void**)&device->client);
+        if(FAILED(err)){
+            WARN("Activate failed: %08x\n", err);
+            HeapFree(GetProcessHeap(), 0, This->pdscbd);
+            This->device->capture_buffer = 0;
+            HeapFree( GetProcessHeap(), 0, This );
+            *ppobj = NULL;
+            return err;
+        }
 
-            if (This->device->drvdesc.dwFlags & DSDDESC_USESYSTEMMEMORY) {
-                /* allocate buffer from system memory */
-                buflen = lpcDSCBufferDesc->dwBufferBytes;
-                TRACE("desired buflen=%d, old buffer=%p\n", buflen, device->buffer);
-                if (device->buffer)
-                    newbuf = HeapReAlloc(GetProcessHeap(),0,device->buffer,buflen);
-                else
-                    newbuf = HeapAlloc(GetProcessHeap(),0,buflen);
+        err = IAudioClient_Initialize(device->client,
+                AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_NOPERSIST,
+                200 * 100000, 50000, device->pwfx, NULL);
+        if(FAILED(err)){
+            WARN("Initialize failed: %08x\n", err);
+            IAudioClient_Release(device->client);
+            device->client = NULL;
+            HeapFree(GetProcessHeap(), 0, This->pdscbd);
+            This->device->capture_buffer = 0;
+            HeapFree( GetProcessHeap(), 0, This );
+            *ppobj = NULL;
+            return err;
+        }
 
-                if (newbuf == NULL) {
-                    WARN("failed to allocate capture buffer\n");
-                    err = DSERR_OUTOFMEMORY;
-                    /* but the old buffer might still exist and must be re-prepared */
-                } else {
-                    device->buffer = newbuf;
-                    device->buflen = buflen;
-                }
-            } else {
-                /* let driver allocate memory */
-                device->buflen = lpcDSCBufferDesc->dwBufferBytes;
-                /* FIXME: */
-                HeapFree( GetProcessHeap(), 0, device->buffer);
-                device->buffer = NULL;
-            }
+        err = IAudioClient_GetService(device->client, &IID_IAudioCaptureClient,
+                (void**)&device->capture);
+        if(FAILED(err)){
+            WARN("GetService failed: %08x\n", err);
+            IAudioClient_Release(device->client);
+            device->client = NULL;
+            HeapFree(GetProcessHeap(), 0, This->pdscbd);
+            This->device->capture_buffer = 0;
+            HeapFree( GetProcessHeap(), 0, This );
+            *ppobj = NULL;
+            return err;
+        }
 
-	    err = IDsCaptureDriver_CreateCaptureBuffer(device->driver,
-		device->pwfx,0,0,&(device->buflen),&(device->buffer),(LPVOID*)&(device->hwbuf));
-	    if (err != DS_OK) {
-		WARN("IDsCaptureDriver_CreateCaptureBuffer failed\n");
-		This->device->capture_buffer = 0;
-		HeapFree( GetProcessHeap(), 0, This );
-		*ppobj = NULL;
-		return err;
-	    }
-	} else {
-	    DWORD flags = CALLBACK_FUNCTION;
-            err = mmErr(waveInOpen(&(device->hwi),
-                device->drvdesc.dnDevNode, device->pwfx,
-                (DWORD_PTR)DSOUND_capture_callback, (DWORD_PTR)device, flags));
-            if (err != DS_OK) {
-                WARN("waveInOpen failed\n");
-		This->device->capture_buffer = 0;
-		HeapFree( GetProcessHeap(), 0, This );
-		*ppobj = NULL;
-		return err;
-            }
-
-	    buflen = lpcDSCBufferDesc->dwBufferBytes;
-            TRACE("desired buflen=%d, old buffer=%p\n", buflen, device->buffer);
-	    if (device->buffer)
-                newbuf = HeapReAlloc(GetProcessHeap(),0,device->buffer,buflen);
-	    else
-		newbuf = HeapAlloc(GetProcessHeap(),0,buflen);
-            if (newbuf == NULL) {
-                WARN("failed to allocate capture buffer\n");
-                err = DSERR_OUTOFMEMORY;
-                /* but the old buffer might still exist and must be re-prepared */
-            } else {
-                device->buffer = newbuf;
-                device->buflen = buflen;
-            }
-	}
+        buflen = lpcDSCBufferDesc->dwBufferBytes;
+        TRACE("desired buflen=%d, old buffer=%p\n", buflen, device->buffer);
+        if (device->buffer)
+            newbuf = HeapReAlloc(GetProcessHeap(),0,device->buffer,buflen);
+        else
+            newbuf = HeapAlloc(GetProcessHeap(),0,buflen);
+        if (newbuf == NULL) {
+            IAudioClient_Release(device->client);
+            device->client = NULL;
+            IAudioCaptureClient_Release(device->capture);
+            device->capture = NULL;
+            HeapFree(GetProcessHeap(), 0, This->pdscbd);
+            This->device->capture_buffer = 0;
+            HeapFree( GetProcessHeap(), 0, This );
+            *ppobj = NULL;
+            return DSERR_OUTOFMEMORY;
+        }
+        device->buffer = newbuf;
+        device->buflen = buflen;
     }
 
     TRACE("returning DS_OK\n");
@@ -979,8 +822,6 @@ static HRESULT IDirectSoundCaptureBufferImpl_Create(
 /*******************************************************************************
  * DirectSoundCaptureDevice
  */
-DirectSoundCaptureDevice * DSOUND_capture[MAXWAVEDRIVERS];
-
 static HRESULT DirectSoundCaptureDevice_Create(
     DirectSoundCaptureDevice ** ppDevice)
 {
@@ -1014,136 +855,208 @@ static ULONG DirectSoundCaptureDevice_Release(
 
     if (!ref) {
         TRACE("deleting object\n");
+
+        timeKillEvent(device->timerID);
+        timeEndPeriod(DS_TIME_RES);
+
+        EnterCriticalSection(&DSOUND_capturers_lock);
+        list_remove(&device->entry);
+        LeaveCriticalSection(&DSOUND_capturers_lock);
+
         if (device->capture_buffer)
             IDirectSoundCaptureBufferImpl_Release(
 		(LPDIRECTSOUNDCAPTUREBUFFER8) device->capture_buffer);
 
-        if (device->driver) {
-            IDsCaptureDriver_Close(device->driver);
-            IDsCaptureDriver_Release(device->driver);
-        }
-
+        if(device->mmdevice)
+            IMMDevice_Release(device->mmdevice);
         HeapFree(GetProcessHeap(), 0, device->pwfx);
         device->lock.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection( &(device->lock) );
-        DSOUND_capture[device->drvdesc.dnDevNode] = NULL;
         HeapFree(GetProcessHeap(), 0, device);
 	TRACE("(%p) released\n", device);
     }
     return ref;
 }
 
+static void CALLBACK DSOUND_capture_timer(UINT timerID, UINT msg, DWORD_PTR user,
+                                          DWORD_PTR dw1, DWORD_PTR dw2)
+{
+    DirectSoundCaptureDevice *device = (DirectSoundCaptureDevice*)user;
+    UINT32 packet_frames, packet_bytes, avail_bytes;
+    DWORD flags;
+    BYTE *buf;
+    HRESULT hr;
+
+    if(!device->ref)
+        return;
+
+    EnterCriticalSection(&device->lock);
+
+    if(!device->capture_buffer || device->state == STATE_STOPPED){
+        LeaveCriticalSection(&device->lock);
+        return;
+    }
+
+    if(device->state == STATE_STOPPING){
+        device->state = STATE_STOPPED;
+        LeaveCriticalSection(&device->lock);
+        return;
+    }
+
+    if(device->state == STATE_STARTING)
+        device->state = STATE_CAPTURING;
+
+    hr = IAudioCaptureClient_GetBuffer(device->capture, &buf, &packet_frames,
+            &flags, NULL, NULL);
+    if(FAILED(hr)){
+        LeaveCriticalSection(&device->lock);
+        WARN("GetBuffer failed: %08x\n", hr);
+        return;
+    }
+
+    packet_bytes = packet_frames * device->pwfx->nBlockAlign;
+
+    avail_bytes = device->buflen - device->write_pos_bytes;
+    if(avail_bytes > packet_bytes)
+        avail_bytes = packet_bytes;
+
+    memcpy(device->buffer + device->write_pos_bytes, buf, avail_bytes);
+    capture_CheckNotify(device->capture_buffer, device->write_pos_bytes, avail_bytes);
+
+    packet_bytes -= avail_bytes;
+    if(packet_bytes > 0){
+        if(device->capture_buffer->flags & DSCBSTART_LOOPING){
+            memcpy(device->buffer, buf + avail_bytes, packet_bytes);
+            capture_CheckNotify(device->capture_buffer, 0, packet_bytes);
+        }else{
+            device->state = STATE_STOPPED;
+            capture_CheckNotify(device->capture_buffer, 0, 0);
+        }
+    }
+
+    device->write_pos_bytes += avail_bytes + packet_bytes;
+    device->write_pos_bytes %= device->buflen;
+
+    hr = IAudioCaptureClient_ReleaseBuffer(device->capture, packet_frames);
+    if(FAILED(hr)){
+        LeaveCriticalSection(&device->lock);
+        WARN("ReleaseBuffer failed: %08x\n", hr);
+        return;
+    }
+
+    LeaveCriticalSection(&device->lock);
+}
+
+static struct _TestFormat {
+    DWORD flag;
+    DWORD rate;
+    DWORD depth;
+    WORD channels;
+} formats_to_test[] = {
+    { WAVE_FORMAT_1M08, 11025, 8, 1 },
+    { WAVE_FORMAT_1M16, 11025, 16, 1 },
+    { WAVE_FORMAT_1S08, 11025, 8, 2 },
+    { WAVE_FORMAT_1S16, 11025, 16, 2 },
+    { WAVE_FORMAT_2M08, 22050, 8, 1 },
+    { WAVE_FORMAT_2M16, 22050, 16, 1 },
+    { WAVE_FORMAT_2S08, 22050, 8, 2 },
+    { WAVE_FORMAT_2S16, 22050, 16, 2 },
+    { WAVE_FORMAT_4M08, 44100, 8, 1 },
+    { WAVE_FORMAT_4M16, 44100, 16, 1 },
+    { WAVE_FORMAT_4S08, 44100, 8, 2 },
+    { WAVE_FORMAT_4S16, 44100, 16, 2 },
+    { WAVE_FORMAT_48M08, 48000, 8, 1 },
+    { WAVE_FORMAT_48M16, 48000, 16, 1 },
+    { WAVE_FORMAT_48S08, 48000, 8, 2 },
+    { WAVE_FORMAT_48S16, 48000, 16, 2 },
+    { WAVE_FORMAT_96M08, 96000, 8, 1 },
+    { WAVE_FORMAT_96M16, 96000, 16, 1 },
+    { WAVE_FORMAT_96S08, 96000, 8, 2 },
+    { WAVE_FORMAT_96S16, 96000, 16, 2 },
+    {0}
+};
+
 static HRESULT DirectSoundCaptureDevice_Initialize(
     DirectSoundCaptureDevice ** ppDevice,
     LPCGUID lpcGUID)
 {
-    HRESULT err = DSERR_INVALIDPARAM;
-    unsigned wid, widn;
-    BOOLEAN found = FALSE;
+    HRESULT hr;
     GUID devGUID;
-    DirectSoundCaptureDevice *device = *ppDevice;
+    IMMDevice *mmdevice;
+    struct _TestFormat *fmt;
+    DirectSoundCaptureDevice *device;
+    IAudioClient *client;
+
     TRACE("(%p, %s)\n", ppDevice, debugstr_guid(lpcGUID));
 
     /* Default device? */
     if ( !lpcGUID || IsEqualGUID(lpcGUID, &GUID_NULL) )
-	lpcGUID = &DSDEVID_DefaultCapture;
+        lpcGUID = &DSDEVID_DefaultCapture;
+
+    if(IsEqualGUID(lpcGUID, &DSDEVID_DefaultPlayback) ||
+            IsEqualGUID(lpcGUID, &DSDEVID_DefaultVoicePlayback))
+        return DSERR_NODRIVER;
 
     if (GetDeviceID(lpcGUID, &devGUID) != DS_OK) {
         WARN("invalid parameter: lpcGUID\n");
         return DSERR_INVALIDPARAM;
     }
 
-    widn = waveInGetNumDevs();
-    if (!widn) {
-	WARN("no audio devices found\n");
-	return DSERR_NODRIVER;
+    hr = get_mmdevice(eCapture, &devGUID, &mmdevice);
+    if(FAILED(hr))
+        return hr;
+
+    EnterCriticalSection(&DSOUND_capturers_lock);
+
+    LIST_FOR_EACH_ENTRY(device, &DSOUND_capturers, DirectSoundCaptureDevice, entry){
+        if(IsEqualGUID(&device->guid, &devGUID)){
+            IMMDevice_Release(mmdevice);
+            LeaveCriticalSection(&DSOUND_capturers_lock);
+            return DSERR_ALLOCATED;
+        }
     }
 
-    wid = devGUID.Data4[7];
-    if (!memcmp(&devGUID, &DSOUND_capture_guid, sizeof(GUID)-1)
-        && wid < widn)
-        found = TRUE;
-
-    if (found == FALSE) {
-	WARN("No device found matching given ID!\n");
-	return DSERR_NODRIVER;
-    }
-
-    if (DSOUND_capture[wid]) {
-        WARN("already in use\n");
-        return DSERR_ALLOCATED;
-    }
-
-    err = DirectSoundCaptureDevice_Create(&(device));
-    if (err != DS_OK) {
+    hr = DirectSoundCaptureDevice_Create(&device);
+    if (hr != DS_OK) {
         WARN("DirectSoundCaptureDevice_Create failed\n");
-        return err;
+        LeaveCriticalSection(&DSOUND_capturers_lock);
+        return hr;
     }
 
-    *ppDevice = device;
     device->guid = devGUID;
 
-    /* Disable the direct sound driver to force emulation if requested. */
-    device->driver = NULL;
-    if (ds_hw_accel != DS_HW_ACCEL_EMULATION)
-    {
-        err = mmErr(waveInMessage(UlongToHandle(wid),DRV_QUERYDSOUNDIFACE,(DWORD_PTR)&device->driver,0));
-        if ( (err != DS_OK) && (err != DSERR_UNSUPPORTED) ) {
-            WARN("waveInMessage failed; err=%x\n",err);
-            return err;
+    device->mmdevice = mmdevice;
+
+    device->drvcaps.dwFlags = 0;
+
+    device->drvcaps.dwFormats = 0;
+    device->drvcaps.dwChannels = 0;
+    hr = IMMDevice_Activate(mmdevice, &IID_IAudioClient,
+            CLSCTX_INPROC_SERVER, NULL, (void**)&client);
+    if(FAILED(hr)){
+        DeleteCriticalSection(&device->lock);
+        HeapFree(GetProcessHeap(), 0, device);
+        return DSERR_NODRIVER;
+    }
+
+    for(fmt = formats_to_test; fmt->flag; ++fmt){
+        if(DSOUND_check_supported(client, fmt->rate, fmt->depth, fmt->channels)){
+            device->drvcaps.dwFormats |= fmt->flag;
+            if(fmt->channels > device->drvcaps.dwChannels)
+                device->drvcaps.dwChannels = fmt->channels;
         }
     }
-    err = DS_OK;
+    IAudioClient_Release(client);
 
-    /* Get driver description */
-    if (device->driver) {
-        TRACE("using DirectSound driver\n");
-        err = IDsCaptureDriver_GetDriverDesc(device->driver, &(device->drvdesc));
-	if (err != DS_OK) {
-	    WARN("IDsCaptureDriver_GetDriverDesc failed\n");
-	    return err;
-	}
-    } else {
-        TRACE("using WINMM\n");
-        /* if no DirectSound interface available, use WINMM API instead */
-        device->drvdesc.dwFlags = DSDDESC_DOMMSYSTEMOPEN |
-            DSDDESC_DOMMSYSTEMSETFORMAT;
-    }
+    device->timerID = DSOUND_create_timer(DSOUND_capture_timer, (DWORD_PTR)device);
 
-    device->drvdesc.dnDevNode = wid;
+    list_add_tail(&DSOUND_capturers, &device->entry);
 
-    /* open the DirectSound driver if available */
-    if (device->driver && (err == DS_OK))
-        err = IDsCaptureDriver_Open(device->driver);
+    *ppDevice = device;
 
-    if (err == DS_OK) {
-        *ppDevice = device;
+    LeaveCriticalSection(&DSOUND_capturers_lock);
 
-        /* the driver is now open, so it's now allowed to call GetCaps */
-        if (device->driver) {
-	    device->drvcaps.dwSize = sizeof(device->drvcaps);
-            err = IDsCaptureDriver_GetCaps(device->driver,&(device->drvcaps));
-	    if (err != DS_OK) {
-		WARN("IDsCaptureDriver_GetCaps failed\n");
-		return err;
-	    }
-        } else /*if (device->hwi)*/ {
-            WAVEINCAPSA    wic;
-            err = mmErr(waveInGetDevCapsA((UINT)device->drvdesc.dnDevNode, &wic, sizeof(wic)));
-
-            if (err == DS_OK) {
-                device->drvcaps.dwFlags = 0;
-                lstrcpynA(device->drvdesc.szDrvname, wic.szPname,
-                          sizeof(device->drvdesc.szDrvname));
-
-                device->drvcaps.dwFlags |= DSCCAPS_EMULDRIVER;
-                device->drvcaps.dwFormats = wic.dwFormats;
-                device->drvcaps.dwChannels = wic.wChannels;
-            }
-        }
-    }
-
-    return err;
+    return S_OK;
 }
 
 
@@ -1152,24 +1065,23 @@ static HRESULT DirectSoundCaptureDevice_Initialize(
  */
 struct IDirectSoundCaptureImpl
 {
-    /* IUnknown fields */
-    const IDirectSoundCaptureVtbl     *lpVtbl;
-    LONG                               ref;
-
-    DirectSoundCaptureDevice          *device;
+    IDirectSoundCapture      IDirectSoundCapture_iface;
+    LONG                     ref;
+    DirectSoundCaptureDevice *device;
 };
 
+static inline struct IDirectSoundCaptureImpl *impl_from_IDirectSoundCapture(IDirectSoundCapture *iface)
+{
+    return CONTAINING_RECORD(iface, struct IDirectSoundCaptureImpl, IDirectSoundCapture_iface);
+}
+
 /***************************************************************************
  * IDirectSoundCaptureImpl
  */
-static HRESULT WINAPI
-IDirectSoundCaptureImpl_QueryInterface(
-    LPDIRECTSOUNDCAPTURE iface,
-    REFIID riid,
-    LPVOID* ppobj )
+static HRESULT WINAPI IDirectSoundCaptureImpl_QueryInterface(IDirectSoundCapture *iface,
+        REFIID riid, void **ppobj)
 {
-    IDirectSoundCaptureImpl *This = (IDirectSoundCaptureImpl *)iface;
-    TRACE( "(%p,%s,%p)\n", This, debugstr_guid(riid), ppobj );
+    TRACE( "(%p,%s,%p)\n", iface, debugstr_guid(riid), ppobj );
 
     if (ppobj == NULL) {
 	WARN("invalid parameter\n");
@@ -1178,34 +1090,30 @@ IDirectSoundCaptureImpl_QueryInterface(
 
     *ppobj = NULL;
 
-    if (IsEqualIID(riid, &IID_IUnknown)) {
-        IDirectSoundCapture_AddRef((LPDIRECTSOUNDCAPTURE)This);
-        *ppobj = This;
-        return DS_OK;
-    } else if (IsEqualIID(riid, &IID_IDirectSoundCapture)) {
-        IDirectSoundCapture_AddRef((LPDIRECTSOUNDCAPTURE)This);
-        *ppobj = This;
-        return DS_OK;
+    if (IsEqualIID(riid, &IID_IUnknown) || IsEqualIID(riid, &IID_IDirectSoundCapture)) {
+        IDirectSoundCapture_AddRef(iface);
+        *ppobj = iface;
+        return S_OK;
     }
 
     WARN("unsupported riid: %s\n", debugstr_guid(riid));
     return E_NOINTERFACE;
 }
 
-static ULONG WINAPI
-IDirectSoundCaptureImpl_AddRef( LPDIRECTSOUNDCAPTURE iface )
+static ULONG WINAPI IDirectSoundCaptureImpl_AddRef(IDirectSoundCapture *iface)
 {
-    IDirectSoundCaptureImpl *This = (IDirectSoundCaptureImpl *)iface;
+    IDirectSoundCaptureImpl *This = impl_from_IDirectSoundCapture(iface);
     ULONG ref = InterlockedIncrement(&(This->ref));
+
     TRACE("(%p) ref was %d\n", This, ref - 1);
     return ref;
 }
 
-static ULONG WINAPI
-IDirectSoundCaptureImpl_Release( LPDIRECTSOUNDCAPTURE iface )
+static ULONG WINAPI IDirectSoundCaptureImpl_Release(IDirectSoundCapture *iface)
 {
-    IDirectSoundCaptureImpl *This = (IDirectSoundCaptureImpl *)iface;
+    IDirectSoundCaptureImpl *This = impl_from_IDirectSoundCapture(iface);
     ULONG ref = InterlockedDecrement(&(This->ref));
+
     TRACE("(%p) ref was %d\n", This, ref + 1);
 
     if (!ref) {
@@ -1218,14 +1126,12 @@ IDirectSoundCaptureImpl_Release( LPDIRECTSOUNDCAPTURE iface )
     return ref;
 }
 
-static HRESULT WINAPI IDirectSoundCaptureImpl_CreateCaptureBuffer(
-    LPDIRECTSOUNDCAPTURE iface,
-    LPCDSCBUFFERDESC lpcDSCBufferDesc,
-    LPDIRECTSOUNDCAPTUREBUFFER* lplpDSCaptureBuffer,
-    LPUNKNOWN pUnk )
+static HRESULT WINAPI IDirectSoundCaptureImpl_CreateCaptureBuffer(IDirectSoundCapture *iface,
+        LPCDSCBUFFERDESC lpcDSCBufferDesc, IDirectSoundCaptureBuffer **lplpDSCaptureBuffer,
+        IUnknown *pUnk)
 {
+    IDirectSoundCaptureImpl *This = impl_from_IDirectSoundCapture(iface);
     HRESULT hr;
-    IDirectSoundCaptureImpl *This = (IDirectSoundCaptureImpl *)iface;
 
     TRACE( "(%p,%p,%p,%p)\n",iface,lpcDSCBufferDesc,lplpDSCaptureBuffer,pUnk);
 
@@ -1246,7 +1152,7 @@ static HRESULT WINAPI IDirectSoundCaptureImpl_CreateCaptureBuffer(
 
     /* FIXME: We can only have one buffer so what do we do here? */
     if (This->device->capture_buffer) {
-	WARN("lnvalid parameter: already has buffer\n");
+	WARN("invalid parameter: already has buffer\n");
 	return DSERR_INVALIDPARAM;    /* DSERR_GENERIC ? */
     }
 
@@ -1259,11 +1165,11 @@ static HRESULT WINAPI IDirectSoundCaptureImpl_CreateCaptureBuffer(
     return hr;
 }
 
-static HRESULT WINAPI IDirectSoundCaptureImpl_GetCaps(
-    LPDIRECTSOUNDCAPTURE iface,
-    LPDSCCAPS lpDSCCaps )
+static HRESULT WINAPI IDirectSoundCaptureImpl_GetCaps(IDirectSoundCapture *iface,
+        LPDSCCAPS lpDSCCaps)
 {
-    IDirectSoundCaptureImpl *This = (IDirectSoundCaptureImpl *)iface;
+    IDirectSoundCaptureImpl *This = impl_from_IDirectSoundCapture(iface);
+
     TRACE("(%p,%p)\n",This,lpDSCCaps);
 
     if (This->device == NULL) {
@@ -1291,11 +1197,11 @@ static HRESULT WINAPI IDirectSoundCaptureImpl_GetCaps(
     return DS_OK;
 }
 
-static HRESULT WINAPI IDirectSoundCaptureImpl_Initialize(
-    LPDIRECTSOUNDCAPTURE iface,
-    LPCGUID lpcGUID )
+static HRESULT WINAPI IDirectSoundCaptureImpl_Initialize(IDirectSoundCapture *iface,
+        LPCGUID lpcGUID)
 {
-    IDirectSoundCaptureImpl *This = (IDirectSoundCaptureImpl *)iface;
+    IDirectSoundCaptureImpl *This = impl_from_IDirectSoundCapture(iface);
+
     TRACE("(%p,%s)\n", This, debugstr_guid(lpcGUID));
 
     if (This->device != NULL) {
@@ -1332,7 +1238,7 @@ static HRESULT IDirectSoundCaptureImpl_Create(
         return DSERR_OUTOFMEMORY;
     }
 
-    pDSC->lpVtbl = &dscvt;
+    pDSC->IDirectSoundCapture_iface.lpVtbl = &dscvt;
     pDSC->ref    = 0;
     pDSC->device = NULL;
 
@@ -1341,11 +1247,9 @@ static HRESULT IDirectSoundCaptureImpl_Create(
     return DS_OK;
 }
 
-HRESULT DSOUND_CaptureCreate(
-    REFIID riid,
-    LPDIRECTSOUNDCAPTURE *ppDSC)
+HRESULT DSOUND_CaptureCreate(REFIID riid, IDirectSoundCapture **ppDSC)
 {
-    LPDIRECTSOUNDCAPTURE pDSC;
+    IDirectSoundCapture *pDSC;
     HRESULT hr;
     TRACE("(%s, %p)\n", debugstr_guid(riid), ppDSC);
 
@@ -1421,13 +1325,12 @@ HRESULT DSOUND_CaptureCreate8(
  *
  *    DSERR_ALLOCATED is returned for sound devices that do not support full duplex.
  */
-HRESULT WINAPI DirectSoundCaptureCreate(
-    LPCGUID lpcGUID,
-    LPDIRECTSOUNDCAPTURE *ppDSC,
-    LPUNKNOWN pUnkOuter)
+HRESULT WINAPI DirectSoundCaptureCreate(LPCGUID lpcGUID, IDirectSoundCapture **ppDSC,
+        IUnknown *pUnkOuter)
 {
     HRESULT hr;
-    LPDIRECTSOUNDCAPTURE pDSC;
+    IDirectSoundCapture *pDSC;
+
     TRACE("(%s,%p,%p)\n", debugstr_guid(lpcGUID), ppDSC, pUnkOuter);
 
     if (ppDSC == NULL) {
