@@ -49,17 +49,36 @@
 #include "dsound.h"
 #include "dsconf.h"
 #include "ks.h"
+#include "rpcproxy.h"
+#include "rpc.h"
+#include "rpcndr.h"
+#include "unknwn.h"
+#include "oleidl.h"
+#include "shobjidl.h"
+
 #include "initguid.h"
 #include "ksmedia.h"
-#include "dsdriver.h"
+#include "propkey.h"
+#include "devpkey.h"
 
 #include "dsound_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(dsound);
 
-DirectSoundDevice*	DSOUND_renderer[MAXWAVEDRIVERS];
-GUID DSOUND_renderer_guid = { 0xbd6dd71a,0x3deb,0x11d1, {0xb1,0x71,0x00,0xc0,0x4f,0xc2,0x00,0x00} };
-GUID DSOUND_capture_guid = { 0xbd6dd71b,0x3deb,0x11d1, {0xb1,0x71,0x00,0xc0,0x4f,0xc2,0x00,0x00} };
+struct list DSOUND_renderers = LIST_INIT(DSOUND_renderers);
+CRITICAL_SECTION DSOUND_renderers_lock;
+
+struct list DSOUND_capturers = LIST_INIT(DSOUND_capturers);
+CRITICAL_SECTION DSOUND_capturers_lock;
+
+GUID                    DSOUND_renderer_guids[MAXWAVEDRIVERS];
+GUID                    DSOUND_capture_guids[MAXWAVEDRIVERS];
+
+static IMMDeviceEnumerator *g_devenum;
+static CRITICAL_SECTION g_devenum_lock;
+static HANDLE g_devenum_thread;
+
+WCHAR wine_vxd_drv[] = { 'w','i','n','e','m','m','.','v','x','d', 0 };
 
 HRESULT mmErr(UINT err)
 {
@@ -89,16 +108,12 @@ HRESULT mmErr(UINT err)
 }
 
 /* All default settings, you most likely don't want to touch these, see wiki on UsefulRegistryKeys */
-int ds_emuldriver = 0;
 int ds_hel_buflen = 32768 * 2;
 int ds_snd_queue_max = 10;
-int ds_snd_queue_min = 6;
 int ds_snd_shadow_maxsize = 2;
-int ds_hw_accel = DS_HW_ACCEL_FULL;
 int ds_default_sample_rate = 44100;
 int ds_default_bits_per_sample = 16;
-static int ds_default_playback;
-static int ds_default_capture;
+static HINSTANCE instance;
 
 /*
  * Get a config key from either the app-specific or the default config
@@ -147,38 +162,15 @@ void setup_dsound_options(void)
 
     /* get options */
 
-    if (!get_config_key( hkey, appkey, "EmulDriver", buffer, MAX_PATH ))
-        ds_emuldriver = strcmp(buffer, "N");
-
     if (!get_config_key( hkey, appkey, "HelBuflen", buffer, MAX_PATH ))
         ds_hel_buflen = atoi(buffer);
 
     if (!get_config_key( hkey, appkey, "SndQueueMax", buffer, MAX_PATH ))
         ds_snd_queue_max = atoi(buffer);
 
-    if (!get_config_key( hkey, appkey, "SndQueueMin", buffer, MAX_PATH ))
-        ds_snd_queue_min = atoi(buffer);
-
-    if (!get_config_key( hkey, appkey, "HardwareAcceleration", buffer, MAX_PATH )) {
-	if (strcmp(buffer, "Full") == 0)
-	    ds_hw_accel = DS_HW_ACCEL_FULL;
-	else if (strcmp(buffer, "Standard") == 0)
-	    ds_hw_accel = DS_HW_ACCEL_STANDARD;
-	else if (strcmp(buffer, "Basic") == 0)
-	    ds_hw_accel = DS_HW_ACCEL_BASIC;
-	else if (strcmp(buffer, "Emulation") == 0)
-	    ds_hw_accel = DS_HW_ACCEL_EMULATION;
-    }
-
-    if (!get_config_key( hkey, appkey, "DefaultPlayback", buffer, MAX_PATH ))
-        ds_default_playback = atoi(buffer);
-
     if (!get_config_key( hkey, appkey, "MaxShadowSize", buffer, MAX_PATH ))
         ds_snd_shadow_maxsize = atoi(buffer);
 
-    if (!get_config_key( hkey, appkey, "DefaultCapture", buffer, MAX_PATH ))
-        ds_default_capture = atoi(buffer);
-
     if (!get_config_key( hkey, appkey, "DefaultSampleRate", buffer, MAX_PATH ))
         ds_default_sample_rate = atoi(buffer);
 
@@ -188,18 +180,8 @@ void setup_dsound_options(void)
     if (appkey) RegCloseKey( appkey );
     if (hkey) RegCloseKey( hkey );
 
-    TRACE("ds_emuldriver = %d\n", ds_emuldriver);
     TRACE("ds_hel_buflen = %d\n", ds_hel_buflen);
     TRACE("ds_snd_queue_max = %d\n", ds_snd_queue_max);
-    TRACE("ds_snd_queue_min = %d\n", ds_snd_queue_min);
-    TRACE("ds_hw_accel = %s\n",
-        ds_hw_accel==DS_HW_ACCEL_FULL ? "Full" :
-        ds_hw_accel==DS_HW_ACCEL_STANDARD ? "Standard" :
-        ds_hw_accel==DS_HW_ACCEL_BASIC ? "Basic" :
-        ds_hw_accel==DS_HW_ACCEL_EMULATION ? "Emulation" :
-        "Unknown");
-    TRACE("ds_default_playback = %d\n", ds_default_playback);
-    TRACE("ds_default_capture = %d\n", ds_default_playback);
     TRACE("ds_default_sample_rate = %d\n", ds_default_sample_rate);
     TRACE("ds_default_bits_per_sample = %d\n", ds_default_bits_per_sample);
     TRACE("ds_snd_shadow_maxsize = %d\n", ds_snd_shadow_maxsize);
@@ -218,6 +200,117 @@ static const char * get_device_id(LPCGUID pGuid)
     return debugstr_guid(pGuid);
 }
 
+/* The MMDeviceEnumerator object has to be created & destroyed
+ * from the same thread. */
+static DWORD WINAPI devenum_thread_proc(void *arg)
+{
+    HANDLE evt = arg;
+    HRESULT hr;
+    MSG msg;
+
+    hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
+    if(FAILED(hr)){
+         ERR("CoInitializeEx failed: %08x\n", hr);
+         return 1;
+    }
+
+    hr = CoCreateInstance(&CLSID_MMDeviceEnumerator, NULL,
+            CLSCTX_INPROC_SERVER, &IID_IMMDeviceEnumerator, (void**)&g_devenum);
+    if(FAILED(hr)){
+        ERR("CoCreateInstance failed: %08x\n", hr);
+        CoUninitialize();
+        return 1;
+    }
+
+    SetEvent(evt);
+
+    PeekMessageW(&msg, NULL, WM_USER, WM_USER, PM_NOREMOVE);
+
+    while(GetMessageW(&msg, NULL, 0, 0)){
+        if(msg.hwnd)
+            DispatchMessageW(&msg);
+        else
+            ERR("Unknown message: %04x\n", msg.message);
+    }
+
+    IMMDeviceEnumerator_Release(g_devenum);
+    g_devenum = NULL;
+    CoUninitialize();
+
+    return 0;
+}
+
+static IMMDeviceEnumerator *get_mmdevenum(void)
+{
+    HANDLE events[2];
+    DWORD wait;
+
+    EnterCriticalSection(&g_devenum_lock);
+
+    if(g_devenum){
+        LeaveCriticalSection(&g_devenum_lock);
+        return g_devenum;
+    }
+
+    events[0] = CreateEventW(NULL, FALSE, FALSE, NULL);
+
+    g_devenum_thread = CreateThread(NULL, 0, devenum_thread_proc,
+            events[0], 0, NULL);
+    if(!g_devenum_thread){
+        LeaveCriticalSection(&g_devenum_lock);
+        CloseHandle(events[0]);
+        return NULL;
+    }
+
+    events[1] = g_devenum_thread;
+    wait = WaitForMultipleObjects(2, events, FALSE, INFINITE);
+    CloseHandle(events[0]);
+    if(wait != WAIT_OBJECT_0){
+        if(wait == 1 + WAIT_OBJECT_0){
+            CloseHandle(g_devenum_thread);
+            g_devenum_thread = NULL;
+        }
+        LeaveCriticalSection(&g_devenum_lock);
+        return NULL;
+    }
+
+    LeaveCriticalSection(&g_devenum_lock);
+
+    return g_devenum;
+}
+
+static HRESULT get_mmdevice_guid(IMMDevice *device, IPropertyStore *ps,
+        GUID *guid)
+{
+    PROPVARIANT pv;
+    HRESULT hr;
+
+    if(!ps){
+        hr = IMMDevice_OpenPropertyStore(device, STGM_READ, &ps);
+        if(FAILED(hr)){
+            WARN("OpenPropertyStore failed: %08x\n", hr);
+            return hr;
+        }
+    }else
+        IPropertyStore_AddRef(ps);
+
+    PropVariantInit(&pv);
+
+    hr = IPropertyStore_GetValue(ps, &PKEY_AudioEndpoint_GUID, &pv);
+    if(FAILED(hr)){
+        IPropertyStore_Release(ps);
+        WARN("GetValue(GUID) failed: %08x\n", hr);
+        return hr;
+    }
+
+    CLSIDFromString(pv.u.pwszVal, guid);
+
+    PropVariantClear(&pv);
+    IPropertyStore_Release(ps);
+
+    return S_OK;
+}
+
 /***************************************************************************
  * GetDeviceID	[DSOUND.9]
  *
@@ -240,36 +333,51 @@ static const char * get_device_id(LPCGUID pGuid)
  */
 HRESULT WINAPI GetDeviceID(LPCGUID pGuidSrc, LPGUID pGuidDest)
 {
+    IMMDeviceEnumerator *devenum;
+    EDataFlow flow = (EDataFlow)-1;
+    ERole role = (ERole)-1;
+    HRESULT hr;
+
     TRACE("(%s,%p)\n", get_device_id(pGuidSrc),pGuidDest);
 
-    if ( pGuidSrc == NULL) {
-	WARN("invalid parameter: pGuidSrc == NULL\n");
-	return DSERR_INVALIDPARAM;
+    if(!pGuidSrc || !pGuidDest)
+        return DSERR_INVALIDPARAM;
+
+    devenum = get_mmdevenum();
+    if(!devenum)
+        return DSERR_GENERIC;
+
+    if(IsEqualGUID(&DSDEVID_DefaultPlayback, pGuidSrc)){
+        role = eMultimedia;
+        flow = eRender;
+    }else if(IsEqualGUID(&DSDEVID_DefaultVoicePlayback, pGuidSrc)){
+        role = eCommunications;
+        flow = eRender;
+    }else if(IsEqualGUID(&DSDEVID_DefaultCapture, pGuidSrc)){
+        role = eMultimedia;
+        flow = eCapture;
+    }else if(IsEqualGUID(&DSDEVID_DefaultVoiceCapture, pGuidSrc)){
+        role = eCommunications;
+        flow = eCapture;
     }
 
-    if ( pGuidDest == NULL ) {
-	WARN("invalid parameter: pGuidDest == NULL\n");
-	return DSERR_INVALIDPARAM;
-    }
+    if(role != (ERole)-1 && flow != (EDataFlow)-1){
+        IMMDevice *device;
 
-    if ( IsEqualGUID( &DSDEVID_DefaultPlayback, pGuidSrc ) ||
-         IsEqualGUID( &DSDEVID_DefaultVoicePlayback, pGuidSrc ) ) {
-        *pGuidDest = DSOUND_renderer_guid;
-        pGuidDest->Data4[7] = ds_default_playback;
-        TRACE("returns %s\n", get_device_id(pGuidDest));
-        return DS_OK;
-    }
+        hr = IMMDeviceEnumerator_GetDefaultAudioEndpoint(devenum,
+                flow, role, &device);
+        if(FAILED(hr)){
+            WARN("GetDefaultAudioEndpoint failed: %08x\n", hr);
+            return DSERR_NODRIVER;
+        }
 
-    if ( IsEqualGUID( &DSDEVID_DefaultCapture, pGuidSrc ) ||
-         IsEqualGUID( &DSDEVID_DefaultVoiceCapture, pGuidSrc ) ) {
-        *pGuidDest = DSOUND_capture_guid;
-        pGuidDest->Data4[7] = ds_default_capture;
-        TRACE("returns %s\n", get_device_id(pGuidDest));
-        return DS_OK;
+        hr = get_mmdevice_guid(device, NULL, pGuidDest);
+        IMMDevice_Release(device);
+
+        return (hr == S_OK) ? DS_OK : hr;
     }
 
     *pGuidDest = *pGuidSrc;
-    TRACE("returns %s\n", get_device_id(pGuidDest));
 
     return DS_OK;
 }
@@ -321,6 +429,141 @@ HRESULT WINAPI DirectSoundEnumerateA(
     return DirectSoundEnumerateW(a_to_w_callback, &context);
 }
 
+HRESULT get_mmdevice(EDataFlow flow, const GUID *tgt, IMMDevice **device)
+{
+    IMMDeviceEnumerator *devenum;
+    IMMDeviceCollection *coll;
+    UINT count, i;
+    HRESULT hr;
+
+    devenum = get_mmdevenum();
+    if(!devenum)
+        return DSERR_GENERIC;
+
+    hr = IMMDeviceEnumerator_EnumAudioEndpoints(devenum, flow,
+            DEVICE_STATE_ACTIVE, &coll);
+    if(FAILED(hr)){
+        WARN("EnumAudioEndpoints failed: %08x\n", hr);
+        return hr;
+    }
+
+    hr = IMMDeviceCollection_GetCount(coll, &count);
+    if(FAILED(hr)){
+        IMMDeviceCollection_Release(coll);
+        WARN("GetCount failed: %08x\n", hr);
+        return hr;
+    }
+
+    for(i = 0; i < count; ++i){
+        GUID guid;
+
+        hr = IMMDeviceCollection_Item(coll, i, device);
+        if(FAILED(hr))
+            continue;
+
+        hr = get_mmdevice_guid(*device, NULL, &guid);
+        if(FAILED(hr)){
+            IMMDevice_Release(*device);
+            continue;
+        }
+
+        if(IsEqualGUID(&guid, tgt))
+            return DS_OK;
+
+        IMMDevice_Release(*device);
+    }
+
+    WARN("No device with GUID %s found!\n", wine_dbgstr_guid(tgt));
+
+    return DSERR_INVALIDPARAM;
+}
+
+/* S_FALSE means the callback returned FALSE at some point
+ * S_OK means the callback always returned TRUE */
+HRESULT enumerate_mmdevices(EDataFlow flow, GUID *guids,
+        LPDSENUMCALLBACKW cb, void *user)
+{
+    IMMDeviceEnumerator *devenum;
+    IMMDeviceCollection *coll;
+    UINT count, i;
+    BOOL keep_going;
+    HRESULT hr;
+
+    static const WCHAR primary_desc[] = {'P','r','i','m','a','r','y',' ',
+        'S','o','u','n','d',' ','D','r','i','v','e','r',0};
+    static const WCHAR empty_drv[] = {0};
+
+    devenum = get_mmdevenum();
+    if(!devenum)
+        return DS_OK;
+
+    hr = IMMDeviceEnumerator_EnumAudioEndpoints(g_devenum, flow,
+            DEVICE_STATE_ACTIVE, &coll);
+    if(FAILED(hr)){
+        WARN("EnumAudioEndpoints failed: %08x\n", hr);
+        return DS_OK;
+    }
+
+    hr = IMMDeviceCollection_GetCount(coll, &count);
+    if(FAILED(hr)){
+        IMMDeviceCollection_Release(coll);
+        WARN("GetCount failed: %08x\n", hr);
+        return DS_OK;
+    }
+
+    if(count == 0)
+        return DS_OK;
+
+    keep_going = cb(NULL, primary_desc, empty_drv, user);
+
+    for(i = 0; keep_going && i < count; ++i){
+        IMMDevice *device;
+        IPropertyStore *ps;
+        PROPVARIANT pv;
+
+        PropVariantInit(&pv);
+
+        hr = IMMDeviceCollection_Item(coll, i, &device);
+        if(FAILED(hr)){
+            WARN("Item failed: %08x\n", hr);
+            continue;
+        }
+
+        hr = IMMDevice_OpenPropertyStore(device, STGM_READ, &ps);
+        if(FAILED(hr)){
+            IMMDevice_Release(device);
+            WARN("OpenPropertyStore failed: %08x\n", hr);
+            continue;
+        }
+
+        hr = get_mmdevice_guid(device, ps, &guids[i]);
+        if(FAILED(hr)){
+            IPropertyStore_Release(ps);
+            IMMDevice_Release(device);
+            continue;
+        }
+
+        hr = IPropertyStore_GetValue(ps,
+                (const PROPERTYKEY *)&DEVPKEY_Device_FriendlyName, &pv);
+        if(FAILED(hr)){
+            IPropertyStore_Release(ps);
+            IMMDevice_Release(device);
+            WARN("GetValue(FriendlyName) failed: %08x\n", hr);
+            continue;
+        }
+
+        keep_going = cb(&guids[i], pv.u.pwszVal, wine_vxd_drv, user);
+
+        PropVariantClear(&pv);
+        IPropertyStore_Release(ps);
+        IMMDevice_Release(device);
+    }
+
+    IMMDeviceCollection_Release(coll);
+
+    return (keep_going == TRUE) ? S_OK : S_FALSE;
+}
+
 /***************************************************************************
  * DirectSoundEnumerateW [DSOUND.3]
  *
@@ -338,55 +581,20 @@ HRESULT WINAPI DirectSoundEnumerateW(
 	LPDSENUMCALLBACKW lpDSEnumCallback,
 	LPVOID lpContext )
 {
-    unsigned devs, wod;
-    DSDRIVERDESC desc;
-    GUID guid;
-    int err;
-    WCHAR wDesc[MAXPNAMELEN];
-    WCHAR wName[MAXPNAMELEN];
+    HRESULT hr;
 
-    TRACE("lpDSEnumCallback = %p, lpContext = %p\n",
-	lpDSEnumCallback, lpContext);
+    TRACE("(%p,%p)\n", lpDSEnumCallback, lpContext);
 
     if (lpDSEnumCallback == NULL) {
-	WARN("invalid parameter: lpDSEnumCallback == NULL\n");
-	return DSERR_INVALIDPARAM;
+        WARN("invalid parameter: lpDSEnumCallback == NULL\n");
+        return DSERR_INVALIDPARAM;
     }
 
-    devs = waveOutGetNumDevs();
-    if (devs > 0) {
-	if (GetDeviceID(&DSDEVID_DefaultPlayback, &guid) == DS_OK) {
-            static const WCHAR empty[] = { 0 };
-            wod = guid.Data4[7];
-            if (wod < devs) {
-                err = mmErr(waveOutMessage(UlongToHandle(wod),DRV_QUERYDSOUNDDESC,(DWORD_PTR)&desc,0));
-                if (err == DS_OK) {
-                    TRACE("calling lpDSEnumCallback(NULL,\"%s\",\"%s\",%p)\n",
-                          "Primary Sound Driver",desc.szDrvname,lpContext);
-                    MultiByteToWideChar( CP_ACP, 0, "Primary Sound Driver", -1,
-                                         wDesc, sizeof(wDesc)/sizeof(WCHAR) );
-                    if (lpDSEnumCallback(NULL, wDesc, empty, lpContext) == FALSE)
-                        return DS_OK;
-		}
-	    }
-	}
-    }
+    setup_dsound_options();
 
-    for (wod = 0; wod < devs; ++wod) {
-        err = mmErr(waveOutMessage(UlongToHandle(wod),DRV_QUERYDSOUNDDESC,(DWORD_PTR)&desc,0));
-	if (err == DS_OK) {
-            guid.Data4[7] = wod;
-            TRACE("calling lpDSEnumCallback(%s,\"%s\",\"%s\",%p)\n",
-                  debugstr_guid(&guid),desc.szDesc,desc.szDrvname,lpContext);
-            MultiByteToWideChar( CP_ACP, 0, desc.szDesc, -1,
-                                 wDesc, sizeof(wDesc)/sizeof(WCHAR) );
-            MultiByteToWideChar( CP_ACP, 0, desc.szDrvname, -1,
-                                 wName, sizeof(wName)/sizeof(WCHAR) );
-            if (lpDSEnumCallback(&guid, wDesc, wName, lpContext) == FALSE)
-                return DS_OK;
-	}
-    }
-    return DS_OK;
+    hr = enumerate_mmdevices(eRender, DSOUND_renderer_guids,
+            lpDSEnumCallback, lpContext);
+    return SUCCEEDED(hr) ? DS_OK : hr;
 }
 
 /***************************************************************************
@@ -437,56 +645,20 @@ DirectSoundCaptureEnumerateW(
     LPDSENUMCALLBACKW lpDSEnumCallback,
     LPVOID lpContext)
 {
-    unsigned devs, wid;
-    DSDRIVERDESC desc;
-    GUID guid;
-    int err;
-    WCHAR wDesc[MAXPNAMELEN];
-    WCHAR wName[MAXPNAMELEN];
+    HRESULT hr;
 
     TRACE("(%p,%p)\n", lpDSEnumCallback, lpContext );
 
     if (lpDSEnumCallback == NULL) {
-	WARN("invalid parameter: lpDSEnumCallback == NULL\n");
+        WARN("invalid parameter: lpDSEnumCallback == NULL\n");
         return DSERR_INVALIDPARAM;
     }
 
-    devs = waveInGetNumDevs();
-    if (devs > 0) {
-	if (GetDeviceID(&DSDEVID_DefaultCapture, &guid) == DS_OK) {
-            wid = guid.Data4[7];
-            if (wid < devs) {
-                err = mmErr(waveInMessage(UlongToHandle(wid),DRV_QUERYDSOUNDDESC,(DWORD_PTR)&desc,0));
-                if (err == DS_OK) {
-                    TRACE("calling lpDSEnumCallback(NULL,\"%s\",\"%s\",%p)\n",
-                          "Primary Sound Capture Driver",desc.szDrvname,lpContext);
-                    MultiByteToWideChar( CP_ACP, 0, "Primary Sound Capture Driver", -1,
-                                         wDesc, sizeof(wDesc)/sizeof(WCHAR) );
-                    MultiByteToWideChar( CP_ACP, 0, desc.szDrvname, -1,
-                                         wName, sizeof(wName)/sizeof(WCHAR) );
-                    if (lpDSEnumCallback(NULL, wDesc, wName, lpContext) == FALSE)
-                        return DS_OK;
-                }
-	    }
-	}
-    }
+    setup_dsound_options();
 
-    for (wid = 0; wid < devs; ++wid) {
-        err = mmErr(waveInMessage(UlongToHandle(wid),DRV_QUERYDSOUNDDESC,(DWORD_PTR)&desc,0));
-	if (err == DS_OK) {
-            guid.Data4[7] = wid;
-            TRACE("calling lpDSEnumCallback(%s,\"%s\",\"%s\",%p)\n",
-                  debugstr_guid(&guid),desc.szDesc,desc.szDrvname,lpContext);
-            MultiByteToWideChar( CP_ACP, 0, desc.szDesc, -1,
-                                 wDesc, sizeof(wDesc)/sizeof(WCHAR) );
-            MultiByteToWideChar( CP_ACP, 0, desc.szDrvname, -1,
-                                 wName, sizeof(wName)/sizeof(WCHAR) );
-            if (lpDSEnumCallback(&guid, wDesc, wName, lpContext) == FALSE)
-                return DS_OK;
-	}
-    }
-
-    return DS_OK;
+    hr = enumerate_mmdevices(eCapture, DSOUND_capture_guids,
+            lpDSEnumCallback, lpContext);
+    return SUCCEEDED(hr) ? DS_OK : hr;
 }
 
 /*******************************************************************************
@@ -496,15 +668,20 @@ DirectSoundCaptureEnumerateW(
 typedef  HRESULT (*FnCreateInstance)(REFIID riid, LPVOID *ppobj);
  
 typedef struct {
-    const IClassFactoryVtbl *lpVtbl;
+    IClassFactory IClassFactory_iface;
     REFCLSID rclsid;
     FnCreateInstance pfnCreateInstance;
 } IClassFactoryImpl;
 
+static inline IClassFactoryImpl *impl_from_IClassFactory(IClassFactory *iface)
+{
+    return CONTAINING_RECORD(iface, IClassFactoryImpl, IClassFactory_iface);
+}
+
 static HRESULT WINAPI
 DSCF_QueryInterface(LPCLASSFACTORY iface, REFIID riid, LPVOID *ppobj)
 {
-    IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
+    IClassFactoryImpl *This = impl_from_IClassFactory(iface);
     TRACE("(%p, %s, %p)\n", This, debugstr_guid(riid), ppobj);
     if (ppobj == NULL)
         return E_POINTER;
@@ -536,7 +713,7 @@ static HRESULT WINAPI DSCF_CreateInstance(
     REFIID riid,
     LPVOID *ppobj)
 {
-    IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
+    IClassFactoryImpl *This = impl_from_IClassFactory(iface);
     TRACE("(%p, %p, %s, %p)\n", This, pOuter, debugstr_guid(riid), ppobj);
 
     if (pOuter)
@@ -552,7 +729,7 @@ static HRESULT WINAPI DSCF_CreateInstance(
  
 static HRESULT WINAPI DSCF_LockServer(LPCLASSFACTORY iface, BOOL dolock)
 {
-    IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
+    IClassFactoryImpl *This = impl_from_IClassFactory(iface);
     FIXME("(%p, %d) stub!\n", This, dolock);
     return S_OK;
 }
@@ -566,13 +743,13 @@ static const IClassFactoryVtbl DSCF_Vtbl = {
 };
 
 static IClassFactoryImpl DSOUND_CF[] = {
-    { &DSCF_Vtbl, &CLSID_DirectSound, (FnCreateInstance)DSOUND_Create },
-    { &DSCF_Vtbl, &CLSID_DirectSound8, (FnCreateInstance)DSOUND_Create8 },
-    { &DSCF_Vtbl, &CLSID_DirectSoundCapture, (FnCreateInstance)DSOUND_CaptureCreate },
-    { &DSCF_Vtbl, &CLSID_DirectSoundCapture8, (FnCreateInstance)DSOUND_CaptureCreate8 },
-    { &DSCF_Vtbl, &CLSID_DirectSoundFullDuplex, (FnCreateInstance)DSOUND_FullDuplexCreate },
-    { &DSCF_Vtbl, &CLSID_DirectSoundPrivate, (FnCreateInstance)IKsPrivatePropertySetImpl_Create },
-    { NULL, NULL, NULL }
+    { { &DSCF_Vtbl }, &CLSID_DirectSound, (FnCreateInstance)DSOUND_Create },
+    { { &DSCF_Vtbl }, &CLSID_DirectSound8, (FnCreateInstance)DSOUND_Create8 },
+    { { &DSCF_Vtbl }, &CLSID_DirectSoundCapture, (FnCreateInstance)DSOUND_CaptureCreate },
+    { { &DSCF_Vtbl }, &CLSID_DirectSoundCapture8, (FnCreateInstance)DSOUND_CaptureCreate8 },
+    { { &DSCF_Vtbl }, &CLSID_DirectSoundFullDuplex, (FnCreateInstance)DSOUND_FullDuplexCreate },
+    { { &DSCF_Vtbl }, &CLSID_DirectSoundPrivate, (FnCreateInstance)IKsPrivatePropertySetImpl_Create },
+    { { NULL }, NULL, NULL }
 };
 
 /*******************************************************************************
@@ -612,7 +789,7 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppv)
 
     while (NULL != DSOUND_CF[i].rclsid) {
         if (IsEqualGUID(rclsid, DSOUND_CF[i].rclsid)) {
-            DSCF_AddRef((IClassFactory*) &DSOUND_CF[i]);
+            DSCF_AddRef(&DSOUND_CF[i].IClassFactory_iface);
             *ppv = &DSOUND_CF[i];
             return S_OK;
         }
@@ -630,12 +807,11 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppv)
  * Determines whether the DLL is in use.
  *
  * RETURNS
- *    Success: S_OK
- *    Failure: S_FALSE
+ *    Can unload now: S_OK
+ *    Cannot unload now (the DLL is still active): S_FALSE
  */
 HRESULT WINAPI DllCanUnloadNow(void)
 {
-    FIXME("(void): stub\n");
     return S_FALSE;
 }
 
@@ -650,19 +826,18 @@ HRESULT WINAPI DllCanUnloadNow(void)
  */
 BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpvReserved)
 {
-    int i;
     TRACE("(%p %d %p)\n", hInstDLL, fdwReason, lpvReserved);
 
     switch (fdwReason) {
     case DLL_PROCESS_ATTACH:
         TRACE("DLL_PROCESS_ATTACH\n");
-        for (i = 0; i < MAXWAVEDRIVERS; i++) {
-            DSOUND_renderer[i] = NULL;
-            DSOUND_capture[i] = NULL;
-        }
+        instance = hInstDLL;
         DisableThreadLibraryCalls(hInstDLL);
         /* Increase refcount on dsound by 1 */
         GetModuleHandleExW(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, (LPCWSTR)hInstDLL, &hInstDLL);
+        InitializeCriticalSection(&DSOUND_renderers_lock);
+        InitializeCriticalSection(&DSOUND_capturers_lock);
+        InitializeCriticalSection(&g_devenum_lock);
         break;
     case DLL_PROCESS_DETACH:
         TRACE("DLL_PROCESS_DETACH\n");
@@ -673,3 +848,19 @@ BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpvReserved)
     }
     return TRUE;
 }
+
+/***********************************************************************
+ *		DllRegisterServer (DSOUND.@)
+ */
+HRESULT WINAPI DllRegisterServer(void)
+{
+    return __wine_register_resources( instance );
+}
+
+/***********************************************************************
+ *		DllUnregisterServer (DSOUND.@)
+ */
+HRESULT WINAPI DllUnregisterServer(void)
+{
+    return __wine_unregister_resources( instance );
+}
