@@ -54,65 +54,31 @@ DWORD nulldrv_GetImage( PHYSDEV dev, HBITMAP hbitmap, BITMAPINFO *info,
                         struct gdi_image_bits *bits, struct bitblt_coords *src )
 {
     BITMAPOBJ *bmp;
-    int height, width_bytes;
+    DWORD ret;
 
     if (!hbitmap) return ERROR_NOT_SUPPORTED;
     if (!(bmp = GDI_GetObjPtr( hbitmap, OBJ_BITMAP ))) return ERROR_INVALID_HANDLE;
 
-    info->bmiHeader.biSize          = sizeof(info->bmiHeader);
-    info->bmiHeader.biPlanes        = 1;
-    info->bmiHeader.biBitCount      = bmp->bitmap.bmBitsPixel;
-    info->bmiHeader.biCompression   = BI_RGB;
-    info->bmiHeader.biXPelsPerMeter = 0;
-    info->bmiHeader.biYPelsPerMeter = 0;
-    info->bmiHeader.biClrUsed       = 0;
-    info->bmiHeader.biClrImportant  = 0;
-    if (bmp->bitmap.bmBitsPixel == 16)
+    if (bmp->bitmap.bmBits || !bits)
     {
-        DWORD *masks = (DWORD *)info->bmiColors;
-        masks[0] = 0x7c00;
-        masks[1] = 0x03e0;
-        masks[2] = 0x001f;
-        info->bmiHeader.biCompression = BI_BITFIELDS;
+        ret = dib_driver.pGetImage( 0, hbitmap, info, bits, src );
     }
-    if (!bits) goto done;
-
-    height = src->visrect.bottom - src->visrect.top;
-    width_bytes = get_dib_stride( bmp->bitmap.bmWidth, bmp->bitmap.bmBitsPixel );
-    info->bmiHeader.biWidth     = bmp->bitmap.bmWidth;
-    info->bmiHeader.biHeight    = -height;
-    info->bmiHeader.biSizeImage = height * width_bytes;
-
-    /* make the source rectangle relative to the returned bits */
-    src->y -= src->visrect.top;
-    offset_rect( &src->visrect, 0, -src->visrect.top );
-
-    if (bmp->bitmap.bmBits && bmp->bitmap.bmWidthBytes == width_bytes)
+    else if (!(ret = dib_driver.pGetImage( 0, hbitmap, info, NULL, src )))
     {
-        bits->ptr = (char *)bmp->bitmap.bmBits + src->visrect.top * width_bytes;
-        bits->is_copy = FALSE;
-        bits->free = NULL;
-    }
-    else
-    {
-        bits->ptr = HeapAlloc( GetProcessHeap(), 0, info->bmiHeader.biSizeImage );
+        info->bmiHeader.biHeight = -(src->visrect.bottom - src->visrect.top);
+        info->bmiHeader.biSizeImage = get_dib_image_size( info );
+
+        /* make the source rectangle relative to the returned bits */
+        src->y -= src->visrect.top;
+        offset_rect( &src->visrect, 0, -src->visrect.top );
+
+        /* return all-zero bits */
+        bits->ptr = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, info->bmiHeader.biSizeImage );
         bits->is_copy = TRUE;
         bits->free = free_heap_bits;
-        if (bmp->bitmap.bmBits)
-        {
-            /* fixup the line alignment */
-            char *src = bmp->bitmap.bmBits, *dst = bits->ptr;
-            for ( ; height > 0; height--, src += bmp->bitmap.bmWidthBytes, dst += width_bytes)
-            {
-                memcpy( dst, src, bmp->bitmap.bmWidthBytes );
-                memset( dst + bmp->bitmap.bmWidthBytes, 0, width_bytes - bmp->bitmap.bmWidthBytes );
-            }
-        }
-        else memset( bits->ptr, 0, info->bmiHeader.biSizeImage );
     }
-done:
     GDI_ReleaseObj( hbitmap );
-    return ERROR_SUCCESS;
+    return ret;
 }
 
 DWORD nulldrv_PutImage( PHYSDEV dev, HBITMAP hbitmap, HRGN clip, BITMAPINFO *info,
@@ -122,73 +88,23 @@ DWORD nulldrv_PutImage( PHYSDEV dev, HBITMAP hbitmap, HRGN clip, BITMAPINFO *inf
     BITMAPOBJ *bmp;
 
     if (!hbitmap) return ERROR_SUCCESS;
-    if (!(bmp = GDI_GetObjPtr( hbitmap, OBJ_BITMAP ))) return ERROR_INVALID_HANDLE;
-
-    if (info->bmiHeader.biPlanes != 1) goto update_format;
-    if (info->bmiHeader.biBitCount != bmp->bitmap.bmBitsPixel) goto update_format;
-    /* FIXME: check color masks */
 
     if (bits)
     {
-        int i, width_bytes = get_dib_stride( info->bmiHeader.biWidth, info->bmiHeader.biBitCount );
-        unsigned char *dst_bits, *src_bits;
-
-        if ((src->width != dst->width) || (src->height != dst->height))
+        if (!(bmp = GDI_GetObjPtr( hbitmap, OBJ_BITMAP ))) return ERROR_INVALID_HANDLE;
+        if (!bmp->bitmap.bmBits)
         {
-            GDI_ReleaseObj( hbitmap );
-            return ERROR_TRANSFORM_NOT_SUPPORTED;
-        }
-        if (src->visrect.left > 0 || src->visrect.right < bmp->bitmap.bmWidth ||
-            dst->visrect.left > 0 || dst->visrect.right < bmp->bitmap.bmWidth)
-        {
-            FIXME( "setting partial rows not supported\n" );
-            GDI_ReleaseObj( hbitmap );
-            return ERROR_NOT_SUPPORTED;
-        }
-        if (clip)
-        {
-            FIXME( "clip region not supported\n" );
-            GDI_ReleaseObj( hbitmap );
-            return ERROR_NOT_SUPPORTED;
-        }
-
-        if (!bmp->bitmap.bmBits &&
-            !(bmp->bitmap.bmBits = HeapAlloc( GetProcessHeap(), 0,
-                                              bmp->bitmap.bmHeight * bmp->bitmap.bmWidthBytes )))
-        {
-            GDI_ReleaseObj( hbitmap );
-            return ERROR_OUTOFMEMORY;
-        }
-        src_bits = bits->ptr;
-        if (info->bmiHeader.biHeight > 0)
-        {
-            src_bits += (info->bmiHeader.biHeight - 1 - src->visrect.top) * width_bytes;
-            width_bytes = -width_bytes;
-        }
-        else
-            src_bits += src->visrect.top * width_bytes;
-
-        dst_bits = (unsigned char *)bmp->bitmap.bmBits + dst->visrect.top * bmp->bitmap.bmWidthBytes;
-        if (width_bytes != bmp->bitmap.bmWidthBytes)
-        {
-            for (i = 0; i < dst->visrect.bottom - dst->visrect.top; i++)
+            int width_bytes = get_dib_stride( bmp->bitmap.bmWidth, bmp->bitmap.bmBitsPixel );
+            if (!(bmp->bitmap.bmBits = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY,
+                                                  bmp->bitmap.bmHeight * width_bytes )))
             {
-                memcpy( dst_bits, src_bits, min( abs(width_bytes), bmp->bitmap.bmWidthBytes ));
-                src_bits += width_bytes;
-                dst_bits += bmp->bitmap.bmWidthBytes;
+                GDI_ReleaseObj( hbitmap );
+                return ERROR_OUTOFMEMORY;
             }
         }
-        else memcpy( dst_bits, src_bits, width_bytes * (dst->visrect.bottom - dst->visrect.top) );
+        GDI_ReleaseObj( hbitmap );
     }
-    GDI_ReleaseObj( hbitmap );
-    return ERROR_SUCCESS;
-
-update_format:
-    info->bmiHeader.biPlanes   = 1;
-    info->bmiHeader.biBitCount = bmp->bitmap.bmBitsPixel;
-    if (info->bmiHeader.biHeight > 0) info->bmiHeader.biHeight = -info->bmiHeader.biHeight;
-    GDI_ReleaseObj( hbitmap );
-    return ERROR_BAD_FORMAT;
+    return dib_driver.pPutImage( NULL, hbitmap, clip, info, bits, src, dst, rop );
 }
 
 
@@ -416,8 +332,8 @@ HBITMAP WINAPI CreateBitmapIndirect( const BITMAP *bmp )
     if (bm.bmBits)
         SetBitmapBits( hbitmap, bm.bmHeight * bm.bmWidthBytes, bm.bmBits );
 
-    TRACE("%dx%d, %d colors returning %p\n", bm.bmWidth, bm.bmHeight,
-          1 << (bm.bmPlanes * bm.bmBitsPixel), hbitmap);
+    TRACE("%dx%d, bpp %d planes %d: returning %p\n", bm.bmWidth, bm.bmHeight,
+          bm.bmBitsPixel, bm.bmPlanes, hbitmap);
 
     return hbitmap;
 }
@@ -691,8 +607,16 @@ HBITMAP BITMAP_CopyBitmap(HBITMAP hbitmap)
 
 static void set_initial_bitmap_bits( HBITMAP hbitmap, BITMAPOBJ *bmp )
 {
+    char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
+    BITMAPINFO *info = (BITMAPINFO *)buffer;
+
     if (!bmp->bitmap.bmBits) return;
-    SetBitmapBits( hbitmap, bmp->bitmap.bmHeight * bmp->bitmap.bmWidthBytes, bmp->bitmap.bmBits );
+    if (bmp->funcs->pPutImage == nulldrv_PutImage) return;
+
+    get_ddb_bitmapinfo( bmp, info );
+    SetDIBits( 0, hbitmap, 0, bmp->bitmap.bmHeight, bmp->bitmap.bmBits, info, DIB_RGB_COLORS );
+    HeapFree( GetProcessHeap(), 0, bmp->bitmap.bmBits );
+    bmp->bitmap.bmBits = NULL;
 }
 
 /***********************************************************************
@@ -778,13 +702,18 @@ static HGDIOBJ BITMAP_SelectObject( HGDIOBJ handle, HDC hdc )
     }
 
     old_physdev = GET_DC_PHYSDEV( dc, pSelectBitmap );
-    if(old_physdev == &dc->dibdrv.dev)
-        pop_dc_driver( dc, old_physdev );
+    if(old_physdev == dc->dibdrv)
+        old_physdev = pop_dc_driver( &dc->physDev );
 
     if(bitmap->dib)
     {
-        physdev = &dc->dibdrv.dev;
-        push_dc_driver( dc, physdev, physdev->funcs );
+        physdev = dc->dibdrv;
+        if (physdev) push_dc_driver( &dc->physDev, physdev, physdev->funcs );
+        else
+        {
+            if (!dib_driver.pCreateDC( &dc->physDev, NULL, NULL, NULL, NULL )) goto done;
+            dc->dibdrv = physdev = dc->physDev;
+        }
     }
     else
         physdev = GET_DC_PHYSDEV( dc, pSelectBitmap );
@@ -818,8 +747,10 @@ static HGDIOBJ BITMAP_SelectObject( HGDIOBJ handle, HDC hdc )
  done:
     if(!ret)
     {
-        if(physdev == &dc->dibdrv.dev) pop_dc_driver( dc, physdev );
-        if(old_physdev == &dc->dibdrv.dev) push_dc_driver( dc, old_physdev, old_physdev->funcs );
+        if (physdev && physdev == dc->dibdrv)
+            pop_dc_driver( &dc->physDev );
+        if (old_physdev && old_physdev == dc->dibdrv)
+            push_dc_driver( &dc->physDev, old_physdev, old_physdev->funcs );
     }
     release_dc_ptr( dc );
     return ret;
