@@ -79,8 +79,7 @@ DC *alloc_dc_ptr( WORD magic )
 
     dc->nulldrv.funcs       = &null_driver;
     dc->nulldrv.next        = NULL;
-    dc->dibdrv.dev.funcs    = &dib_driver;
-    dc->dibdrv.dev.next     = NULL;
+    dc->dibdrv              = NULL;
     dc->physDev             = &dc->nulldrv;
     dc->thread              = GetCurrentThreadId();
     dc->refcount            = 1;
@@ -181,7 +180,13 @@ void free_dc_ptr( DC *dc )
 {
     assert( dc->refcount == 1 );
 
-    while (dc->physDev != &dc->nulldrv) pop_dc_driver( dc, dc->physDev );
+    while (dc->physDev != &dc->nulldrv)
+    {
+        PHYSDEV physdev = pop_dc_driver( &dc->physDev );
+        physdev->funcs->pDeleteDC( physdev );
+        if (physdev == dc->dibdrv) dc->dibdrv = NULL;
+    }
+    if (dc->dibdrv) dc->dibdrv->funcs->pDeleteDC( dc->dibdrv );
     free_gdi_handle( dc->hSelf );
     free_dc_state( dc );
 }
@@ -242,34 +247,6 @@ void update_dc( DC *dc )
 }
 
 
-/***********************************************************************
- *           push_dc_driver
- *
- * Push a driver on top of the DC driver stack.
- */
-void push_dc_driver( DC * dc, PHYSDEV physdev, const DC_FUNCTIONS *funcs )
-{
-    physdev->funcs = funcs;
-    physdev->next = dc->physDev;
-    physdev->hdc = dc->hSelf;
-    dc->physDev = physdev;
-}
-
-
-/***********************************************************************
- *           pop_dc_driver
- *
- * Pop the top driver from the DC driver stack.
- */
-void pop_dc_driver( DC * dc, PHYSDEV physdev )
-{
-    assert( physdev == dc->physDev );
-    assert( physdev != &dc->nulldrv );
-    dc->physDev = physdev->next;
-    physdev->funcs->pDeleteDC( physdev );
-}
-
-
 /***********************************************************************
  *           DC_DeleteObject
  */
@@ -652,13 +629,11 @@ HDC WINAPI CreateDCW( LPCWSTR driver, LPCWSTR device, LPCWSTR output,
 
     if (funcs->pCreateDC)
     {
-        PHYSDEV physdev;
-        if (!funcs->pCreateDC( hdc, &physdev, buf, device, output, initData ))
+        if (!funcs->pCreateDC( &dc->physDev, buf, device, output, initData ))
         {
             WARN("creation aborted by device\n" );
             goto error;
         }
-        push_dc_driver( dc, physdev, funcs );
     }
 
     dc->vis_rect.left   = 0;
@@ -743,7 +718,7 @@ HDC WINAPI CreateCompatibleDC( HDC hdc )
 {
     DC *dc, *origDC;
     HDC ret;
-    const DC_FUNCTIONS *funcs = NULL;
+    const struct gdi_dc_funcs *funcs = &null_driver;
     PHYSDEV physDev = NULL;
 
     GDI_CheckNotLock();
@@ -751,14 +726,11 @@ HDC WINAPI CreateCompatibleDC( HDC hdc )
     if (hdc)
     {
         if (!(origDC = get_dc_ptr( hdc ))) return 0;
-        physDev = GET_DC_PHYSDEV( origDC, pCreateDC );
-        if (physDev != &origDC->nulldrv) funcs = physDev->funcs;
-        else physDev = NULL;
+        physDev = GET_DC_PHYSDEV( origDC, pCreateCompatibleDC );
+        funcs = physDev->funcs;
         release_dc_ptr( origDC );
     }
 
-    if (!funcs && !(funcs = DRIVER_get_display_driver())) return 0;
-
     if (!(dc = alloc_dc_ptr( OBJ_MEMDC ))) goto error;
 
     TRACE("(%p): returning %p\n", hdc, dc->hSelf );
@@ -772,17 +744,10 @@ HDC WINAPI CreateCompatibleDC( HDC hdc )
 
     ret = dc->hSelf;
 
-    /* Pass the driver-specific physical device info into
-     * the new DC. The driver may use this read-only info
-     * while creating the compatible DC. */
-    if (funcs->pCreateDC)
+    if (!funcs->pCreateCompatibleDC( physDev, &dc->physDev ))
     {
-        if (!funcs->pCreateDC( dc->hSelf, &physDev, NULL, NULL, NULL, NULL ))
-        {
-            WARN("creation aborted by device\n");
-            goto error;
-        }
-        push_dc_driver( dc, physDev, funcs );
+        WARN("creation aborted by device\n");
+        goto error;
     }
     DC_InitDC( dc );
     release_dc_ptr( dc );
@@ -1321,6 +1286,7 @@ BOOL WINAPI GetDeviceGammaRamp(HDC hDC, LPVOID ptr)
     BOOL ret = FALSE;
     DC *dc = get_dc_ptr( hDC );
 
+    TRACE("%p, %p\n", hDC, ptr);
     if( dc )
     {
         PHYSDEV physdev = GET_DC_PHYSDEV( dc, pGetDeviceGammaRamp );
@@ -1338,6 +1304,7 @@ BOOL WINAPI SetDeviceGammaRamp(HDC hDC, LPVOID ptr)
     BOOL ret = FALSE;
     DC *dc = get_dc_ptr( hDC );
 
+    TRACE("%p, %p\n", hDC, ptr);
     if( dc )
     {
         PHYSDEV physdev = GET_DC_PHYSDEV( dc, pSetDeviceGammaRamp );
@@ -1352,7 +1319,7 @@ BOOL WINAPI SetDeviceGammaRamp(HDC hDC, LPVOID ptr)
  */
 HCOLORSPACE WINAPI GetColorSpace(HDC hdc)
 {
-/*FIXME    Need to to whatever GetColorSpace actually does */
+/*FIXME    Need to do whatever GetColorSpace actually does */
     return 0;
 }
 
@@ -1399,7 +1366,7 @@ HCOLORSPACE WINAPI SetColorSpace( HDC hDC, HCOLORSPACE hColorSpace )
  */
 UINT WINAPI GetBoundsRect(HDC hdc, LPRECT rect, UINT flags)
 {
-    UINT ret;
+    UINT ret = 0;
     DC *dc = get_dc_ptr( hdc );
 
     if ( !dc ) return 0;
@@ -1407,18 +1374,15 @@ UINT WINAPI GetBoundsRect(HDC hdc, LPRECT rect, UINT flags)
     if (rect)
     {
         *rect = dc->BoundsRect;
-        ret = ((dc->flags & DC_BOUNDS_SET) ? DCB_SET : DCB_RESET);
+        ret = is_rect_empty( rect ) ? DCB_RESET : DCB_SET;
+        DPtoLP( hdc, (POINT *)rect, 2 );
     }
-    else
-        ret = 0;
-
     if (flags & DCB_RESET)
     {
         dc->BoundsRect.left   = 0;
         dc->BoundsRect.top    = 0;
         dc->BoundsRect.right  = 0;
         dc->BoundsRect.bottom = 0;
-        dc->flags &= ~DC_BOUNDS_SET;
     }
     release_dc_ptr( dc );
     return ret;
@@ -1437,7 +1401,7 @@ UINT WINAPI SetBoundsRect(HDC hdc, const RECT* rect, UINT flags)
     if (!(dc = get_dc_ptr( hdc ))) return 0;
 
     ret = ((dc->flags & DC_BOUNDS_ENABLE) ? DCB_ENABLE : DCB_DISABLE) |
-          ((dc->flags & DC_BOUNDS_SET) ? DCB_SET : DCB_RESET);
+           (is_rect_empty( &dc->BoundsRect ) ? DCB_RESET : DCB_SET);
 
     if (flags & DCB_RESET)
     {
@@ -1445,22 +1409,23 @@ UINT WINAPI SetBoundsRect(HDC hdc, const RECT* rect, UINT flags)
         dc->BoundsRect.top    = 0;
         dc->BoundsRect.right  = 0;
         dc->BoundsRect.bottom = 0;
-        dc->flags &= ~DC_BOUNDS_SET;
     }
 
-    if ((flags & DCB_ACCUMULATE) && rect && rect->left < rect->right && rect->top < rect->bottom)
+    if ((flags & DCB_ACCUMULATE) && rect)
     {
-        if (dc->flags & DC_BOUNDS_SET)
+        RECT rc = *rect;
+
+        LPtoDP( hdc, (POINT *)&rc, 2 );
+        if (!is_rect_empty( &rc ))
         {
-            dc->BoundsRect.left   = min( dc->BoundsRect.left, rect->left );
-            dc->BoundsRect.top    = min( dc->BoundsRect.top, rect->top );
-            dc->BoundsRect.right  = max( dc->BoundsRect.right, rect->right );
-            dc->BoundsRect.bottom = max( dc->BoundsRect.bottom, rect->bottom );
-        }
-        else
-        {
-            dc->BoundsRect = *rect;
-            dc->flags |= DC_BOUNDS_SET;
+            if (!is_rect_empty( &dc->BoundsRect))
+            {
+                dc->BoundsRect.left   = min( dc->BoundsRect.left, rc.left );
+                dc->BoundsRect.top    = min( dc->BoundsRect.top, rc.top );
+                dc->BoundsRect.right  = max( dc->BoundsRect.right, rc.right );
+                dc->BoundsRect.bottom = max( dc->BoundsRect.bottom, rc.bottom );
+            }
+            else dc->BoundsRect = rc;
         }
     }
 
