@@ -116,20 +116,38 @@ extern HBRUSH   dibdrv_SelectBrush( PHYSDEV dev, HBRUSH hbrush ) DECLSPEC_HIDDEN
 extern HPEN     dibdrv_SelectPen( PHYSDEV dev, HPEN hpen ) DECLSPEC_HIDDEN;
 extern COLORREF dibdrv_SetDCBrushColor( PHYSDEV dev, COLORREF color ) DECLSPEC_HIDDEN;
 extern COLORREF dibdrv_SetDCPenColor( PHYSDEV dev, COLORREF color ) DECLSPEC_HIDDEN;
+extern BOOL     dibdrv_StretchBlt( PHYSDEV dst_dev, struct bitblt_coords *dst,
+                                   PHYSDEV src_dev, struct bitblt_coords *src, DWORD rop ) DECLSPEC_HIDDEN;
 
 static inline dibdrv_physdev *get_dibdrv_pdev( PHYSDEV dev )
 {
     return (dibdrv_physdev *)dev;
 }
 
+struct stretch_params
+{
+    int err_start, err_add_1, err_add_2;
+    unsigned int length;
+    int dst_inc, src_inc;
+};
+
 typedef struct primitive_funcs
 {
-    void        (* solid_rects)(const dib_info *dib, int num, const RECT *rc, DWORD and, DWORD xor);
-    void      (* pattern_rects)(const dib_info *dib, int num, const RECT *rc, const POINT *orign, const dib_info *brush, void *and_bits, void *xor_bits);
-    void          (* copy_rect)(const dib_info *dst, const RECT *rc, const dib_info *src, const POINT *origin, int rop2, int overlap);
-    DWORD (* colorref_to_pixel)(const dib_info *dib, COLORREF color);
-    BOOL         (* convert_to)(dib_info *dst, const dib_info *src, const RECT *src_rect);
-    BOOL   (* create_rop_masks)(const dib_info *dib, const dib_info *hatch, const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits);
+    void            (* solid_rects)(const dib_info *dib, int num, const RECT *rc, DWORD and, DWORD xor);
+    void          (* pattern_rects)(const dib_info *dib, int num, const RECT *rc, const POINT *orign,
+                                    const dib_info *brush, void *and_bits, void *xor_bits);
+    void              (* copy_rect)(const dib_info *dst, const RECT *rc, const dib_info *src,
+                                    const POINT *origin, int rop2, int overlap);
+    DWORD     (* colorref_to_pixel)(const dib_info *dib, COLORREF color);
+    BOOL             (* convert_to)(dib_info *dst, const dib_info *src, const RECT *src_rect);
+    BOOL       (* create_rop_masks)(const dib_info *dib, const dib_info *hatch,
+                                    const rop_mask *fg, const rop_mask *bg, rop_mask_bits *bits);
+    void            (* stretch_row)(const dib_info *dst_dib, const POINT *dst_start,
+                                    const dib_info *src_dib, const POINT *src_start,
+                                    const struct stretch_params *params, int mode, BOOL keep_dst);
+    void             (* shrink_row)(const dib_info *dst_dib, const POINT *dst_start,
+                                    const dib_info *src_dib, const POINT *src_start,
+                                    const struct stretch_params *params, int mode, BOOL keep_dst);
 } primitive_funcs;
 
 extern const primitive_funcs funcs_8888 DECLSPEC_HIDDEN;
@@ -152,7 +170,14 @@ struct rop_codes
 #define OVERLAP_ABOVE 0x04  /* dest starts above source */
 #define OVERLAP_BELOW 0x08  /* dest starts below source */
 
-extern void get_rop_codes(INT rop, struct rop_codes *codes);
+typedef struct
+{
+    unsigned int dx, dy;
+    int bias;
+    DWORD octant;
+} bres_params;
+
+extern void get_rop_codes(INT rop, struct rop_codes *codes) DECLSPEC_HIDDEN;
 extern void calc_and_xor_masks(INT rop, DWORD color, DWORD *and, DWORD *xor) DECLSPEC_HIDDEN;
 extern void update_brush_rop( dibdrv_physdev *pdev, INT rop ) DECLSPEC_HIDDEN;
 extern void reset_dash_origin(dibdrv_physdev *pdev) DECLSPEC_HIDDEN;
@@ -171,6 +196,8 @@ extern DWORD get_fg_color(dibdrv_physdev *pdev, COLORREF color) DECLSPEC_HIDDEN;
 extern BOOL brush_rects( dibdrv_physdev *pdev, int num, const RECT *rects ) DECLSPEC_HIDDEN;
 extern HRGN add_extra_clipping_region( dibdrv_physdev *pdev, HRGN rgn ) DECLSPEC_HIDDEN;
 extern void restore_clipping_region( dibdrv_physdev *pdev, HRGN rgn ) DECLSPEC_HIDDEN;
+extern int clip_line(const POINT *start, const POINT *end, const RECT *clip,
+                     const bres_params *params, POINT *pt1, POINT *pt2) DECLSPEC_HIDDEN;
 
 static inline BOOL defer_pen(dibdrv_physdev *pdev)
 {
