@@ -255,7 +255,7 @@ extern DWORD convert_bitmapinfo( const BITMAPINFO *src_info, void *src_bits, str
                                  const BITMAPINFO *dst_info, void *dst_bits ) DECLSPEC_HIDDEN;
 
 extern DWORD stretch_bitmapinfo( const BITMAPINFO *src_info, void *src_bits, struct bitblt_coords *src,
-                                 const BITMAPINFO *dst_info, void *dst_bits, const struct bitblt_coords *dst,
+                                 const BITMAPINFO *dst_info, void *dst_bits, struct bitblt_coords *dst,
                                  INT mode ) DECLSPEC_HIDDEN;
 
 /* driver.c */
