@@ -30,6 +30,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(metafile);
 
+static BOOL MFDRV_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev );
 static BOOL MFDRV_DeleteDC( PHYSDEV dev );
 
 
@@ -87,10 +88,12 @@ static const DC_FUNCTIONS MFDRV_Funcs =
     MFDRV_Arc,                       /* pArc */
     NULL,                            /* pArcTo */
     MFDRV_BeginPath,                 /* pBeginPath */
+    NULL,                            /* pBlendImage */
     NULL,                            /* pChoosePixelFormat */
     MFDRV_Chord,                     /* pChord */
     MFDRV_CloseFigure,               /* pCloseFigure */
     NULL,                            /* pCreateBitmap */
+    MFDRV_CreateCompatibleDC,        /* pCreateCompatibleDC */
     NULL,                            /* pCreateDC */
     NULL,                            /* pCreateDIBSection */
     NULL,                            /* pDeleteBitmap */
@@ -224,7 +227,7 @@ static DC *MFDRV_AllocMetaFile(void)
         return NULL;
     }
 
-    push_dc_driver( dc, &physDev->dev, &MFDRV_Funcs );
+    push_dc_driver( &dc->physDev, &physDev->dev, &MFDRV_Funcs );
 
     physDev->handles = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, HANDLE_LIST_INC * sizeof(physDev->handles[0]));
     physDev->handles_size = HANDLE_LIST_INC;
@@ -245,6 +248,16 @@ static DC *MFDRV_AllocMetaFile(void)
 }
 
 
+/**********************************************************************
+ *	     MFDRV_CreateCompatibleDC
+ */
+static BOOL MFDRV_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
+{
+    /* not supported on metafile DCs */
+    return FALSE;
+}
+
+
 /**********************************************************************
  *	     MFDRV_DeleteDC
  */
