@@ -44,13 +44,21 @@ static void testGetModuleFileName(const char* name)
 
     /* first test, with enough space in buffer */
     memset(bufA, '-', sizeof(bufA));
+    SetLastError(0xdeadbeef);
     len1A = GetModuleFileNameA(hMod, bufA, sizeof(bufA));
+    ok(GetLastError() == ERROR_SUCCESS ||
+       broken(GetLastError() == 0xdeadbeef), /* <= XP SP3 */
+       "LastError was not reset: %u\n", GetLastError());
     ok(len1A > 0, "Getting module filename for handle %p\n", hMod);
 
     if (is_unicode_enabled)
     {
         memset(bufW, '-', sizeof(bufW));
+        SetLastError(0xdeadbeef);
         len1W = GetModuleFileNameW(hMod, bufW, sizeof(bufW) / sizeof(WCHAR));
+        ok(GetLastError() == ERROR_SUCCESS ||
+           broken(GetLastError() == 0xdeadbeef), /* <= XP SP3 */
+           "LastError was not reset: %u\n", GetLastError());
         ok(len1W > 0, "Getting module filename for handle %p\n", hMod);
     }
 
@@ -221,6 +229,7 @@ static void testLoadLibraryEx(void)
     CHAR path[MAX_PATH];
     HMODULE hmodule;
     HANDLE hfile;
+    BOOL ret;
 
     hfile = CreateFileA("testfile.dll", GENERIC_READ | GENERIC_WRITE,
                         FILE_SHARE_READ | FILE_SHARE_WRITE,
@@ -228,13 +237,18 @@ static void testLoadLibraryEx(void)
     ok(hfile != INVALID_HANDLE_VALUE, "Expected a valid file handle\n");
 
     /* NULL lpFileName */
-    SetLastError(0xdeadbeef);
-    hmodule = LoadLibraryExA(NULL, NULL, 0);
-    ok(hmodule == 0, "Expected 0, got %p\n", hmodule);
-    ok(GetLastError() == ERROR_MOD_NOT_FOUND ||
-       GetLastError() == ERROR_INVALID_PARAMETER, /* win9x */
-       "Expected ERROR_MOD_NOT_FOUND or ERROR_INVALID_PARAMETER, got %d\n",
-       GetLastError());
+    if (is_unicode_enabled)
+    {
+        SetLastError(0xdeadbeef);
+        hmodule = LoadLibraryExA(NULL, NULL, 0);
+        ok(hmodule == 0, "Expected 0, got %p\n", hmodule);
+        ok(GetLastError() == ERROR_MOD_NOT_FOUND ||
+           GetLastError() == ERROR_INVALID_PARAMETER, /* win9x */
+           "Expected ERROR_MOD_NOT_FOUND or ERROR_INVALID_PARAMETER, got %d\n",
+           GetLastError());
+    }
+    else
+        win_skip("NULL filename crashes on WinMe\n");
 
     /* empty lpFileName */
     SetLastError(0xdeadbeef);
@@ -281,13 +295,16 @@ static void testLoadLibraryEx(void)
     }
 
     /* lpFileName does not matter */
-    SetLastError(0xdeadbeef);
-    hmodule = LoadLibraryExA(NULL, hfile, 0);
-    ok(hmodule == 0, "Expected 0, got %p\n", hmodule);
-    ok(GetLastError() == ERROR_MOD_NOT_FOUND ||
-       GetLastError() == ERROR_INVALID_PARAMETER, /* win2k3 */
-       "Expected ERROR_MOD_NOT_FOUND or ERROR_INVALID_PARAMETER, got %d\n",
-       GetLastError());
+    if (is_unicode_enabled)
+    {
+        SetLastError(0xdeadbeef);
+        hmodule = LoadLibraryExA(NULL, hfile, 0);
+        ok(hmodule == 0, "Expected 0, got %p\n", hmodule);
+        ok(GetLastError() == ERROR_MOD_NOT_FOUND ||
+           GetLastError() == ERROR_INVALID_PARAMETER, /* win2k3 */
+           "Expected ERROR_MOD_NOT_FOUND or ERROR_INVALID_PARAMETER, got %d\n",
+           GetLastError());
+    }
 
     CloseHandle(hfile);
 
@@ -351,6 +368,22 @@ static void testLoadLibraryEx(void)
     ok(GetLastError() == ERROR_FILE_NOT_FOUND ||
        broken(GetLastError() == ERROR_INVALID_HANDLE),  /* nt4 */
        "Expected ERROR_FILE_NOT_FOUND, got %d\n", GetLastError());
+
+    /* Free the loaded dll when its the first time this dll is loaded
+       in process - First time should pass, second fail */
+    SetLastError(0xdeadbeef);
+    hmodule = LoadLibraryExA("comctl32.dll", NULL, LOAD_LIBRARY_AS_DATAFILE);
+    ok(hmodule != 0, "Expected valid module handle\n");
+
+    SetLastError(0xdeadbeef);
+    ret = FreeLibrary(hmodule);
+    ok(ret, "Expected to be able to free the module, failed with %d\n", GetLastError());
+    SetLastError(0xdeadbeef);
+    ret = FreeLibrary(hmodule);
+    ok(!ret, "Unexpected ability to free the module, failed with %d\n", GetLastError());
+
+    CloseHandle(hmodule);
+
 }
 
 START_TEST(module)
