@@ -85,6 +85,8 @@ static const IMMDeviceVtbl MMDeviceVtbl;
 static const IPropertyStoreVtbl MMDevPropVtbl;
 static const IMMEndpointVtbl MMEndpointVtbl;
 
+static IMMDevice info_device;
+
 typedef struct MMDevColImpl
 {
     IMMDeviceCollection IMMDeviceCollection_iface;
@@ -907,21 +909,79 @@ static HRESULT WINAPI MMDevEnum_EnumAudioEndpoints(IMMDeviceEnumerator *iface, E
 static HRESULT WINAPI MMDevEnum_GetDefaultAudioEndpoint(IMMDeviceEnumerator *iface, EDataFlow flow, ERole role, IMMDevice **device)
 {
     MMDevEnumImpl *This = impl_from_IMMDeviceEnumerator(iface);
+    WCHAR reg_key[256];
+    HKEY key;
+    HRESULT hr;
+
+    static const WCHAR slashW[] = {'\\',0};
+    static const WCHAR reg_out_nameW[] = {'D','e','f','a','u','l','t','O','u','t','p','u','t',0};
+    static const WCHAR reg_vout_nameW[] = {'D','e','f','a','u','l','t','V','o','i','c','e','O','u','t','p','u','t',0};
+    static const WCHAR reg_in_nameW[] = {'D','e','f','a','u','l','t','I','n','p','u','t',0};
+    static const WCHAR reg_vin_nameW[] = {'D','e','f','a','u','l','t','V','o','i','c','e','I','n','p','u','t',0};
+
     TRACE("(%p)->(%u,%u,%p)\n", This, flow, role, device);
 
     if (!device)
         return E_POINTER;
+
+    if((flow != eRender && flow != eCapture) ||
+            (role != eConsole && role != eMultimedia && role != eCommunications)){
+        WARN("Unknown flow (%u) or role (%u)\n", flow, role);
+        return E_INVALIDARG;
+    }
+
     *device = NULL;
 
+    if(!drvs.module_name[0])
+        return E_NOTFOUND;
+
+    lstrcpyW(reg_key, drv_keyW);
+    lstrcatW(reg_key, slashW);
+    lstrcatW(reg_key, drvs.module_name);
+
+    if(RegOpenKeyW(HKEY_CURRENT_USER, reg_key, &key) == ERROR_SUCCESS){
+        const WCHAR *reg_x_name, *reg_vx_name;
+        WCHAR def_id[256];
+        DWORD size = sizeof(def_id);
+
+        if(flow == eRender){
+            reg_x_name = reg_out_nameW;
+            reg_vx_name = reg_vout_nameW;
+        }else{
+            reg_x_name = reg_in_nameW;
+            reg_vx_name = reg_vin_nameW;
+        }
+
+        if(role == eCommunications &&
+                RegQueryValueExW(key, reg_vx_name, 0, NULL,
+                    (BYTE*)def_id, &size) == ERROR_SUCCESS){
+            hr = IMMDeviceEnumerator_GetDevice(iface, def_id, device);
+            if(SUCCEEDED(hr)){
+                RegCloseKey(key);
+                return S_OK;
+            }
+
+            TRACE("Unable to find voice device %s\n", wine_dbgstr_w(def_id));
+        }
+
+        if(RegQueryValueExW(key, reg_x_name, 0, NULL,
+                    (BYTE*)def_id, &size) == ERROR_SUCCESS){
+            hr = IMMDeviceEnumerator_GetDevice(iface, def_id, device);
+            if(SUCCEEDED(hr)){
+                RegCloseKey(key);
+                return S_OK;
+            }
+
+            TRACE("Unable to find device %s\n", wine_dbgstr_w(def_id));
+        }
+
+        RegCloseKey(key);
+    }
+
     if (flow == eRender)
         *device = &MMDevice_def_play->IMMDevice_iface;
-    else if (flow == eCapture)
-        *device = &MMDevice_def_rec->IMMDevice_iface;
     else
-    {
-        WARN("Unknown flow %u\n", flow);
-        return E_INVALIDARG;
-    }
+        *device = &MMDevice_def_rec->IMMDevice_iface;
 
     if (!*device)
         return E_NOTFOUND;
@@ -935,10 +995,19 @@ static HRESULT WINAPI MMDevEnum_GetDevice(IMMDeviceEnumerator *iface, const WCHA
     DWORD i=0;
     IMMDevice *dev = NULL;
 
+    static const WCHAR wine_info_deviceW[] = {'W','i','n','e',' ',
+        'i','n','f','o',' ','d','e','v','i','c','e',0};
+
+    TRACE("(%p)->(%s,%p)\n", This, debugstr_w(name), device);
+
     if(!name || !device)
         return E_POINTER;
 
-    TRACE("(%p)->(%s,%p)\n", This, debugstr_w(name), device);
+    if(!lstrcmpW(name, wine_info_deviceW)){
+        *device = &info_device;
+        return S_OK;
+    }
+
     for (i = 0; i < MMDevice_count; ++i)
     {
         WCHAR *str;
@@ -1211,3 +1280,83 @@ static const IPropertyBagVtbl PB_Vtbl =
     PB_Read,
     PB_Write
 };
+
+static ULONG WINAPI info_device_ps_AddRef(IPropertyStore *iface)
+{
+    return 2;
+}
+
+static ULONG WINAPI info_device_ps_Release(IPropertyStore *iface)
+{
+    return 1;
+}
+
+static HRESULT WINAPI info_device_ps_GetValue(IPropertyStore *iface,
+        REFPROPERTYKEY key, PROPVARIANT *pv)
+{
+    TRACE("(static)->(\"%s,%u\", %p)\n", debugstr_guid(&key->fmtid), key ? key->pid : 0, pv);
+
+    if (!key || !pv)
+        return E_POINTER;
+
+    if (IsEqualPropertyKey(*key, DEVPKEY_Device_Driver))
+    {
+        INT size = (lstrlenW(drvs.module_name) + 1) * sizeof(WCHAR);
+        pv->vt = VT_LPWSTR;
+        pv->u.pwszVal = CoTaskMemAlloc(size);
+        if (!pv->u.pwszVal)
+            return E_OUTOFMEMORY;
+        memcpy(pv->u.pwszVal, drvs.module_name, size);
+        return S_OK;
+    }
+
+    return E_INVALIDARG;
+}
+
+static const IPropertyStoreVtbl info_device_ps_Vtbl =
+{
+    NULL,
+    info_device_ps_AddRef,
+    info_device_ps_Release,
+    NULL,
+    NULL,
+    info_device_ps_GetValue,
+    NULL,
+    NULL
+};
+
+static IPropertyStore info_device_ps = {
+    &info_device_ps_Vtbl
+};
+
+static ULONG WINAPI info_device_AddRef(IMMDevice *iface)
+{
+    return 2;
+}
+
+static ULONG WINAPI info_device_Release(IMMDevice *iface)
+{
+    return 1;
+}
+
+static HRESULT WINAPI info_device_OpenPropertyStore(IMMDevice *iface,
+        DWORD access, IPropertyStore **ppv)
+{
+    *ppv = &info_device_ps;
+    return S_OK;
+}
+
+static const IMMDeviceVtbl info_device_Vtbl =
+{
+    NULL,
+    info_device_AddRef,
+    info_device_Release,
+    NULL,
+    info_device_OpenPropertyStore,
+    NULL,
+    NULL
+};
+
+static IMMDevice info_device = {
+    &info_device_Vtbl
+};
