@@ -211,6 +211,56 @@ static void test_audioclient(void)
     hr = IAudioClient_Initialize(ac, AUDCLNT_SHAREMODE_SHARED, 0, 5000000, 0, NULL, NULL);
     ok(hr == E_POINTER, "Initialize with null format returns %08x\n", hr);
 
+    hr = IAudioClient_Initialize(ac, AUDCLNT_SHAREMODE_SHARED, 0, 0, 0, pwfx, NULL);
+    ok(hr == S_OK, "Initialize with 0 buffer size returns %08x\n", hr);
+
+    IAudioClient_Release(ac);
+
+    hr = IMMDevice_Activate(dev, &IID_IAudioClient, CLSCTX_INPROC_SERVER,
+            NULL, (void**)&ac);
+    ok(hr == S_OK, "Activation failed with %08x\n", hr);
+
+    if(pwfx->wFormatTag == WAVE_FORMAT_EXTENSIBLE){
+        WAVEFORMATEXTENSIBLE *fmtex = (WAVEFORMATEXTENSIBLE*)pwfx;
+        WAVEFORMATEX *fmt2 = NULL;
+
+        ok(fmtex->dwChannelMask != 0, "Got empty dwChannelMask\n");
+
+        fmtex->dwChannelMask = 0xffff;
+
+        hr = IAudioClient_Initialize(ac, AUDCLNT_SHAREMODE_SHARED, 0, 5000000, 0, pwfx, NULL);
+        ok(hr == S_OK, "Initialize(dwChannelMask = 0xffff) returns %08x\n", hr);
+
+        IAudioClient_Release(ac);
+
+        hr = IMMDevice_Activate(dev, &IID_IAudioClient, CLSCTX_INPROC_SERVER,
+                NULL, (void**)&ac);
+        ok(hr == S_OK, "Activation failed with %08x\n", hr);
+
+        fmtex->dwChannelMask = 0;
+
+        hr = IAudioClient_IsFormatSupported(ac, AUDCLNT_SHAREMODE_SHARED, pwfx, &fmt2);
+        ok(hr == S_OK, "IsFormatSupported(dwChannelMask = 0) call returns %08x\n", hr);
+        ok(fmtex->dwChannelMask == 0, "Passed format was modified\n");
+
+        CoTaskMemFree(fmt2);
+
+        hr = IAudioClient_Initialize(ac, AUDCLNT_SHAREMODE_SHARED, 0, 5000000, 0, pwfx, NULL);
+        ok(hr == S_OK, "Initialize(dwChannelMask = 0) returns %08x\n", hr);
+
+        IAudioClient_Release(ac);
+
+        hr = IMMDevice_Activate(dev, &IID_IAudioClient, CLSCTX_INPROC_SERVER,
+                NULL, (void**)&ac);
+        ok(hr == S_OK, "Activation failed with %08x\n", hr);
+
+        CoTaskMemFree(pwfx);
+
+        hr = IAudioClient_GetMixFormat(ac, &pwfx);
+        ok(hr == S_OK, "Valid GetMixFormat returns %08x\n", hr);
+    }else
+        skip("Skipping dwChannelMask tests\n");
+
     hr = IAudioClient_Initialize(ac, AUDCLNT_SHAREMODE_SHARED, 0, 5000000, 0, pwfx, NULL);
     ok(hr == S_OK, "Valid Initialize returns %08x\n", hr);
 
@@ -562,7 +612,6 @@ static void test_clock(void)
     ok(hr == S_OK, "GetPosition failed: %08x\n", hr);
     ok(pos == 0, "GetPosition returned non-zero pos before being started\n");
     ok(pcpos != 0, "GetPosition returned zero pcpos\n");
-    last = pos;
 
     hr = IAudioClient_GetService(ac, &IID_IAudioRenderClient, (void**)&arc);
     ok(hr == S_OK, "GetService(IAudioRenderClient) failed: %08x\n", hr);
@@ -618,7 +667,6 @@ static void test_clock(void)
     hr = IAudioClock_GetPosition(acl, &pos, NULL);
     ok(hr == S_OK, "GetPosition failed: %08x\n", hr);
     ok(pos == 0, "GetPosition returned non-zero pos after Reset\n");
-    last = pos;
 
     hr = IAudioRenderClient_GetBuffer(arc, pwfx->nSamplesPerSec / 2., &data);
     ok(hr == S_OK, "GetBuffer failed: %08x\n", hr);
