@@ -213,7 +213,7 @@ static LCID MSVCRT_locale_to_LCID(const char *locale)
 {
     LCID lcid;
     locale_search_t search;
-    char *cp, *region;
+    const char *cp, *region;
 
     memset(&search, 0, sizeof(locale_search_t));
 
@@ -676,7 +676,7 @@ MSVCRT__locale_t MSVCRT__create_locale(int category, const char *locale)
     else if(!locale[0])
         lcid[0] = GetSystemDefaultLCID();
     else if (locale[0] == 'L' && locale[1] == 'C' && locale[2] == '_') {
-        char *p;
+        const char *p;
 
         while(1) {
             locale += 3; /* LC_ */
@@ -769,7 +769,7 @@ MSVCRT__locale_t MSVCRT__create_locale(int category, const char *locale)
             return NULL;
         }
     } else
-        loc->locinfo->lc_category[MSVCRT_LC_COLLATE].locale = _strdup("C");
+        loc->locinfo->lc_category[MSVCRT_LC_COLLATE].locale = MSVCRT__strdup("C");
 
     if(lcid[MSVCRT_LC_CTYPE] && (category==MSVCRT_LC_ALL || category==MSVCRT_LC_CTYPE)) {
         CPINFO cp;
@@ -815,7 +815,7 @@ MSVCRT__locale_t MSVCRT__create_locale(int category, const char *locale)
         loc->locinfo->lc_clike = 1;
         loc->locinfo->mb_cur_max = 1;
         loc->locinfo->pctype = MSVCRT__ctype+1;
-        loc->locinfo->lc_category[MSVCRT_LC_CTYPE].locale = _strdup("C");
+        loc->locinfo->lc_category[MSVCRT_LC_CTYPE].locale = MSVCRT__strdup("C");
     }
 
     for(i=0; i<256; i++) {
@@ -826,10 +826,17 @@ MSVCRT__locale_t MSVCRT__create_locale(int category, const char *locale)
 
     }
 
-    LCMapStringA(lcid[MSVCRT_LC_CTYPE], LCMAP_LOWERCASE, buf, 256,
-            (char*)loc->locinfo->pclmap, 256);
-    LCMapStringA(lcid[MSVCRT_LC_CTYPE], LCMAP_UPPERCASE, buf, 256,
-            (char*)loc->locinfo->pcumap, 256);
+    if(lcid[MSVCRT_LC_CTYPE]) {
+        LCMapStringA(lcid[MSVCRT_LC_CTYPE], LCMAP_LOWERCASE, buf, 256,
+                (char*)loc->locinfo->pclmap, 256);
+        LCMapStringA(lcid[MSVCRT_LC_CTYPE], LCMAP_UPPERCASE, buf, 256,
+                (char*)loc->locinfo->pcumap, 256);
+    } else {
+        for(i=0; i<256; i++) {
+            loc->locinfo->pclmap[i] = (i>='A' && i<='Z' ? i-'A'+'a' : i);
+            loc->locinfo->pcumap[i] = (i>='a' && i<='z' ? i-'a'+'A' : i);
+        }
+    }
 
     loc->mbcinfo->refcount = 1;
     loc->mbcinfo->mbcodepage = loc->locinfo->lc_id[MSVCRT_LC_CTYPE].wCodePage;
@@ -1025,7 +1032,7 @@ MSVCRT__locale_t MSVCRT__create_locale(int category, const char *locale)
         loc->locinfo->lconv->p_sign_posn = 127;
         loc->locinfo->lconv->n_sign_posn = 127;
 
-        loc->locinfo->lc_category[MSVCRT_LC_MONETARY].locale = _strdup("C");
+        loc->locinfo->lc_category[MSVCRT_LC_MONETARY].locale = MSVCRT__strdup("C");
     }
 
     if(lcid[MSVCRT_LC_NUMERIC] && (category==MSVCRT_LC_ALL || category==MSVCRT_LC_NUMERIC)) {
@@ -1092,7 +1099,7 @@ MSVCRT__locale_t MSVCRT__create_locale(int category, const char *locale)
         loc->locinfo->lconv->thousands_sep[0] = '\0';
         loc->locinfo->lconv->grouping[0] = '\0';
 
-        loc->locinfo->lc_category[MSVCRT_LC_NUMERIC].locale = _strdup("C");
+        loc->locinfo->lc_category[MSVCRT_LC_NUMERIC].locale = MSVCRT__strdup("C");
     }
 
     if(lcid[MSVCRT_LC_TIME] && (category==MSVCRT_LC_ALL || category==MSVCRT_LC_TIME)) {
@@ -1101,7 +1108,7 @@ MSVCRT__locale_t MSVCRT__create_locale(int category, const char *locale)
             return NULL;
         }
     } else
-        loc->locinfo->lc_category[MSVCRT_LC_TIME].locale = _strdup("C");
+        loc->locinfo->lc_category[MSVCRT_LC_TIME].locale = MSVCRT__strdup("C");
 
     return loc;
 }
@@ -1144,6 +1151,7 @@ char* CDECL MSVCRT_setlocale(int category, const char* locale)
 
             if(category != MSVCRT_LC_ALL)
                 break;
+            /* fall through */
         case MSVCRT_LC_CTYPE:
             locinfo->lc_handle[MSVCRT_LC_CTYPE] =
                 loc->locinfo->lc_handle[MSVCRT_LC_CTYPE];
@@ -1168,6 +1176,7 @@ char* CDECL MSVCRT_setlocale(int category, const char* locale)
 
             if(category != MSVCRT_LC_ALL)
                 break;
+            /* fall through */
         case MSVCRT_LC_MONETARY:
             locinfo->lc_handle[MSVCRT_LC_MONETARY] =
                 loc->locinfo->lc_handle[MSVCRT_LC_MONETARY];
@@ -1201,6 +1210,7 @@ char* CDECL MSVCRT_setlocale(int category, const char* locale)
 
             if(category != MSVCRT_LC_ALL)
                 break;
+            /* fall through */
         case MSVCRT_LC_NUMERIC:
             locinfo->lc_handle[MSVCRT_LC_NUMERIC] =
                 loc->locinfo->lc_handle[MSVCRT_LC_NUMERIC];
@@ -1218,6 +1228,7 @@ char* CDECL MSVCRT_setlocale(int category, const char* locale)
 
             if(category != MSVCRT_LC_ALL)
                 break;
+            /* fall through */
         case MSVCRT_LC_TIME:
             locinfo->lc_handle[MSVCRT_LC_TIME] =
                 loc->locinfo->lc_handle[MSVCRT_LC_TIME];
