@@ -40,7 +40,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
  *		_mbsdup (MSVCRT.@)
  *		_strdup (MSVCRT.@)
  */
-char* CDECL _strdup(const char* str)
+char* CDECL MSVCRT__strdup(const char* str)
 {
     if(str)
     {
@@ -79,7 +79,7 @@ int CDECL _strlwr_s_l(char *str, MSVCRT_size_t len, MSVCRT__locale_t locale)
 
     while (*str)
     {
-        *str = MSVCRT__tolower_l(*str, locale);
+        *str = MSVCRT__tolower_l((unsigned char)*str, locale);
         str++;
     }
 
@@ -106,7 +106,7 @@ char* CDECL _strlwr_l(char *str, MSVCRT__locale_t locale)
 /*********************************************************************
  *		_strlwr (MSVCRT.@)
  */
-char* CDECL _strlwr(char *str)
+char* CDECL MSVCRT__strlwr(char *str)
 {
     _strlwr_s_l(str, -1, NULL);
     return str;
@@ -140,7 +140,7 @@ int CDECL _strupr_s_l(char *str, MSVCRT_size_t len, MSVCRT__locale_t locale)
 
     while (*str)
     {
-        *str = MSVCRT__toupper_l(*str, locale);
+        *str = MSVCRT__toupper_l((unsigned char)*str, locale);
         str++;
     }
 
@@ -158,7 +158,7 @@ int CDECL _strupr_s(char *str, MSVCRT_size_t len)
 /*********************************************************************
  *              _strupr_l (MSVCRT.@)
  */
-char* CDECL _strupr_l(char *str, MSVCRT__locale_t locale)
+char* CDECL MSVCRT__strupr_l(char *str, MSVCRT__locale_t locale)
 {
     _strupr_s_l(str, -1, locale);
     return str;
@@ -167,7 +167,7 @@ char* CDECL _strupr_l(char *str, MSVCRT__locale_t locale)
 /*********************************************************************
  *              _strupr (MSVCRT.@)
  */
-char* CDECL _strupr(char *str)
+char* CDECL MSVCRT__strupr(char *str)
 {
     _strupr_s_l(str, -1, NULL);
     return str;
@@ -187,7 +187,7 @@ char* CDECL MSVCRT__strnset(char* str, int value, MSVCRT_size_t len)
 /*********************************************************************
  *		_strrev (MSVCRT.@)
  */
-char* CDECL _strrev(char* str)
+char* CDECL MSVCRT__strrev(char* str)
 {
   char * p1;
   char * p2;
@@ -842,7 +842,7 @@ __int64 CDECL MSVCRT_strtoi64_l(const char *nptr, char **endptr, int base, MSVCR
     BOOL negative = FALSE;
     __int64 ret = 0;
 
-    TRACE("(%s %p %d %p)\n", nptr, endptr, base, locale);
+    TRACE("(%s %p %d %p)\n", debugstr_a(nptr), endptr, base, locale);
 
     if (!MSVCRT_CHECK_PMT(nptr != NULL) || !MSVCRT_CHECK_PMT(base == 0 || base >= 2) ||
         !MSVCRT_CHECK_PMT(base <= 36)) {
@@ -922,7 +922,7 @@ unsigned __int64 CDECL MSVCRT_strtoui64_l(const char *nptr, char **endptr, int b
     BOOL negative = FALSE;
     unsigned __int64 ret = 0;
 
-    TRACE("(%s %p %d %p)\n", nptr, endptr, base, locale);
+    TRACE("(%s %p %d %p)\n", debugstr_a(nptr), endptr, base, locale);
 
     if (!MSVCRT_CHECK_PMT(nptr != NULL) || !MSVCRT_CHECK_PMT(base == 0 || base >= 2) ||
         !MSVCRT_CHECK_PMT(base <= 36)) {
