@@ -1958,6 +1958,7 @@ static HRESULT WINAPI domdoc_createNode(
     case NODE_ENTITY_REFERENCE:
     case NODE_PROCESSING_INSTRUCTION:
         if (!name || *name == 0) return E_FAIL;
+        break;
     default:
         break;
     }
@@ -2760,7 +2761,7 @@ static HRESULT WINAPI domdoc_validateNode(
     {
 
         hr = SchemaCache_validate_tree(This->properties->schemaCache, get_node_obj(node)->node);
-        if (!FAILED(hr))
+        if (SUCCEEDED(hr))
         {
             ++validated;
             /* TODO: get a real error code here */
@@ -3430,7 +3431,8 @@ static HRESULT WINAPI domdoc_Safety_SetInterfaceSafetyOptions(IObjectSafety *ifa
     if ((mask & ~SAFETY_SUPPORTED_OPTIONS) != 0)
         return E_FAIL;
 
-    This->safeopt = enabled & mask & SAFETY_SUPPORTED_OPTIONS;
+    This->safeopt = (This->safeopt & ~mask) | (mask & enabled);
+
     return S_OK;
 }
 
