@@ -49,6 +49,25 @@ WINE_DEFAULT_DEBUG_CHANNEL(msxml);
 
 #ifdef HAVE_LIBXML2
 
+enum ReaderFeatures
+{
+    ExhaustiveErrors             = 1 << 1,
+    ExternalGeneralEntities      = 1 << 2,
+    ExternalParameterEntities    = 1 << 3,
+    ForcedResync                 = 1 << 4,
+    NamespacePrefixes            = 1 << 5,
+    Namespaces                   = 1 << 6,
+    ParameterEntities            = 1 << 7,
+    PreserveSystemIndentifiers   = 1 << 8,
+    ProhibitDTD                  = 1 << 9,
+    SchemaValidation             = 1 << 10,
+    ServerHttpRequest            = 1 << 11,
+    SuppressValidationfatalError = 1 << 12,
+    UseInlineSchema              = 1 << 13,
+    UseSchemaLocation            = 1 << 14,
+    LexicalHandlerParEntities    = 1 << 15
+};
+
 struct bstrpool
 {
     BSTR *pool;
@@ -72,6 +91,7 @@ typedef struct _saxreader
     xmlSAXHandler sax;
     BOOL isParsing;
     struct bstrpool pool;
+    enum ReaderFeatures features;
 } saxreader;
 
 typedef struct _saxlocator
@@ -180,6 +200,48 @@ static const WCHAR PropertyXMLDeclVersionW[] = {
     'x','m','l','d','e','c','l','-','v','e','r','s','i','o','n',0
 };
 
+/* feature names */
+static const WCHAR FeatureExternalGeneralEntitiesW[] = {
+    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/',
+    'f','e','a','t','u','r','e','s','/','e','x','t','e','r','n','a','l','-','g','e','n','e','r','a','l',
+    '-','e','n','t','i','t','i','e','s',0
+};
+
+static const WCHAR FeatureExternalParameterEntitiesW[] = {
+    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/','f','e','a','t','u','r','e','s',
+    '/','e','x','t','e','r','n','a','l','-','p','a','r','a','m','e','t','e','r','-','e','n','t','i','t','i','e','s',0
+};
+
+static const WCHAR FeatureLexicalHandlerParEntitiesW[] = {
+    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/','f','e','a','t','u','r','e','s',
+    '/','l','e','x','i','c','a','l','-','h','a','n','d','l','e','r','/','p','a','r','a','m','e','t','e','r','-','e','n','t','i','t','i','e','s',0
+};
+
+static const WCHAR FeatureProhibitDTDW[] = {
+    'p','r','o','h','i','b','i','t','-','d','t','d',0
+};
+
+static const WCHAR FeatureNamespacesW[] = {
+    'h','t','t','p',':','/','/','x','m','l','.','o','r','g','/','s','a','x','/','f','e','a','t','u','r','e','s',
+    '/','n','a','m','e','s','p','a','c','e','s',0
+};
+
+static inline HRESULT set_feature_value(saxreader *reader, enum ReaderFeatures feature, VARIANT_BOOL value)
+{
+    if (value == VARIANT_TRUE)
+        reader->features |=  feature;
+    else
+        reader->features &= ~feature;
+
+    return S_OK;
+}
+
+static inline HRESULT get_feature_value(const saxreader *reader, enum ReaderFeatures feature, VARIANT_BOOL *value)
+{
+    *value = reader->features & feature ? VARIANT_TRUE : VARIANT_FALSE;
+    return S_OK;
+}
+
 static inline BOOL has_content_handler(const saxlocator *locator)
 {
     return  (locator->vbInterface && locator->saxreader->vbcontentHandler) ||
@@ -762,18 +824,20 @@ static HRESULT WINAPI isaxattributes_getLength(
 
 static HRESULT WINAPI isaxattributes_getURI(
         ISAXAttributes* iface,
-        int nIndex,
-        const WCHAR **pUrl,
-        int *pUriSize)
+        int index,
+        const WCHAR **url,
+        int *size)
 {
     saxattributes *This = impl_from_ISAXAttributes( iface );
-    TRACE("(%p)->(%d)\n", This, nIndex);
+    TRACE("(%p)->(%d)\n", This, index);
 
-    if(nIndex>=This->nb_attributes || nIndex<0) return E_INVALIDARG;
-    if(!pUrl || !pUriSize) return E_POINTER;
+    if(index >= This->nb_attributes || index < 0) return E_INVALIDARG;
+    if(!url || !size) return E_POINTER;
 
-    *pUriSize = SysStringLen(This->szURI[nIndex]);
-    *pUrl = This->szURI[nIndex];
+    *size = SysStringLen(This->szURI[index]);
+    *url = This->szURI[index];
+
+    TRACE("(%s:%d)\n", debugstr_w(This->szURI[index]), *size);
 
     return S_OK;
 }
@@ -816,27 +880,29 @@ static HRESULT WINAPI isaxattributes_getQName(
 
 static HRESULT WINAPI isaxattributes_getName(
         ISAXAttributes* iface,
-        int nIndex,
-        const WCHAR **pUri,
+        int index,
+        const WCHAR **uri,
         int *pUriLength,
-        const WCHAR **pLocalName,
+        const WCHAR **localName,
         int *pLocalNameSize,
-        const WCHAR **pQName,
+        const WCHAR **QName,
         int *pQNameLength)
 {
     saxattributes *This = impl_from_ISAXAttributes( iface );
-    TRACE("(%p)->(%d)\n", This, nIndex);
+    TRACE("(%p)->(%d)\n", This, index);
 
-    if(nIndex>=This->nb_attributes || nIndex<0) return E_INVALIDARG;
-    if(!pUri || !pUriLength || !pLocalName || !pLocalNameSize
-            || !pQName || !pQNameLength) return E_POINTER;
+    if(index>=This->nb_attributes || index<0) return E_INVALIDARG;
+    if(!uri || !pUriLength || !localName || !pLocalNameSize
+            || !QName || !pQNameLength) return E_POINTER;
 
-    *pUriLength = SysStringLen(This->szURI[nIndex]);
-    *pUri = This->szURI[nIndex];
-    *pLocalNameSize = SysStringLen(This->szLocalname[nIndex]);
-    *pLocalName = This->szLocalname[nIndex];
-    *pQNameLength = SysStringLen(This->szQName[nIndex]);
-    *pQName = This->szQName[nIndex];
+    *pUriLength = SysStringLen(This->szURI[index]);
+    *uri = This->szURI[index];
+    *pLocalNameSize = SysStringLen(This->szLocalname[index]);
+    *localName = This->szLocalname[index];
+    *pQNameLength = SysStringLen(This->szQName[index]);
+    *QName = This->szQName[index];
+
+    TRACE("(%s, %s, %s)\n", debugstr_w(*uri), debugstr_w(*localName), debugstr_w(*QName));
 
     return S_OK;
 }
@@ -943,18 +1009,20 @@ static HRESULT WINAPI isaxattributes_getTypeFromQName(
 
 static HRESULT WINAPI isaxattributes_getValue(
         ISAXAttributes* iface,
-        int nIndex,
-        const WCHAR **pValue,
+        int index,
+        const WCHAR **value,
         int *nValue)
 {
     saxattributes *This = impl_from_ISAXAttributes( iface );
-    TRACE("(%p)->(%d)\n", This, nIndex);
+    TRACE("(%p)->(%d)\n", This, index);
 
-    if(nIndex>=This->nb_attributes || nIndex<0) return E_INVALIDARG;
-    if(!pValue || !nValue) return E_POINTER;
+    if(index>=This->nb_attributes || index<0) return E_INVALIDARG;
+    if(!value || !nValue) return E_POINTER;
 
-    *nValue = SysStringLen(This->szValue[nIndex]);
-    *pValue = This->szValue[nIndex];
+    *nValue = SysStringLen(This->szValue[index]);
+    *value = This->szValue[index];
+
+    TRACE("(%s:%d)\n", debugstr_w(*value), *nValue);
 
     return S_OK;
 }
@@ -2332,7 +2400,21 @@ static HRESULT internal_putProperty(
         return S_OK;
     }
 
-    FIXME("(%p)->(%s): unsupported property\n", This, debugstr_w(prop));
+    if(!memcmp(prop, PropertyMaxXMLSizeW, sizeof(PropertyMaxXMLSizeW)))
+    {
+        if (V_VT(&value) == VT_I4 && V_I4(&value) == 0) return S_OK;
+        FIXME("(%p)->(%s): max-xml-size unsupported\n", This, debugstr_variant(&value));
+        return E_NOTIMPL;
+    }
+
+    if(!memcmp(prop, PropertyMaxElementDepthW, sizeof(PropertyMaxElementDepthW)))
+    {
+        if (V_VT(&value) == VT_I4 && V_I4(&value) == 0) return S_OK;
+        FIXME("(%p)->(%s): max-element-depth unsupported\n", This, debugstr_variant(&value));
+        return E_NOTIMPL;
+    }
+
+    FIXME("(%p)->(%s:%s): unsupported property\n", This, debugstr_w(prop), debugstr_variant(&value));
 
     if(!memcmp(prop, PropertyCharsetW, sizeof(PropertyCharsetW)))
         return E_NOTIMPL;
@@ -2343,12 +2425,6 @@ static HRESULT internal_putProperty(
     if(!memcmp(prop, PropertyInputSourceW, sizeof(PropertyInputSourceW)))
         return E_NOTIMPL;
 
-    if(!memcmp(prop, PropertyMaxElementDepthW, sizeof(PropertyMaxElementDepthW)))
-        return E_NOTIMPL;
-
-    if(!memcmp(prop, PropertyMaxXMLSizeW, sizeof(PropertyMaxXMLSizeW)))
-        return E_NOTIMPL;
-
     if(!memcmp(prop, PropertySchemaDeclHandlerW, sizeof(PropertySchemaDeclHandlerW)))
         return E_NOTIMPL;
 
@@ -2557,23 +2633,49 @@ static HRESULT WINAPI saxxmlreader_Invoke(
 /*** IVBSAXXMLReader methods ***/
 static HRESULT WINAPI saxxmlreader_getFeature(
     IVBSAXXMLReader* iface,
-    const WCHAR *pFeature,
-    VARIANT_BOOL *pValue)
+    const WCHAR *feature,
+    VARIANT_BOOL *value)
 {
     saxreader *This = impl_from_IVBSAXXMLReader( iface );
 
-    FIXME("(%p)->(%s %p) stub\n", This, debugstr_w(pFeature), pValue);
+    if (!strcmpW(FeatureNamespacesW, feature))
+        return get_feature_value(This, Namespaces, value);
+
+    FIXME("(%p)->(%s %p) stub\n", This, debugstr_w(feature), value);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI saxxmlreader_putFeature(
     IVBSAXXMLReader* iface,
-    const WCHAR *pFeature,
-    VARIANT_BOOL vfValue)
+    const WCHAR *feature,
+    VARIANT_BOOL value)
 {
     saxreader *This = impl_from_IVBSAXXMLReader( iface );
 
-    FIXME("(%p)->(%s %x) stub\n", This, debugstr_w(pFeature), vfValue);
+    TRACE("(%p)->(%s %x)\n", This, debugstr_w(feature), value);
+
+    if (!strcmpW(FeatureExternalGeneralEntitiesW, feature) && value == VARIANT_FALSE)
+        return set_feature_value(This, ExternalGeneralEntities, value);
+
+    if (!strcmpW(FeatureExternalParameterEntitiesW, feature) && value == VARIANT_FALSE)
+        return set_feature_value(This, ExternalParameterEntities, value);
+
+    if (!strcmpW(FeatureLexicalHandlerParEntitiesW, feature))
+    {
+        FIXME("(%p)->(%s %x) stub\n", This, debugstr_w(feature), value);
+        return set_feature_value(This, LexicalHandlerParEntities, value);
+    }
+
+    if (!strcmpW(FeatureProhibitDTDW, feature))
+    {
+        FIXME("(%p)->(%s %x) stub\n", This, debugstr_w(feature), value);
+        return set_feature_value(This, ProhibitDTD, value);
+    }
+
+    if (!strcmpW(FeatureNamespacesW, feature) && value == VARIANT_TRUE)
+        return set_feature_value(This, Namespaces, value);
+
+    FIXME("(%p)->(%s %x) stub\n", This, debugstr_w(feature), value);
     return E_NOTIMPL;
 }
 
@@ -2964,6 +3066,7 @@ HRESULT SAXXMLReader_create(IUnknown *pUnkOuter, LPVOID *ppObj)
     reader->pool.pool = NULL;
     reader->pool.index = 0;
     reader->pool.len = 0;
+    reader->features = Namespaces;
 
     memset(&reader->sax, 0, sizeof(xmlSAXHandler));
     reader->sax.initialized = XML_SAX2_MAGIC;
