@@ -217,6 +217,14 @@ static const FMTDATERES VarFormat_date_results[] =
   { 40531.0, "ddd", "Sun" }
 };
 
+/* The following tests require that the time separator is a colon (:) */
+static const FMTDATERES VarFormat_namedtime_results[] =
+{
+  { 2.525, "short time", "12:36" },
+  { 2.525, "medium time", "12:36 PM" },
+  { 2.525, "long time", "12:36:00 PM" }
+};
+
 #define VNUMFMT(vt,v) \
   for (i = 0; i < sizeof(VarFormat_results)/sizeof(FMTRES); i++) \
   { \
@@ -247,7 +255,7 @@ static void test_VarFormat(void)
 
   if (PRIMARYLANGID(LANGIDFROMLCID(GetUserDefaultLCID())) != LANG_ENGLISH)
   {
-    skip("Skipping VarFormat tests for non english language\n");
+    skip("Skipping VarFormat tests for non English language\n");
     return;
   }
   GetLocaleInfoA(LOCALE_USER_DEFAULT, LOCALE_SDECIMAL, buff, sizeof(buff)/sizeof(char));
@@ -301,6 +309,23 @@ static void test_VarFormat(void)
            VarFormat_date_results[i].res);
   }
 
+  /* Named time formats */
+  GetLocaleInfoA(LOCALE_USER_DEFAULT, LOCALE_STIMEFORMAT, buff, sizeof(buff)/sizeof(char));
+  if (strcmp(buff, "h:mm:ss tt"))
+  {
+    skip("Skipping named time tests as time format is '%s'\n", buff);
+  }
+  else
+  {
+    for (i = 0; i < sizeof(VarFormat_namedtime_results)/sizeof(FMTDATERES); i++)
+    {
+      fd = 0;
+      VARFMT(VT_DATE,V_DATE,VarFormat_namedtime_results[i].val,
+             VarFormat_namedtime_results[i].fmt,S_OK,
+             VarFormat_namedtime_results[i].res);
+    }
+  }
+
   /* Strings */
   bstrin = SysAllocString(szTesting);
   VARFMT(VT_BSTR,V_BSTR,bstrin,"",S_OK,"testing");
