@@ -155,7 +155,7 @@ static inline IShellViewImpl *impl_from_IShellFolderView(IShellFolderView *iface
     return CONTAINING_RECORD(iface, IShellViewImpl, IShellFolderView_iface);
 }
 
-/* ListView Header ID's */
+/* ListView Header IDs */
 #define LISTVIEW_COLUMN_NAME 0
 #define LISTVIEW_COLUMN_SIZE 1
 #define LISTVIEW_COLUMN_TYPE 2
@@ -728,15 +728,31 @@ static LRESULT ShellView_OnCreate(IShellViewImpl *This)
     hr = IShellFolder_QueryInterface(This->pSFParent, &IID_IPersistFolder2, (LPVOID*)&ppf2);
     if (hr == S_OK)
     {
+        LPITEMIDLIST raw_pidl;
         SHChangeNotifyEntry ntreg;
 
-        hr = IPersistFolder2_GetCurFolder(ppf2, (LPITEMIDLIST*)&ntreg.pidl);
-        if (hr == S_OK)
+        hr = IPersistFolder2_GetCurFolder(ppf2, &raw_pidl);
+        if(SUCCEEDED(hr))
         {
+            LPITEMIDLIST computer_pidl;
+            SHGetFolderLocation(NULL,CSIDL_DRIVES,NULL,0,&computer_pidl);
+            if(ILIsParent(computer_pidl,raw_pidl,FALSE))
+            {
+                /* Normalize the pidl to unixfs to workaround an issue with
+                 * sending notifications on dos paths
+                 */
+                WCHAR path[MAX_PATH];
+                SHGetPathFromIDListW(raw_pidl,path);
+                SHParseDisplayName(path,NULL,(LPITEMIDLIST*)&ntreg.pidl,0,NULL);
+                SHFree(raw_pidl);
+            }
+            else
+                ntreg.pidl = raw_pidl;
             ntreg.fRecursive = TRUE;
-            This->hNotify = SHChangeNotifyRegister(This->hWnd, SHCNF_IDLIST, SHCNE_ALLEVENTS,
+            This->hNotify = SHChangeNotifyRegister(This->hWnd, SHCNRF_InterruptLevel, SHCNE_ALLEVENTS,
                                                    SHV_CHANGE_NOTIFY, 1, &ntreg);
             SHFree((LPITEMIDLIST)ntreg.pidl);
+            SHFree(computer_pidl);
         }
         IPersistFolder2_Release(ppf2);
     }
@@ -1257,7 +1273,7 @@ static LRESULT ShellView_OnKillFocus(IShellViewImpl * This)
 * ShellView_OnCommand()
 *
 * NOTES
-*	the CmdID's are the ones from the context menu
+*	the CmdIDs are the ones from the context menu
 */
 static LRESULT ShellView_OnCommand(IShellViewImpl * This,DWORD dwCmdID, DWORD dwCmd, HWND hwndCmd)
 {
@@ -1289,7 +1305,7 @@ static LRESULT ShellView_OnCommand(IShellViewImpl * This,DWORD dwCmdID, DWORD dw
 	    CheckToolbar(This);
 	    break;
 
-	  /* the menu-ID's for sorting are 0x30... see shrec.rc */
+	  /* the menu IDs for sorting are 0x30... see shell32.rc */
 	  case 0x30:
 	  case 0x31:
 	  case 0x32:
@@ -2079,9 +2095,7 @@ static HRESULT WINAPI IShellView2_fnCreateViewWindow2(IShellView2 *iface,
 
     TRACE("-- psvPrev %p, pfs %p, psbOwner %p, prcView %p\n",
             view_params->psvPrev, view_params->pfs, view_params->psbOwner, view_params->prcView);
-    TRACE("-- vmode %#x, flags %#x, left %d, top %d, right %d, bottom %d\n",
-            view_params->pfs->ViewMode, view_params->pfs->fFlags, view_params->prcView->left,
-            view_params->prcView->top, view_params->prcView->right, view_params->prcView->bottom);
+    TRACE("-- vmode %#x, flags %#x, view %s\n", view_params->pfs->ViewMode, view_params->pfs->fFlags, wine_dbgstr_rect(view_params->prcView));
 
     if (!view_params->psbOwner) return E_UNEXPECTED;
 
