@@ -48,12 +48,13 @@
 #include "winuser.h"
 #include "winnls.h"
 #include "mmddk.h"
-#include "alsa.h"
+#include "mmreg.h"
+#include "dsound.h"
 #include "wine/debug.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(midi);
+#include <alsa/asoundlib.h>
 
-#ifdef HAVE_ALSA
+WINE_DEFAULT_DEBUG_CHANNEL(midi);
 
 #ifndef SND_SEQ_PORT_TYPE_PORT
 #define SND_SEQ_PORT_TYPE_PORT (1<<19)  /* Appears in version 1.0.12rc1 */
@@ -149,12 +150,12 @@ static	int 	MIDI_AlsaToWindowsDeviceType(unsigned int type)
      * MOD_SQSYNTH      square wave internal synth
      * MOD_FMSYNTH      FM internal synth
      * MOD_MAPPER       MIDI mapper
-     * MOD_WAVETABLE    hardware watetable internal synth
+     * MOD_WAVETABLE    hardware wavetable internal synth
      * MOD_SWSYNTH      software internal synth
      */
 
     /* FIXME Is this really the correct equivalence from ALSA to
-       Windows Sound type */
+       Windows Sound type? */
 
     if (type & SND_SEQ_PORT_TYPE_SYNTH)
         return MOD_FMSYNTH;
@@ -361,14 +362,17 @@ static DWORD WINAPI midRecThread(LPVOID arg)
                     toSend = 0xFC;
                     break;
                 case SND_SEQ_EVENT_SONGPOS:
-                    toSend = (((ev->data.control.value >> 7) & 0x7f) << 16) | ((ev->data.control.value & 0x7f) << 8) | 0xF2;
+                    toSend = (((ev->data.control.value >> 7) & 0x7f) << 16) | ((ev->data.control.value & 0x7f) << 8) | MIDI_CMD_COMMON_SONG_POS;
                     break;
                 case SND_SEQ_EVENT_SONGSEL:
-                  toSend = ((ev->data.control.value & 0x7f) << 8) | 0xF3;
+                  toSend = ((ev->data.control.value & 0x7f) << 8) | MIDI_CMD_COMMON_SONG_SELECT;
                     break;
                 case SND_SEQ_EVENT_RESET:
                     toSend = 0xFF;
                     break;
+                case SND_SEQ_EVENT_QFRAME:
+                  toSend = ((ev->data.control.value & 0x7f) << 8) | MIDI_CMD_COMMON_MTC_QUARTER;
+                    break;
 		case SND_SEQ_EVENT_SYSEX:
 		    {
 			int pos = 0;
@@ -863,7 +867,6 @@ static DWORD modData(WORD wDevID, DWORD dwParam)
 		switch (evt & 0x0F) {
 		case 0x00:	/* System Exclusive, don't do it on modData,
 				 * should require modLongData*/
-		case 0x01:	/* Undefined */
 		case 0x04:	/* Undefined. */
 		case 0x05:	/* Undefined. */
 		case 0x07:	/* End of Exclusive. */
@@ -889,6 +892,7 @@ static DWORD modData(WORD wDevID, DWORD dwParam)
 			snd_seq_ev_set_sysex(&event, sizeof(reset_sysex_seq), reset_sysex_seq);
 		    }
 		    break;
+		case 0x01:	/* MTC Quarter frame */
 		case 0x03:	/* Song Select. */
 		    {
 			BYTE buf[2];
@@ -1065,6 +1069,17 @@ static DWORD modUnprepare(WORD wDevID, LPMIDIHDR lpMidiHdr, DWORD dwSize)
     return MMSYSERR_NOERROR;
 }
 
+/**************************************************************************
+ * 			modGetVolume				[internal]
+ */
+static DWORD modGetVolume(WORD wDevID, DWORD* lpdwVolume)
+{
+    if (!lpdwVolume) return MMSYSERR_INVALPARAM;
+    if (wDevID >= MODM_NumDevs) return MMSYSERR_BADDEVICEID;
+    *lpdwVolume = 0xFFFFFFFF;
+    return (MidiOutDev[wDevID].caps.dwSupport & MIDICAPS_VOLUME) ? 0 : MMSYSERR_NOTSUPPORTED;
+}
+
 /**************************************************************************
  * 			modReset				[internal]
  */
@@ -1124,14 +1139,11 @@ static void ALSA_AddMidiPort(snd_seq_client_info_t* cinfo, snd_seq_port_info_t*
 	MidiOutDev[MODM_NumDevs].caps.wPid = 0x0001; 	/* FIXME Product ID  */
 	/* Product Version. We simply say "1" */
 	MidiOutDev[MODM_NumDevs].caps.vDriverVersion = 0x001;
+	/* The following are mandatory for MOD_MIDIPORT */
 	MidiOutDev[MODM_NumDevs].caps.wChannelMask   = 0xFFFF;
-
-	/* FIXME Do we have this information?
-	 * Assuming the soundcards can handle
-	 * MIDICAPS_VOLUME and MIDICAPS_LRVOLUME but
-	 * not MIDICAPS_CACHE.
-	 */
-	MidiOutDev[MODM_NumDevs].caps.dwSupport      = MIDICAPS_VOLUME|MIDICAPS_LRVOLUME;
+	MidiOutDev[MODM_NumDevs].caps.wVoices        = 0;
+	MidiOutDev[MODM_NumDevs].caps.wNotes         = 0;
+	MidiOutDev[MODM_NumDevs].caps.dwSupport      = 0;
 
 	/* Try to use both client and port names, if this is too long take the port name only.
            In the second case the port name should be explicit enough due to its big size.
@@ -1139,22 +1151,30 @@ static void ALSA_AddMidiPort(snd_seq_client_info_t* cinfo, snd_seq_port_info_t*
 	if ( (strlen(snd_seq_client_info_get_name(cinfo)) + strlen(snd_seq_port_info_get_name(pinfo)) + 3) < MAXPNAMELEN ) {
 	    sprintf(midiPortName, "%s - %s", snd_seq_client_info_get_name(cinfo), snd_seq_port_info_get_name(pinfo));
 	} else {
-	    lstrcpynA(midiPortName, snd_seq_port_info_get_name(pinfo), MAXPNAMELEN-1);
-	    midiPortName[MAXPNAMELEN-1] = 0;
+	    lstrcpynA(midiPortName, snd_seq_port_info_get_name(pinfo), MAXPNAMELEN);
 	}
 	MultiByteToWideChar(CP_UNIXCP, 0, midiPortName, -1,
                             MidiOutDev[MODM_NumDevs].caps.szPname,
                             sizeof(MidiOutDev[MODM_NumDevs].caps.szPname) / sizeof(WCHAR));
 
 	MidiOutDev[MODM_NumDevs].caps.wTechnology = MIDI_AlsaToWindowsDeviceType(type);
-	MidiOutDev[MODM_NumDevs].caps.wVoices     = 16;
 
-        /* FIXME Is it possible to know the maximum
-         * number of simultaneous notes of a soundcard ?
-         * I believe we don't have this information, but
-         * it's probably equal or more than wVoices
-         */
-	MidiOutDev[MODM_NumDevs].caps.wNotes = 16;
+	if (MOD_MIDIPORT != MidiOutDev[MODM_NumDevs].caps.wTechnology) {
+	    /* FIXME Do we have this information?
+	     * Assuming the soundcards can handle
+	     * MIDICAPS_VOLUME and MIDICAPS_LRVOLUME but
+	     * not MIDICAPS_CACHE.
+	     */
+	    MidiOutDev[MODM_NumDevs].caps.dwSupport = MIDICAPS_VOLUME|MIDICAPS_LRVOLUME;
+	    MidiOutDev[MODM_NumDevs].caps.wVoices   = 16;
+
+            /* FIXME Is it possible to know the maximum
+             * number of simultaneous notes of a soundcard ?
+             * I believe we don't have this information, but
+             * it's probably equal or more than wVoices
+             */
+	    MidiOutDev[MODM_NumDevs].caps.wNotes    = 16;
+	}
 	MidiOutDev[MODM_NumDevs].bEnabled    = TRUE;
 
 	TRACE("MidiOut[%d]\tname='%s' techn=%d voices=%d notes=%d chnMsk=%04x support=%d\n"
@@ -1190,13 +1210,7 @@ static void ALSA_AddMidiPort(snd_seq_client_info_t* cinfo, snd_seq_port_info_t*
 	MidiInDev[MIDM_NumDevs].caps.wPid = 0x0001; 	/* FIXME Product ID  */
 	/* Product Version. We simply say "1" */
 	MidiInDev[MIDM_NumDevs].caps.vDriverVersion = 0x001;
-
-	/* FIXME Do we have this information?
-	 * Assuming the soundcards can handle
-	 * MIDICAPS_VOLUME and MIDICAPS_LRVOLUME but
-	 * not MIDICAPS_CACHE.
-	 */
-	MidiInDev[MIDM_NumDevs].caps.dwSupport      = MIDICAPS_VOLUME|MIDICAPS_LRVOLUME;
+	MidiInDev[MIDM_NumDevs].caps.dwSupport = 0; /* mandatory with MIDIINCAPS */
 
 	/* Try to use both client and port names, if this is too long take the port name only.
            In the second case the port name should be explicit enough due to its big size.
@@ -1204,8 +1218,7 @@ static void ALSA_AddMidiPort(snd_seq_client_info_t* cinfo, snd_seq_port_info_t*
 	if ( (strlen(snd_seq_client_info_get_name(cinfo)) + strlen(snd_seq_port_info_get_name(pinfo)) + 3) < MAXPNAMELEN ) {
 	    sprintf(midiPortName, "%s - %s", snd_seq_client_info_get_name(cinfo), snd_seq_port_info_get_name(pinfo));
 	} else {
-	    lstrcpynA(midiPortName, snd_seq_port_info_get_name(pinfo), MAXPNAMELEN-1);
-	    midiPortName[MAXPNAMELEN-1] = 0;
+	    lstrcpynA(midiPortName, snd_seq_port_info_get_name(pinfo), MAXPNAMELEN);
         }
 	MultiByteToWideChar(CP_UNIXCP, 0, midiPortName, -1,
                             MidiInDev[MIDM_NumDevs].caps.szPname,
@@ -1290,8 +1303,6 @@ static LONG ALSA_MidiInit(void)
     return TRUE;
 }
 
-#endif
-
 /**************************************************************************
  * 			midMessage (WINEALSA.@)
  */
@@ -1301,7 +1312,6 @@ DWORD WINAPI ALSA_midMessage(UINT wDevID, UINT wMsg, DWORD_PTR dwUser,
     TRACE("(%04X, %04X, %08lX, %08lX, %08lX);\n",
 	  wDevID, wMsg, dwUser, dwParam1, dwParam2);
     switch (wMsg) {
-#ifdef HAVE_ALSA
     case DRVM_INIT:
         ALSA_MidiInit();
     case DRVM_EXIT:
@@ -1329,7 +1339,6 @@ DWORD WINAPI ALSA_midMessage(UINT wDevID, UINT wMsg, DWORD_PTR dwUser,
 	return midStart(wDevID);
     case MIDM_STOP:
 	return midStop(wDevID);
-#endif
     default:
 	TRACE("Unsupported message\n");
     }
@@ -1346,7 +1355,6 @@ DWORD WINAPI ALSA_modMessage(UINT wDevID, UINT wMsg, DWORD_PTR dwUser,
 	  wDevID, wMsg, dwUser, dwParam1, dwParam2);
 
     switch (wMsg) {
-#ifdef HAVE_ALSA
     case DRVM_INIT:
         ALSA_MidiInit();
     case DRVM_EXIT:
@@ -1371,16 +1379,40 @@ DWORD WINAPI ALSA_modMessage(UINT wDevID, UINT wMsg, DWORD_PTR dwUser,
     case MODM_GETNUMDEVS:
 	return MODM_NumDevs;
     case MODM_GETVOLUME:
-	return 0;
+	return modGetVolume(wDevID, (DWORD*)dwParam1);
     case MODM_SETVOLUME:
 	return 0;
     case MODM_RESET:
 	return modReset(wDevID);
-#endif
     default:
 	TRACE("Unsupported message\n");
     }
     return MMSYSERR_NOTSUPPORTED;
 }
 
-/*-----------------------------------------------------------------------*/
+/**************************************************************************
+ * 				DriverProc (WINEALSA.@)
+ */
+LRESULT CALLBACK ALSA_DriverProc(DWORD_PTR dwDevID, HDRVR hDriv, UINT wMsg,
+                                 LPARAM dwParam1, LPARAM dwParam2)
+{
+/* EPP     TRACE("(%08lX, %04X, %08lX, %08lX, %08lX)\n",  */
+/* EPP 	  dwDevID, hDriv, wMsg, dwParam1, dwParam2); */
+
+    switch(wMsg) {
+    case DRV_LOAD:
+    case DRV_FREE:
+    case DRV_OPEN:
+    case DRV_CLOSE:
+    case DRV_ENABLE:
+    case DRV_DISABLE:
+    case DRV_QUERYCONFIGURE:
+    case DRV_CONFIGURE:
+        return 1;
+    case DRV_INSTALL:
+    case DRV_REMOVE:
+        return DRV_SUCCESS;
+    default:
+	return 0;
+    }
+}
