@@ -37,12 +37,11 @@
 #include "devpkey.h"
 #include "dshow.h"
 #include "dsound.h"
-#include "endpointvolume.h"
 
 #include "initguid.h"
+#include "endpointvolume.h"
 #include "audioclient.h"
 #include "audiopolicy.h"
-#include "dsdriver.h"
 
 #include <alsa/asoundlib.h>
 
@@ -158,7 +157,6 @@ static const IAudioStreamVolumeVtbl AudioStreamVolume_Vtbl;
 static const IChannelAudioVolumeVtbl ChannelAudioVolume_Vtbl;
 static const IAudioSessionManager2Vtbl AudioSessionManager2_Vtbl;
 
-int wine_snd_pcm_recover(snd_pcm_t *pcm, int err, int silent);
 static AudioSessionWrapper *AudioSessionWrapper_Create(ACImpl *client);
 
 static inline ACImpl *impl_from_IAudioClient(IAudioClient *iface)
@@ -237,6 +235,49 @@ int WINAPI AUDDRV_GetPriority(void)
     return Priority_Neutral;
 }
 
+/**************************************************************************
+ * 			wine_snd_pcm_recover		[internal]
+ *
+ * Code slightly modified from alsa-lib v1.0.23 snd_pcm_recover implementation.
+ * used to recover from XRUN errors (buffer underflow/overflow)
+ */
+static int wine_snd_pcm_recover(snd_pcm_t *pcm, int err, int silent)
+{
+    if (err > 0)
+        err = -err;
+    if (err == -EINTR)	/* nothing to do, continue */
+        return 0;
+    if (err == -EPIPE) {
+        const char *s;
+        if (snd_pcm_stream(pcm) == SND_PCM_STREAM_PLAYBACK)
+            s = "underrun";
+        else
+            s = "overrun";
+        if (!silent)
+            ERR("%s occurred\n", s);
+        err = snd_pcm_prepare(pcm);
+        if (err < 0) {
+            ERR("cannot recover from %s, prepare failed: %s\n", s, snd_strerror(err));
+            return err;
+        }
+        return 0;
+    }
+    if (err == -ESTRPIPE) {
+        while ((err = snd_pcm_resume(pcm)) == -EAGAIN)
+            /* wait until suspend flag is released */
+            poll(NULL, 0, 1000);
+        if (err < 0) {
+            err = snd_pcm_prepare(pcm);
+            if (err < 0) {
+                ERR("cannot recover from suspend, prepare failed: %s\n", snd_strerror(err));
+                return err;
+            }
+        }
+        return 0;
+    }
+    return err;
+}
+
 static BOOL alsa_try_open(const char *devnode, snd_pcm_stream_t stream)
 {
     snd_pcm_t *handle;
@@ -253,8 +294,7 @@ static BOOL alsa_try_open(const char *devnode, snd_pcm_stream_t stream)
 }
 
 static HRESULT alsa_get_card_devices(snd_pcm_stream_t stream, WCHAR **ids, char **keys,
-        UINT *num, snd_ctl_t *ctl, int card, const WCHAR *cardnameW,
-        BOOL count_failed)
+        UINT *num, snd_ctl_t *ctl, int card, const WCHAR *cardnameW)
 {
     static const WCHAR dashW[] = {' ','-',' ',0};
     int err, device;
@@ -286,11 +326,8 @@ static HRESULT alsa_get_card_devices(snd_pcm_stream_t stream, WCHAR **ids, char
         }
 
         sprintf(devnode, "plughw:%d,%d", card, device);
-        if(!alsa_try_open(devnode, stream)){
-            if(count_failed)
-                ++(*num);
+        if(!alsa_try_open(devnode, stream))
             continue;
-        }
 
         if(ids && keys){
             DWORD len, cardlen;
@@ -339,7 +376,7 @@ static HRESULT alsa_get_card_devices(snd_pcm_stream_t stream, WCHAR **ids, char
 }
 
 static HRESULT alsa_enum_devices(EDataFlow flow, WCHAR **ids, char **keys,
-        UINT *num, BOOL count_failed)
+        UINT *num)
 {
     snd_pcm_stream_t stream = (flow == eRender ? SND_PCM_STREAM_PLAYBACK :
         SND_PCM_STREAM_CAPTURE);
@@ -356,8 +393,7 @@ static HRESULT alsa_enum_devices(EDataFlow flow, WCHAR **ids, char **keys,
             memcpy(*keys, defname, sizeof(defname));
         }
         ++*num;
-    }else if(count_failed)
-        ++*num;
+    }
 
     for(err = snd_card_next(&card); card != -1 && err >= 0;
             err = snd_card_next(&card)){
@@ -390,8 +426,7 @@ static HRESULT alsa_enum_devices(EDataFlow flow, WCHAR **ids, char **keys,
         }
         MultiByteToWideChar(CP_UNIXCP, 0, cardname, -1, cardnameW, len);
 
-        alsa_get_card_devices(stream, ids, keys, num, ctl, card, cardnameW,
-                count_failed);
+        alsa_get_card_devices(stream, ids, keys, num, ctl, card, cardnameW);
 
         HeapFree(GetProcessHeap(), 0, cardnameW);
 
@@ -412,7 +447,7 @@ HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids, char ***keys,
 
     TRACE("%d %p %p %p %p\n", flow, ids, keys, num, def_index);
 
-    hr = alsa_enum_devices(flow, NULL, NULL, num, TRUE);
+    hr = alsa_enum_devices(flow, NULL, NULL, num);
     if(FAILED(hr))
         return hr;
 
@@ -433,7 +468,7 @@ HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids, char ***keys,
 
     *def_index = 0;
 
-    hr = alsa_enum_devices(flow, *ids, *keys, num, FALSE);
+    hr = alsa_enum_devices(flow, *ids, *keys, num);
     if(FAILED(hr)){
         int i;
         for(i = 0; i < *num; ++i){
@@ -1342,6 +1377,7 @@ static HRESULT WINAPI AudioClient_IsFormatSupported(IAudioClient *iface,
         ((WAVEFORMATEXTENSIBLE*)closest)->dwChannelMask = mask;
 
         if(fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
+                fmtex->dwChannelMask != 0 &&
                 fmtex->dwChannelMask != mask)
             hr = S_FALSE;
     }
@@ -2076,11 +2112,7 @@ static HRESULT WINAPI AudioRenderClient_ReleaseBuffer(
         buffer = This->tmp_buffer;
 
     if(flags & AUDCLNT_BUFFERFLAGS_SILENT){
-        WAVEFORMATEXTENSIBLE *fmtex = (WAVEFORMATEXTENSIBLE*)This->fmt;
-        if((This->fmt->wFormatTag == WAVE_FORMAT_PCM ||
-                (This->fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
-                 IsEqualGUID(&fmtex->SubFormat, &KSDATAFORMAT_SUBTYPE_PCM))) &&
-                This->fmt->wBitsPerSample == 8)
+        if(This->fmt->wBitsPerSample == 8)
             memset(buffer, 128, written_frames * This->fmt->nBlockAlign);
         else
             memset(buffer, 0, written_frames * This->fmt->nBlockAlign);
