@@ -35,9 +35,9 @@
 #include "devpkey.h"
 #include "dshow.h"
 #include "dsound.h"
-#include "endpointvolume.h"
 
 #include "initguid.h"
+#include "endpointvolume.h"
 #include "audioclient.h"
 #include "audiopolicy.h"
 
@@ -124,10 +124,10 @@ struct ACImpl {
     AudioQueueRef aqueue;
     AudioObjectPropertyScope scope;
     HANDLE timer;
-    UINT32 period_ms, bufsize_frames, inbuf_frames, written_frames;
-    UINT64 last_time;
+    UINT32 period_ms, bufsize_frames, inbuf_frames;
+    UINT64 last_time, written_frames;
     AudioQueueBufferRef public_buffer;
-    BOOL getbuf_last;
+    UINT32 getbuf_last;
     int playing;
 
     AudioSession *session;
@@ -242,6 +242,19 @@ BOOL WINAPI DllMain(HINSTANCE dll, DWORD reason, void *reserved)
     return TRUE;
 }
 
+/* From <dlls/mmdevapi/mmdevapi.h> */
+enum DriverPriority {
+    Priority_Unavailable = 0,
+    Priority_Low,
+    Priority_Neutral,
+    Priority_Preferred
+};
+
+int WINAPI AUDDRV_GetPriority(void)
+{
+    return Priority_Neutral;
+}
+
 HRESULT WINAPI AUDDRV_GetEndpointIDs(EDataFlow flow, WCHAR ***ids,
         AudioDeviceID ***keys, UINT *num, UINT *def_index)
 {
@@ -555,25 +568,21 @@ static DWORD get_channel_mask(unsigned int channels)
     case 0:
         return 0;
     case 1:
-        return SPEAKER_FRONT_CENTER;
+        return KSAUDIO_SPEAKER_MONO;
     case 2:
-        return SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT;
+        return KSAUDIO_SPEAKER_STEREO;
     case 3:
-        return SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT |
-            SPEAKER_LOW_FREQUENCY;
+        return KSAUDIO_SPEAKER_STEREO | SPEAKER_LOW_FREQUENCY;
     case 4:
-        return SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT | SPEAKER_BACK_LEFT |
-            SPEAKER_BACK_RIGHT;
+        return KSAUDIO_SPEAKER_QUAD;    /* not _SURROUND */
     case 5:
-        return SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT | SPEAKER_BACK_LEFT |
-            SPEAKER_BACK_RIGHT | SPEAKER_LOW_FREQUENCY;
+        return KSAUDIO_SPEAKER_QUAD | SPEAKER_LOW_FREQUENCY;
     case 6:
-        return SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT | SPEAKER_BACK_LEFT |
-            SPEAKER_BACK_RIGHT | SPEAKER_LOW_FREQUENCY | SPEAKER_FRONT_CENTER;
+        return KSAUDIO_SPEAKER_5POINT1; /* not 5POINT1_SURROUND */
     case 7:
-        return SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT | SPEAKER_BACK_LEFT |
-            SPEAKER_BACK_RIGHT | SPEAKER_LOW_FREQUENCY | SPEAKER_FRONT_CENTER |
-            SPEAKER_BACK_CENTER;
+        return KSAUDIO_SPEAKER_5POINT1 | SPEAKER_BACK_CENTER;
+    case 8:
+        return KSAUDIO_SPEAKER_7POINT1; /* not 7POINT1_SURROUND */
     }
     FIXME("Unknown speaker configuration: %u\n", channels);
     return 0;
@@ -1109,6 +1118,7 @@ static HRESULT WINAPI AudioClient_IsFormatSupported(IAudioClient *iface,
         WAVEFORMATEX **outpwfx)
 {
     ACImpl *This = impl_from_IAudioClient(iface);
+    WAVEFORMATEXTENSIBLE *fmtex = (WAVEFORMATEXTENSIBLE*)pwfx;
     AudioQueueRef aqueue;
     HRESULT hr;
 
@@ -1126,6 +1136,11 @@ static HRESULT WINAPI AudioClient_IsFormatSupported(IAudioClient *iface,
 
     dump_fmt(pwfx);
 
+    if(pwfx->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
+            fmtex->dwChannelMask != 0 &&
+            fmtex->dwChannelMask != get_channel_mask(pwfx->nChannels))
+        return AUDCLNT_E_UNSUPPORTED_FORMAT;
+
     OSSpinLockLock(&This->lock);
 
     hr = ca_setup_aqueue(This->adevid, This->dataflow, pwfx, NULL, &aqueue);
@@ -1593,6 +1608,7 @@ static HRESULT WINAPI AudioRenderClient_GetBuffer(IAudioRenderClient *iface,
 
     if(!data)
         return E_POINTER;
+    *data = NULL;
 
     OSSpinLockLock(&This->lock);
 
@@ -1602,7 +1618,6 @@ static HRESULT WINAPI AudioRenderClient_GetBuffer(IAudioRenderClient *iface,
     }
 
     if(!frames){
-        This->getbuf_last = TRUE;
         OSSpinLockUnlock(&This->lock);
         return S_OK;
     }
@@ -1647,7 +1662,7 @@ static HRESULT WINAPI AudioRenderClient_GetBuffer(IAudioRenderClient *iface,
 
     *data = This->public_buffer->mAudioData;
 
-    This->getbuf_last = TRUE;
+    This->getbuf_last = frames;
 
     OSSpinLockUnlock(&This->lock);
 
@@ -1664,11 +1679,27 @@ static HRESULT WINAPI AudioRenderClient_ReleaseBuffer(
 
     OSSpinLockLock(&This->lock);
 
+    if(!frames){
+        This->getbuf_last = 0;
+        if(This->public_buffer){
+            AQBuffer *buf = This->public_buffer->mUserData;
+            list_add_tail(&This->avail_buffers, &buf->entry);
+            This->public_buffer = NULL;
+        }
+        OSSpinLockUnlock(&This->lock);
+        return S_OK;
+    }
+
     if(!This->getbuf_last){
         OSSpinLockUnlock(&This->lock);
         return AUDCLNT_E_OUT_OF_ORDER;
     }
 
+    if(frames > This->getbuf_last){
+        OSSpinLockUnlock(&This->lock);
+        return AUDCLNT_E_INVALID_SIZE;
+    }
+
     if(flags & AUDCLNT_BUFFERFLAGS_SILENT){
         WAVEFORMATEXTENSIBLE *fmtex = (WAVEFORMATEXTENSIBLE*)This->fmt;
         if((This->fmt->wFormatTag == WAVE_FORMAT_PCM ||
@@ -1695,7 +1726,7 @@ static HRESULT WINAPI AudioRenderClient_ReleaseBuffer(
         AudioQueuePrime(This->aqueue, 0, NULL);
 
     This->public_buffer = NULL;
-    This->getbuf_last = FALSE;
+    This->getbuf_last = 0;
     This->written_frames += frames;
     This->inbuf_frames += frames;
 
@@ -1786,7 +1817,7 @@ static HRESULT WINAPI AudioCaptureClient_GetBuffer(IAudioCaptureClient *iface,
     *flags = 0;
     This->written_frames += *frames;
     This->inbuf_frames -= *frames;
-    This->getbuf_last = TRUE;
+    This->getbuf_last = 1;
 
     if(devpos || qpcpos)
         AudioClock_GetPosition_nolock(This, devpos, qpcpos, FALSE);
@@ -1800,8 +1831,7 @@ static HRESULT WINAPI AudioCaptureClient_ReleaseBuffer(
         IAudioCaptureClient *iface, UINT32 done)
 {
     ACImpl *This = impl_from_IAudioCaptureClient(iface);
-    UINT32 pbuf_frames =
-        This->public_buffer->mAudioDataByteSize / This->fmt->nBlockAlign;
+    UINT32 pbuf_frames;
     OSStatus sc;
 
     TRACE("(%p)->(%u)\n", This, done);
@@ -1813,6 +1843,7 @@ static HRESULT WINAPI AudioCaptureClient_ReleaseBuffer(
         return AUDCLNT_E_OUT_OF_ORDER;
     }
 
+    pbuf_frames = This->public_buffer->mAudioDataByteSize / This->fmt->nBlockAlign;
     if(done != 0 && done != pbuf_frames){
         OSSpinLockUnlock(&This->lock);
         return AUDCLNT_E_INVALID_SIZE;
@@ -1826,7 +1857,7 @@ static HRESULT WINAPI AudioCaptureClient_ReleaseBuffer(
         This->public_buffer = NULL;
     }
 
-    This->getbuf_last = FALSE;
+    This->getbuf_last = 0;
 
     OSSpinLockUnlock(&This->lock);
 
