@@ -1164,7 +1164,7 @@ void shader_dump_src_param(const struct wined3d_shader_src_param *param,
 
 /* Shared code in order to generate the bulk of the shader string.
  * NOTE: A description of how to parse tokens can be found on MSDN. */
-void shader_generate_main(struct wined3d_shader *shader, struct wined3d_shader_buffer *buffer,
+void shader_generate_main(const struct wined3d_shader *shader, struct wined3d_shader_buffer *buffer,
         const struct wined3d_shader_reg_maps *reg_maps, const DWORD *byte_code, void *backend_ctx)
 {
     struct wined3d_device *device = shader->device;
@@ -1553,7 +1553,8 @@ const struct wined3d_shader_backend_ops none_shader_backend =
 };
 
 static HRESULT shader_set_function(struct wined3d_shader *shader, const DWORD *byte_code,
-        const struct wined3d_shader_signature *output_signature, DWORD float_const_count)
+        const struct wined3d_shader_signature *output_signature, DWORD float_const_count,
+        enum wined3d_shader_type type, unsigned int max_version)
 {
     struct wined3d_shader_reg_maps *reg_maps = &shader->reg_maps;
     const struct wined3d_shader_frontend *fe;
@@ -1591,6 +1592,17 @@ static HRESULT shader_set_function(struct wined3d_shader *shader, const DWORD *b
             byte_code, float_const_count);
     if (FAILED(hr)) return hr;
 
+    if (reg_maps->shader_version.type != type)
+    {
+        WARN("Wrong shader type %d.\n", reg_maps->shader_version.type);
+        return WINED3DERR_INVALIDCALL;
+    }
+    if (reg_maps->shader_version.major > max_version)
+    {
+        WARN("Shader version %d not supported by this D3D API version.\n", reg_maps->shader_version.major);
+        return WINED3DERR_INVALIDCALL;
+    }
+
     shader->function = HeapAlloc(GetProcessHeap(), 0, shader->functionLength);
     if (!shader->function)
         return E_OUTOFMEMORY;
@@ -1706,7 +1718,7 @@ static BOOL match_usage(BYTE usage1, BYTE usage_idx1, BYTE usage2, BYTE usage_id
     return FALSE;
 }
 
-BOOL vshader_get_input(struct wined3d_shader *shader,
+BOOL vshader_get_input(const struct wined3d_shader *shader,
         BYTE usage_req, BYTE usage_idx_req, unsigned int *regnum)
 {
     WORD map = shader->reg_maps.input_registers;
@@ -1802,7 +1814,7 @@ static void vertexshader_set_limits(struct wined3d_shader *shader)
 
 static HRESULT vertexshader_init(struct wined3d_shader *shader, struct wined3d_device *device,
         const DWORD *byte_code, const struct wined3d_shader_signature *output_signature,
-        void *parent, const struct wined3d_parent_ops *parent_ops)
+        void *parent, const struct wined3d_parent_ops *parent_ops, unsigned int max_version)
 {
     struct wined3d_shader_reg_maps *reg_maps = &shader->reg_maps;
     unsigned int i;
@@ -1812,7 +1824,8 @@ static HRESULT vertexshader_init(struct wined3d_shader *shader, struct wined3d_d
     if (!byte_code) return WINED3DERR_INVALIDCALL;
 
     shader_init(shader, device, parent, parent_ops);
-    hr = shader_set_function(shader, byte_code, output_signature, device->d3d_vshader_constantF);
+    hr = shader_set_function(shader, byte_code, output_signature, device->d3d_vshader_constantF,
+            WINED3D_SHADER_TYPE_VERTEX, max_version);
     if (FAILED(hr))
     {
         WARN("Failed to set function, hr %#x.\n", hr);
@@ -1851,12 +1864,13 @@ static HRESULT vertexshader_init(struct wined3d_shader *shader, struct wined3d_d
 
 static HRESULT geometryshader_init(struct wined3d_shader *shader, struct wined3d_device *device,
         const DWORD *byte_code, const struct wined3d_shader_signature *output_signature,
-        void *parent, const struct wined3d_parent_ops *parent_ops)
+        void *parent, const struct wined3d_parent_ops *parent_ops, unsigned int max_version)
 {
     HRESULT hr;
 
     shader_init(shader, device, parent, parent_ops);
-    hr = shader_set_function(shader, byte_code, output_signature, 0);
+    hr = shader_set_function(shader, byte_code, output_signature, 0,
+            WINED3D_SHADER_TYPE_GEOMETRY, max_version);
     if (FAILED(hr))
     {
         WARN("Failed to set function, hr %#x.\n", hr);
@@ -1879,7 +1893,7 @@ void find_ps_compile_args(const struct wined3d_state *state,
     memset(args, 0, sizeof(*args)); /* FIXME: Make sure all bits are set. */
     if (state->render_states[WINED3DRS_SRGBWRITEENABLE])
     {
-        struct wined3d_surface *rt = device->fb.render_targets[0];
+        const struct wined3d_surface *rt = state->fb->render_targets[0];
         if (rt->resource.format->flags & WINED3DFMT_FLAG_SRGB_WRITE) args->srgb_correction = 1;
     }
 
@@ -2057,7 +2071,7 @@ static void pixelshader_set_limits(struct wined3d_shader *shader)
 
 static HRESULT pixelshader_init(struct wined3d_shader *shader, struct wined3d_device *device,
         const DWORD *byte_code, const struct wined3d_shader_signature *output_signature,
-        void *parent, const struct wined3d_parent_ops *parent_ops)
+        void *parent, const struct wined3d_parent_ops *parent_ops, unsigned int max_version)
 {
     const struct wined3d_gl_info *gl_info = &device->adapter->gl_info;
     unsigned int i, highest_reg_used = 0, num_regs_used = 0;
@@ -2066,7 +2080,8 @@ static HRESULT pixelshader_init(struct wined3d_shader *shader, struct wined3d_de
     if (!byte_code) return WINED3DERR_INVALIDCALL;
 
     shader_init(shader, device, parent, parent_ops);
-    hr = shader_set_function(shader, byte_code, output_signature, device->d3d_pshader_constantF);
+    hr = shader_set_function(shader, byte_code, output_signature, device->d3d_pshader_constantF,
+            WINED3D_SHADER_TYPE_PIXEL, max_version);
     if (FAILED(hr))
     {
         WARN("Failed to set function, hr %#x.\n", hr);
@@ -2167,7 +2182,7 @@ void pixelshader_update_samplers(struct wined3d_shader_reg_maps *reg_maps, struc
 
 HRESULT CDECL wined3d_shader_create_gs(struct wined3d_device *device, const DWORD *byte_code,
         const struct wined3d_shader_signature *output_signature, void *parent,
-        const struct wined3d_parent_ops *parent_ops, struct wined3d_shader **shader)
+        const struct wined3d_parent_ops *parent_ops, struct wined3d_shader **shader, unsigned int max_version)
 {
     struct wined3d_shader *object;
     HRESULT hr;
@@ -2182,7 +2197,7 @@ HRESULT CDECL wined3d_shader_create_gs(struct wined3d_device *device, const DWOR
         return E_OUTOFMEMORY;
     }
 
-    hr = geometryshader_init(object, device, byte_code, output_signature, parent, parent_ops);
+    hr = geometryshader_init(object, device, byte_code, output_signature, parent, parent_ops, max_version);
     if (FAILED(hr))
     {
         WARN("Failed to initialize geometry shader, hr %#x.\n", hr);
@@ -2198,7 +2213,7 @@ HRESULT CDECL wined3d_shader_create_gs(struct wined3d_device *device, const DWOR
 
 HRESULT CDECL wined3d_shader_create_ps(struct wined3d_device *device, const DWORD *byte_code,
         const struct wined3d_shader_signature *output_signature, void *parent,
-        const struct wined3d_parent_ops *parent_ops, struct wined3d_shader **shader)
+        const struct wined3d_parent_ops *parent_ops, struct wined3d_shader **shader, unsigned int max_version)
 {
     struct wined3d_shader *object;
     HRESULT hr;
@@ -2216,7 +2231,7 @@ HRESULT CDECL wined3d_shader_create_ps(struct wined3d_device *device, const DWOR
         return E_OUTOFMEMORY;
     }
 
-    hr = pixelshader_init(object, device, byte_code, output_signature, parent, parent_ops);
+    hr = pixelshader_init(object, device, byte_code, output_signature, parent, parent_ops, max_version);
     if (FAILED(hr))
     {
         WARN("Failed to initialize pixel shader, hr %#x.\n", hr);
@@ -2232,7 +2247,7 @@ HRESULT CDECL wined3d_shader_create_ps(struct wined3d_device *device, const DWOR
 
 HRESULT CDECL wined3d_shader_create_vs(struct wined3d_device *device, const DWORD *byte_code,
         const struct wined3d_shader_signature *output_signature, void *parent,
-        const struct wined3d_parent_ops *parent_ops, struct wined3d_shader **shader)
+        const struct wined3d_parent_ops *parent_ops, struct wined3d_shader **shader, unsigned int max_version)
 {
     struct wined3d_shader *object;
     HRESULT hr;
@@ -2250,7 +2265,7 @@ HRESULT CDECL wined3d_shader_create_vs(struct wined3d_device *device, const DWOR
         return E_OUTOFMEMORY;
     }
 
-    hr = vertexshader_init(object, device, byte_code, output_signature, parent, parent_ops);
+    hr = vertexshader_init(object, device, byte_code, output_signature, parent, parent_ops, max_version);
     if (FAILED(hr))
     {
         WARN("Failed to initialize vertex shader, hr %#x.\n", hr);
