@@ -47,9 +47,9 @@
 #include "devpkey.h"
 #include "dshow.h"
 #include "dsound.h"
-#include "endpointvolume.h"
 
 #include "initguid.h"
+#include "endpointvolume.h"
 #include "audiopolicy.h"
 #include "audioclient.h"
 
@@ -667,8 +667,8 @@ static WAVEFORMATEX *clone_format(const WAVEFORMATEX *fmt)
     return ret;
 }
 
-static HRESULT setup_oss_device(ACImpl *This, const WAVEFORMATEX *fmt,
-        WAVEFORMATEX **out)
+static HRESULT setup_oss_device(int fd, const WAVEFORMATEX *fmt,
+        WAVEFORMATEX **out, BOOL query)
 {
     int tmp, oss_format;
     double tenth;
@@ -682,7 +682,7 @@ static HRESULT setup_oss_device(ACImpl *This, const WAVEFORMATEX *fmt,
     tmp = oss_format = get_oss_format(fmt);
     if(oss_format < 0)
         return AUDCLNT_E_UNSUPPORTED_FORMAT;
-    if(ioctl(This->fd, SNDCTL_DSP_SETFMT, &tmp) < 0){
+    if(ioctl(fd, SNDCTL_DSP_SETFMT, &tmp) < 0){
         WARN("SETFMT failed: %d (%s)\n", errno, strerror(errno));
         return E_FAIL;
     }
@@ -696,7 +696,7 @@ static HRESULT setup_oss_device(ACImpl *This, const WAVEFORMATEX *fmt,
         return E_OUTOFMEMORY;
 
     tmp = fmt->nSamplesPerSec;
-    if(ioctl(This->fd, SNDCTL_DSP_SPEED, &tmp) < 0){
+    if(ioctl(fd, SNDCTL_DSP_SPEED, &tmp) < 0){
         WARN("SPEED failed: %d (%s)\n", errno, strerror(errno));
         CoTaskMemFree(closest);
         return E_FAIL;
@@ -708,7 +708,7 @@ static HRESULT setup_oss_device(ACImpl *This, const WAVEFORMATEX *fmt,
     }
 
     tmp = fmt->nChannels;
-    if(ioctl(This->fd, SNDCTL_DSP_CHANNELS, &tmp) < 0){
+    if(ioctl(fd, SNDCTL_DSP_CHANNELS, &tmp) < 0){
         WARN("CHANNELS failed: %d (%s)\n", errno, strerror(errno));
         CoTaskMemFree(closest);
         return E_FAIL;
@@ -723,7 +723,8 @@ static HRESULT setup_oss_device(ACImpl *This, const WAVEFORMATEX *fmt,
 
         ((WAVEFORMATEXTENSIBLE*)closest)->dwChannelMask = mask;
 
-        if(fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
+        if(query && fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
+                fmtex->dwChannelMask != 0 &&
                 fmtex->dwChannelMask != mask)
             ret = S_FALSE;
     }
@@ -861,7 +862,7 @@ static HRESULT WINAPI AudioClient_Initialize(IAudioClient *iface,
         return AUDCLNT_E_ALREADY_INITIALIZED;
     }
 
-    hr = setup_oss_device(This, fmt, NULL);
+    hr = setup_oss_device(This->fd, fmt, NULL, FALSE);
     if(hr == S_FALSE){
         LeaveCriticalSection(&This->lock);
         return AUDCLNT_E_UNSUPPORTED_FORMAT;
@@ -1081,6 +1082,7 @@ static HRESULT WINAPI AudioClient_IsFormatSupported(IAudioClient *iface,
         WAVEFORMATEX **outpwfx)
 {
     ACImpl *This = impl_from_IAudioClient(iface);
+    int fd = -1;
     HRESULT ret;
 
     TRACE("(%p)->(%x, %p, %p)\n", This, mode, pwfx, outpwfx);
@@ -1097,11 +1099,20 @@ static HRESULT WINAPI AudioClient_IsFormatSupported(IAudioClient *iface,
 
     dump_fmt(pwfx);
 
-    EnterCriticalSection(&This->lock);
+    if(This->dataflow == eRender)
+        fd = open(This->ai.devnode, O_WRONLY, 0);
+    else if(This->dataflow == eCapture)
+        fd = open(This->ai.devnode, O_RDONLY, 0);
 
-    ret = setup_oss_device(This, pwfx, outpwfx);
+    if(fd < 0){
+        ERR("Unable to open device %s: %d (%s)\n", This->ai.devnode, errno,
+                strerror(errno));
+        return AUDCLNT_E_DEVICE_INVALIDATED;
+    }
 
-    LeaveCriticalSection(&This->lock);
+    ret = setup_oss_device(fd, pwfx, outpwfx, TRUE);
+
+    close(fd);
 
     return ret;
 }
@@ -1196,11 +1207,7 @@ static HRESULT WINAPI AudioClient_GetDevicePeriod(IAudioClient *iface,
 
 static void oss_silence_buffer(ACImpl *This, BYTE *buf, UINT32 frames)
 {
-    WAVEFORMATEXTENSIBLE *fmtex = (WAVEFORMATEXTENSIBLE*)This->fmt;
-    if((This->fmt->wFormatTag == WAVE_FORMAT_PCM ||
-            (This->fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
-             IsEqualGUID(&fmtex->SubFormat, &KSDATAFORMAT_SUBTYPE_PCM))) &&
-            This->fmt->wBitsPerSample == 8)
+    if(This->fmt->wBitsPerSample == 8)
         memset(buf, 128, frames * This->fmt->nBlockAlign);
     else
         memset(buf, 0, frames * This->fmt->nBlockAlign);
