@@ -218,7 +218,7 @@ static BOOL BRUSH_SelectPatternBrush( X11DRV_PDEVICE *physDev, HBITMAP hbitmap )
 
     X11DRV_DIB_Lock( physBitmap, DIB_Status_GdiMod );
 
-    if ((physDev->depth == 1) && (physBitmap->pixmap_depth != 1))
+    if ((physDev->depth == 1) && (physBitmap->depth != 1))
     {
         wine_tsx11_lock();
         /* Special case: a color pattern on a monochrome DC */
@@ -237,7 +237,7 @@ static BOOL BRUSH_SelectPatternBrush( X11DRV_PDEVICE *physDev, HBITMAP hbitmap )
 
     X11DRV_DIB_Unlock( physBitmap, TRUE );
 
-    if (physBitmap->pixmap_depth > 1)
+    if (physBitmap->depth > 1)
     {
 	physDev->brush.fillStyle = FillTiled;
 	physDev->brush.pixel = 0;  /* Ignored */
