@@ -3226,7 +3226,7 @@ XFontStruct* XFONT_GetFontStruct( X_PHYSFONT pFont )
 /***********************************************************************
  *           SelectFont   (X11DRV.@)
  */
-HFONT CDECL X11DRV_SelectFont( PHYSDEV dev, HFONT hfont, HANDLE gdiFont )
+HFONT X11DRV_SelectFont( PHYSDEV dev, HFONT hfont, HANDLE gdiFont )
 {
     X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     LOGFONTW logfont;
@@ -3238,12 +3238,6 @@ HFONT CDECL X11DRV_SelectFont( PHYSDEV dev, HFONT hfont, HANDLE gdiFont )
 
     TRACE("gdiFont = %p\n", gdiFont);
 
-    if(gdiFont && using_client_side_fonts) {
-        X11DRV_XRender_SelectFont(physDev, hfont);
-        physDev->has_gdi_font = TRUE;
-	return FALSE;
-    }
-
     EnterCriticalSection( &crtsc_fonts_X11 );
 
     if(fontList == NULL) X11DRV_FONT_InitX11Metrics();
@@ -3260,14 +3254,14 @@ HFONT CDECL X11DRV_SelectFont( PHYSDEV dev, HFONT hfont, HANDLE gdiFont )
         /* FIXME - check that the other drivers do this correctly */
         if (lf.lfWidth)
         {
-            INT width = X11DRV_XWStoDS( physDev, lf.lfWidth );
+            INT width = X11DRV_XWStoDS( dev->hdc, lf.lfWidth );
             lf.lfWidth = (lf.lfWidth < 0) ? -abs(width) : abs(width);
             if (lf.lfWidth == 0)
                 lf.lfWidth = 1; /* Minimum width */
         }
         if (lf.lfHeight)
         {
-            INT height = X11DRV_YWStoDS( physDev, lf.lfHeight );
+            INT height = X11DRV_YWStoDS( dev->hdc, lf.lfHeight );
             lf.lfHeight = (lf.lfHeight < 0) ? -abs(height) : abs(height);
             if (lf.lfHeight == 0)
                 lf.lfHeight = MIN_FONT_SIZE;
@@ -3320,8 +3314,7 @@ HFONT CDECL X11DRV_SelectFont( PHYSDEV dev, HFONT hfont, HANDLE gdiFont )
  *
  *           X11DRV_EnumDeviceFonts
  */
-BOOL CDECL X11DRV_EnumDeviceFonts( PHYSDEV dev, LPLOGFONTW plf,
-                                   FONTENUMPROCW proc, LPARAM lp )
+BOOL X11DRV_EnumDeviceFonts( PHYSDEV dev, LPLOGFONTW plf, FONTENUMPROCW proc, LPARAM lp )
 {
     X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     ENUMLOGFONTEXW	lf;
@@ -3386,7 +3379,7 @@ BOOL CDECL X11DRV_EnumDeviceFonts( PHYSDEV dev, LPLOGFONTW plf,
 /***********************************************************************
  *           X11DRV_GetTextMetrics
  */
-BOOL CDECL X11DRV_GetTextMetrics(PHYSDEV dev, TEXTMETRICW *metrics)
+BOOL X11DRV_GetTextMetrics(PHYSDEV dev, TEXTMETRICW *metrics)
 {
     X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
 
@@ -3403,7 +3396,7 @@ BOOL CDECL X11DRV_GetTextMetrics(PHYSDEV dev, TEXTMETRICW *metrics)
 /***********************************************************************
  *           X11DRV_GetCharWidth
  */
-BOOL CDECL X11DRV_GetCharWidth( PHYSDEV dev, UINT firstChar, UINT lastChar, LPINT buffer )
+BOOL X11DRV_GetCharWidth( PHYSDEV dev, UINT firstChar, UINT lastChar, LPINT buffer )
 {
     X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
     fontObject* pfo = XFONT_GetFontObject( physDev->font );
