@@ -335,10 +335,15 @@ static BOOL ComputeGammaFromRamp(WORD ramp[256], float *gamma)
     return FALSE;
   }
   /* check that the gamma is reasonably uniform across the ramp */
-  if (g_max - g_min > 0.1) {
+  if (g_max - g_min > 12.8) {
     ERR("ramp not uniform (max=%f, min=%f, avg=%f), rejected\n", g_max, g_min, g_avg);
     return FALSE;
   }
+  /* check that the gamma is not too bright */
+  if (g_avg < 0.2) {
+    ERR("too bright gamma ( %5.3f), rejected\n", g_avg);
+    return FALSE;
+  }
   /* ok, now we're pretty sure we can set the desired gamma ramp,
    * so go for it */
   *gamma = 1/g_avg;
@@ -386,35 +391,25 @@ static BOOL X11DRV_XF86VM_GetGammaRamp(LPDDGAMMARAMP ramp)
 
 static BOOL X11DRV_XF86VM_SetGammaRamp(LPDDGAMMARAMP ramp)
 {
+  Bool ret = FALSE;
 #ifdef X_XF86VidModeSetGamma
   XF86VidModeGamma gamma;
 
   if (xf86vm_major < 2 || !usexvidmode) return FALSE; /* no gamma control */
+  if (!ComputeGammaFromRamp(ramp->red,   &gamma.red) || /* ramp validation */
+      !ComputeGammaFromRamp(ramp->green, &gamma.green) ||
+      !ComputeGammaFromRamp(ramp->blue,  &gamma.blue)) return FALSE;
+  wine_tsx11_lock();
 #ifdef X_XF86VidModeSetGammaRamp
-  else if (xf86vm_use_gammaramp)
-  {
-      Bool ret;
-      wine_tsx11_lock();
+  if (xf86vm_use_gammaramp)
       ret = pXF86VidModeSetGammaRamp(gdi_display, DefaultScreen(gdi_display), 256,
 				    ramp->red, ramp->green, ramp->blue);
-      wine_tsx11_unlock();
-      return ret;
-  }
-#endif
   else
-  {
-      if (ComputeGammaFromRamp(ramp->red,   &gamma.red) &&
-	  ComputeGammaFromRamp(ramp->green, &gamma.green) &&
-	  ComputeGammaFromRamp(ramp->blue,  &gamma.blue)) {
-	  Bool ret;
-	  wine_tsx11_lock();
-	  ret = pXF86VidModeSetGamma(gdi_display, DefaultScreen(gdi_display), &gamma);
-	  wine_tsx11_unlock();
-	  return ret;
-      }
-  }
+#endif
+      ret = pXF86VidModeSetGamma(gdi_display, DefaultScreen(gdi_display), &gamma);
+  wine_tsx11_unlock();
 #endif /* X_XF86VidModeSetGamma */
-  return FALSE;
+  return ret;
 }
 
 #endif /* SONAME_LIBXXF86VM */
