@@ -97,6 +97,9 @@ typedef DWORD CONFIGRET;
 #define CR_INVALID_STRUCTURE_SIZE       0x3b
 #define NUM_CR_RESULTS                  0x3c
 
+#define MAX_DEVICE_ID_LEN               200
+#define MAX_DEVNODE_ID_LEN              MAX_DEVICE_ID_LEN
+
 #define MAX_CLASS_NAME_LEN              32
 #define MAX_GUID_STRING_LEN             39
 #define MAX_PROFILE_LEN                 80
@@ -117,10 +120,16 @@ extern "C" {
 CMAPI CONFIGRET WINAPI CM_Connect_MachineA(PCSTR,PHMACHINE);
 CMAPI CONFIGRET WINAPI CM_Connect_MachineW(PCWSTR,PHMACHINE);
 #define     CM_Connect_Machine WINELIB_NAME_AW(CM_Connect_Machine)
+CMAPI CONFIGRET WINAPI CM_Create_DevNodeA(PDEVINST,DEVINSTID_A,DEVINST,ULONG);
+CMAPI CONFIGRET WINAPI CM_Create_DevNodeW(PDEVINST,DEVINSTID_W,DEVINST,ULONG);
+#define     CM_Create_DevNode WINELIB_NAME_AW(CM_Create_DevNode)
 CMAPI CONFIGRET WINAPI CM_Disconnect_Machine(HMACHINE);
 CMAPI CONFIGRET WINAPI CM_Get_Device_IDA(DEVINST,PSTR,ULONG,ULONG);
 CMAPI CONFIGRET WINAPI CM_Get_Device_IDW(DEVINST,PWSTR,ULONG,ULONG);
 #define     CM_Get_Device_ID WINELIB_NAME_AW(CM_Get_Device_ID)
+CMAPI CONFIGRET WINAPI CM_Get_Device_ID_ExA(DEVINST,PSTR,ULONG,ULONG,HMACHINE);
+CMAPI CONFIGRET WINAPI CM_Get_Device_ID_ExW(DEVINST,PWSTR,ULONG,ULONG,HMACHINE);
+#define     CM_Get_Device_ID_Ex WINELIB_NAME_AW(CM_Get_Device_ID_Ex)
 CMAPI CONFIGRET WINAPI CM_Get_Device_ID_ListA(PCSTR,PCHAR,ULONG,ULONG);
 CMAPI CONFIGRET WINAPI CM_Get_Device_ID_ListW(PCWSTR,PWCHAR,ULONG,ULONG);
 #define     CM_Get_Device_ID_List WINELIB_NAME_AW(CM_Get_Device_ID_List)
@@ -129,6 +138,9 @@ CMAPI CONFIGRET WINAPI CM_Get_Device_ID_List_ExW(PCWSTR,PWCHAR,ULONG,ULONG,HMACH
 #define     CM_Get_Device_ID_List_Ex WINELIB_NAME_AW(CM_Get_Device_ID_List_Ex)
 CMAPI CONFIGRET WINAPI CM_Get_Device_ID_Size(PULONG,DEVINST,ULONG);
 CMAPI CONFIGRET WINAPI CM_Get_Device_ID_Size_Ex(PULONG,DEVINST,ULONG,HMACHINE);
+CMAPI CONFIGRET WINAPI CM_Locate_DevNodeA(PDEVINST,DEVINSTID_A,ULONG);
+CMAPI CONFIGRET WINAPI CM_Locate_DevNodeW(PDEVINST,DEVINSTID_W,ULONG);
+#define     CM_Locate_DevNode WINELIB_NAME_AW(CM_Locate_DevNode)
 
 #ifdef __cplusplus
 }
