@@ -89,7 +89,7 @@ void WCMD_batch (WCHAR *file, WCHAR *command, int called, WCHAR *startLabel, HAN
 
   while (context -> skip_rest == FALSE) {
       CMD_LIST *toExecute = NULL;         /* Commands left to be executed */
-      if (WCMD_ReadAndParseLine(NULL, &toExecute, h) == NULL)
+      if (!WCMD_ReadAndParseLine(NULL, &toExecute, h))
         break;
       WCMD_process_commands(toExecute, FALSE, NULL, NULL);
       WCMD_free_commands(toExecute);
@@ -176,29 +176,54 @@ WCHAR *WCMD_parameter (WCHAR *s, int n, WCHAR **where, WCHAR **end) {
 /****************************************************************************
  * WCMD_fgets
  *
- * Get one line from a batch file. We can't use the native f* functions because
- * of the filename syntax differences between DOS and Unix. Also need to lose
- * the LF (or CRLF) from the line.
+ * Gets one line from a file/console and puts it into buffer buf
+ * Pre:  buf has size noChars
+ *       1 <= noChars <= MAXSTRING
+ * Post: buf is filled with at most noChars-1 characters, and gets nul-terminated
+         buf does not include EOL terminator
+ * Returns:
+ *       buf on success
+ *       NULL on error or EOF
  */
 
-WCHAR *WCMD_fgets (WCHAR *s, int noChars, HANDLE h) {
-
-  DWORD bytes;
+WCHAR *WCMD_fgets(WCHAR *buf, int noChars, HANDLE h)
+{
+  DWORD charsRead;
   BOOL status;
-  WCHAR *p;
+  LARGE_INTEGER filepos;
+  int i;
 
-  p = s;
-  do {
-    status = WCMD_ReadFile (h, s, 1, &bytes, NULL);
-    if ((status == 0) || ((bytes == 0) && (s == p))) return NULL;
-    if (*s == '\n') bytes = 0;
-    else if (*s != '\r') {
-      s++;
-      noChars--;
-    }
-    *s = '\0';
-  } while ((bytes == 1) && (noChars > 1));
-  return p;
+  /* We can't use the native f* functions because of the filename syntax differences
+     between DOS and Unix. Also need to lose the LF (or CRLF) from the line. */
+
+  if (!WCMD_is_console_handle(h)) {
+    /* Save current file position */
+    filepos.QuadPart = 0;
+    SetFilePointerEx(h, filepos, &filepos, FILE_CURRENT);
+  }
+
+  status = WCMD_ReadFile(h, buf, noChars, &charsRead);
+  if (!status || charsRead == 0) return NULL;
+
+  /* Find first EOL */
+  for (i = 0; i < charsRead; i++) {
+    if (buf[i] == '\n' || buf[i] == '\r')
+      break;
+  }
+
+  if (!WCMD_is_console_handle(h) && i != charsRead) {
+    /* Sets file pointer to the start of the next line, if any */
+    filepos.QuadPart += i + 1 + (buf[i] == '\r' ? 1 : 0);
+    SetFilePointerEx(h, filepos, NULL, FILE_BEGIN);
+  }
+
+  /* Truncate at EOL (or end of buffer) */
+  if (i == noChars)
+    i--;
+
+  buf[i] = '\0';
+
+  return buf;
 }
 
 /* WCMD_splitpath - copied from winefile as no obvious way to use it otherwise */
@@ -585,7 +610,7 @@ void WCMD_call (WCHAR *command) {
       SetFilePointer(context -> h, li.u.LowPart,
                      &li.u.HighPart, FILE_BEGIN);
     } else {
-      WCMD_output_asis( WCMD_LoadMessage(WCMD_CALLINSCRIPT));
+      WCMD_output_asis_stderr(WCMD_LoadMessage(WCMD_CALLINSCRIPT));
     }
   }
 }
