@@ -115,8 +115,7 @@ static BOOL WCMD_ask_confirm (const WCHAR *message, BOOL showSureText,
           WCMD_output_asis (Abuffer);
       }
       WCMD_output_asis (endBkt);
-      WCMD_ReadFile (GetStdHandle(STD_INPUT_HANDLE), answer,
-                     sizeof(answer)/sizeof(WCHAR), &count, NULL);
+      WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), answer, sizeof(answer)/sizeof(WCHAR), &count);
       answer[0] = toupperW(answer[0]);
     }
 
@@ -306,7 +305,7 @@ void WCMD_choice (const WCHAR * command) {
     while (TRUE) {
 
         /* FIXME: Add support for option /T */
-        WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), answer, 1, &count, NULL);
+        WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), answer, 1, &count);
 
         if (!opt_s)
             answer[0] = toupperW(answer[0]);
@@ -524,7 +523,7 @@ void WCMD_create_dir (WCHAR *command) {
     WCHAR *argN = command;
 
     if (param1[0] == 0x00) {
-        WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
+        WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOARG));
         return;
     }
     /* Loop through all args */
@@ -575,7 +574,7 @@ static void WCMD_delete_parse_attributes(DWORD *wantSet, DWORD *wantClear) {
             case 'S': mask = FILE_ATTRIBUTE_SYSTEM;   break;
             case 'A': mask = FILE_ATTRIBUTE_ARCHIVE;  break;
             default:
-                WCMD_output (WCMD_LoadMessage(WCMD_SYNTAXERR));
+                WCMD_output_stderr(WCMD_LoadMessage(WCMD_SYNTAXERR));
             }
             if (negate)
                 *wantClear |= mask;
@@ -843,14 +842,14 @@ BOOL WCMD_delete (WCHAR *command) {
         found = WCMD_delete_one(thisArg);
         if (!found) {
             errorlevel = 1;
-            WCMD_output (WCMD_LoadMessage(WCMD_FILENOTFOUND), thisArg);
+            WCMD_output_stderr(WCMD_LoadMessage(WCMD_FILENOTFOUND), thisArg);
         }
         foundAny |= found;
     }
 
     /* Handle no valid args */
     if (!argsProcessed)
-        WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
+        WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOARG));
 
     return foundAny;
 }
@@ -1125,7 +1124,7 @@ void WCMD_for (WCHAR *p, CMD_LIST **cmdList) {
           /* Process the input file */
           if (input == INVALID_HANDLE_VALUE) {
             WCMD_print_error ();
-            WCMD_output (WCMD_LoadMessage(WCMD_READFAIL), item);
+            WCMD_output_stderr(WCMD_LoadMessage(WCMD_READFAIL), item);
             errorlevel = 1;
             return; /* FOR loop aborts at first failure here */
 
@@ -1134,7 +1133,7 @@ void WCMD_for (WCHAR *p, CMD_LIST **cmdList) {
             WCHAR buffer[MAXSTRING] = {'\0'};
             WCHAR *where, *parm;
 
-            while (WCMD_fgets (buffer, sizeof(buffer)/sizeof(WCHAR), input)) {
+            while (WCMD_fgets(buffer, sizeof(buffer)/sizeof(WCHAR), input)) {
 
               /* Skip blank lines*/
               parm = WCMD_parameter (buffer, 0, &where, NULL);
@@ -1380,7 +1379,7 @@ void WCMD_goto (CMD_LIST **cmdList) {
     static const WCHAR eofW[] = {':','e','o','f','\0'};
 
     if (param1[0] == 0x00) {
-      WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
+      WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOARG));
       return;
     }
 
@@ -1408,7 +1407,7 @@ void WCMD_goto (CMD_LIST **cmdList) {
         if (lstrcmpiW (current, paramStart) == 0) return;
       }
     }
-    WCMD_output (WCMD_LoadMessage(WCMD_NOTARGET));
+    WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOTARGET));
   }
   return;
 }
@@ -1540,7 +1539,7 @@ void WCMD_if (WCHAR *p, CMD_LIST **cmdList) {
     WCMD_parameter(s, 1, &command, NULL);
   }
   else {
-    WCMD_output (WCMD_LoadMessage(WCMD_SYNTAXERR));
+    WCMD_output_stderr(WCMD_LoadMessage(WCMD_SYNTAXERR));
     return;
   }
 
@@ -1568,7 +1567,7 @@ void WCMD_move (void) {
   WCHAR            ext[MAX_PATH];
 
   if (param1[0] == 0x00) {
-    WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
+    WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOARG));
     return;
   }
 
@@ -1588,7 +1587,10 @@ void WCMD_move (void) {
   WCMD_splitpath(input, drive, dir, fname, ext);
 
   hff = FindFirstFileW(input, &fd);
-  while (hff != INVALID_HANDLE_VALUE) {
+  if (hff == INVALID_HANDLE_VALUE)
+    return;
+
+  do {
     WCHAR  dest[MAX_PATH];
     WCHAR  src[MAX_PATH];
     DWORD attribs;
@@ -1675,30 +1677,33 @@ void WCMD_move (void) {
       WCMD_print_error ();
       errorlevel = 1;
     }
+  } while (FindNextFileW(hff, &fd) != 0);
 
-    /* Step on to next match */
-    if (FindNextFileW(hff, &fd) == 0) {
-      FindClose(hff);
-      hff = INVALID_HANDLE_VALUE;
-      break;
-    }
-  }
+  FindClose(hff);
 }
 
 /****************************************************************************
  * WCMD_pause
  *
- * Wait for keyboard input.
+ * Suspend execution of a batch script until a key is typed
  */
 
-void WCMD_pause (void) {
-
+void WCMD_pause (void)
+{
+  DWORD oldmode;
+  BOOL have_console;
   DWORD count;
-  WCHAR string[32];
+  WCHAR key;
+  HANDLE hIn = GetStdHandle(STD_INPUT_HANDLE);
 
-  WCMD_output (anykey);
-  WCMD_ReadFile (GetStdHandle(STD_INPUT_HANDLE), string,
-                 sizeof(string)/sizeof(WCHAR), &count, NULL);
+  have_console = GetConsoleMode(hIn, &oldmode);
+  if (have_console)
+      SetConsoleMode(hIn, 0);
+
+  WCMD_output(anykey);
+  WCMD_ReadFile(hIn, &key, 1, &count);
+  if (have_console)
+    SetConsoleMode(hIn, oldmode);
 }
 
 /****************************************************************************
@@ -1760,7 +1765,7 @@ void WCMD_remove_dir (WCHAR *command) {
 
   /* Handle no valid args */
   if (argsProcessed == 0) {
-    WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
+    WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOARG));
     return;
   }
 
@@ -1789,7 +1794,7 @@ void WCMD_rename (void) {
 
   /* Must be at least two args */
   if (param1[0] == 0x00 || param2[0] == 0x00) {
-    WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
+    WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOARG));
     errorlevel = 1;
     return;
   }
@@ -1812,7 +1817,10 @@ void WCMD_rename (void) {
   WCMD_splitpath(input, drive, dir, fname, ext);
 
   hff = FindFirstFileW(input, &fd);
-  while (hff != INVALID_HANDLE_VALUE) {
+  if (hff == INVALID_HANDLE_VALUE)
+    return;
+
+ do {
     WCHAR  dest[MAX_PATH];
     WCHAR  src[MAX_PATH];
     WCHAR *dotSrc = NULL;
@@ -1868,14 +1876,9 @@ void WCMD_rename (void) {
       WCMD_print_error ();
       errorlevel = 1;
     }
+  } while (FindNextFileW(hff, &fd) != 0);
 
-    /* Step on to next match */
-    if (FindNextFileW(hff, &fd) == 0) {
-      FindClose(hff);
-      hff = INVALID_HANDLE_VALUE;
-      break;
-    }
-  }
+  FindClose(hff);
 }
 
 /*****************************************************************************
@@ -2060,33 +2063,26 @@ void WCMD_setshow_default (const WCHAR *command) {
     /* Search for appropriate directory */
     WINE_TRACE("Looking for directory '%s'\n", wine_dbgstr_w(string));
     hff = FindFirstFileW(string, &fd);
-    while (hff != INVALID_HANDLE_VALUE) {
-      if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
-        WCHAR fpath[MAX_PATH];
-        WCHAR drive[10];
-        WCHAR dir[MAX_PATH];
-        WCHAR fname[MAX_PATH];
-        WCHAR ext[MAX_PATH];
-        static const WCHAR fmt[] = {'%','s','%','s','%','s','\0'};
+    if (hff != INVALID_HANDLE_VALUE) {
+      do {
+        if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
+          WCHAR fpath[MAX_PATH];
+          WCHAR drive[10];
+          WCHAR dir[MAX_PATH];
+          WCHAR fname[MAX_PATH];
+          WCHAR ext[MAX_PATH];
+          static const WCHAR fmt[] = {'%','s','%','s','%','s','\0'};
 
-        /* Convert path into actual directory spec */
-        GetFullPathNameW(string, sizeof(fpath)/sizeof(WCHAR), fpath, NULL);
-        WCMD_splitpath(fpath, drive, dir, fname, ext);
+          /* Convert path into actual directory spec */
+          GetFullPathNameW(string, sizeof(fpath)/sizeof(WCHAR), fpath, NULL);
+          WCMD_splitpath(fpath, drive, dir, fname, ext);
 
-        /* Rebuild path */
-        wsprintfW(string, fmt, drive, dir, fd.cFileName);
-
-        FindClose(hff);
-        hff = INVALID_HANDLE_VALUE;
-        break;
-      }
-
-      /* Step on to next match */
-      if (FindNextFileW(hff, &fd) == 0) {
-        FindClose(hff);
-        hff = INVALID_HANDLE_VALUE;
-        break;
-      }
+          /* Rebuild path */
+          wsprintfW(string, fmt, drive, dir, fd.cFileName);
+          break;
+        }
+      } while (FindNextFileW(hff, &fd) != 0);
+      FindClose(hff);
     }
 
     /* Change to that directory */
@@ -2146,8 +2142,7 @@ void WCMD_setshow_date (void) {
       WCMD_output (WCMD_LoadMessage(WCMD_CURRENTDATE), curdate);
       if (strstrW (quals, parmT) == NULL) {
         WCMD_output (WCMD_LoadMessage(WCMD_NEWDATE));
-        WCMD_ReadFile (GetStdHandle(STD_INPUT_HANDLE),
-                       buffer, sizeof(buffer)/sizeof(WCHAR), &count, NULL);
+        WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), buffer, sizeof(buffer)/sizeof(WCHAR), &count);
         if (count > 2) {
           WCMD_output (WCMD_LoadMessage(WCMD_NYI));
         }
@@ -2256,7 +2251,7 @@ void WCMD_setshow_env (WCHAR *s) {
 
     /* If no parameter, or no '=' sign, return an error */
     if (!(*s) || ((p = strchrW (s, '=')) == NULL )) {
-      WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
+      WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOARG));
       return;
     }
 
@@ -2265,8 +2260,7 @@ void WCMD_setshow_env (WCHAR *s) {
     if (strlenW(p) != 0) WCMD_output(p);
 
     /* Read the reply */
-    WCMD_ReadFile (GetStdHandle(STD_INPUT_HANDLE), string,
-                   sizeof(string)/sizeof(WCHAR), &count, NULL);
+    WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), string, sizeof(string)/sizeof(WCHAR), &count);
     if (count > 1) {
       string[count-1] = '\0'; /* ReadFile output is not null-terminated! */
       if (string[count-2] == '\r') string[count-2] = '\0'; /* Under Windoze we get CRLF! */
@@ -2284,7 +2278,7 @@ void WCMD_setshow_env (WCHAR *s) {
     if (p == NULL) {
       env = GetEnvironmentStringsW();
       if (WCMD_setshow_sortenv( env, s ) == 0) {
-        WCMD_output (WCMD_LoadMessage(WCMD_MISSINGENV), s);
+        WCMD_output_stderr(WCMD_LoadMessage(WCMD_MISSINGENV), s);
         errorlevel = 1;
       }
       return;
@@ -2321,7 +2315,7 @@ void WCMD_setshow_path (const WCHAR *command) {
       WCMD_output_asis ( newline);
     }
     else {
-      WCMD_output (WCMD_LoadMessage(WCMD_NOPATH));
+      WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOPATH));
     }
   }
   else {
@@ -2376,8 +2370,7 @@ void WCMD_setshow_time (void) {
       WCMD_output (WCMD_LoadMessage(WCMD_CURRENTTIME), curtime);
       if (strstrW (quals, parmT) == NULL) {
         WCMD_output (WCMD_LoadMessage(WCMD_NEWTIME));
-        WCMD_ReadFile (GetStdHandle(STD_INPUT_HANDLE), buffer,
-                       sizeof(buffer)/sizeof(WCHAR), &count, NULL);
+        WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), buffer, sizeof(buffer)/sizeof(WCHAR), &count);
         if (count > 2) {
           WCMD_output (WCMD_LoadMessage(WCMD_NYI));
         }
@@ -2445,7 +2438,7 @@ void WCMD_type (WCHAR *command) {
   BOOL  writeHeaders  = FALSE;
 
   if (param1[0] == 0x00) {
-    WCMD_output (WCMD_LoadMessage(WCMD_NOARG));
+    WCMD_output_stderr(WCMD_LoadMessage(WCMD_NOARG));
     return;
   }
 
@@ -2467,14 +2460,14 @@ void WCMD_type (WCHAR *command) {
 		FILE_ATTRIBUTE_NORMAL, NULL);
     if (h == INVALID_HANDLE_VALUE) {
       WCMD_print_error ();
-      WCMD_output (WCMD_LoadMessage(WCMD_READFAIL), thisArg);
+      WCMD_output(WCMD_LoadMessage(WCMD_READFAIL), thisArg); /* should be _stderr */
       errorlevel = 1;
     } else {
       if (writeHeaders) {
         static const WCHAR fmt[] = {'\n','%','s','\n','\n','\0'};
         WCMD_output(fmt, thisArg);
       }
-      while (WCMD_ReadFile (h, buffer, sizeof(buffer)/sizeof(WCHAR) - 1, &count, NULL)) {
+      while (WCMD_ReadFile(h, buffer, sizeof(buffer)/sizeof(WCHAR) - 1, &count)) {
         if (count == 0) break;	/* ReadFile reports success on EOF! */
         buffer[count] = 0;
         WCMD_output_asis (buffer);
@@ -2528,7 +2521,7 @@ void WCMD_more (WCHAR *command) {
     wsprintfW(moreStrPage, moreFmt, moreStr);
 
     WCMD_enter_paged_mode(moreStrPage);
-    while (WCMD_ReadFile (hstdin, buffer, (sizeof(buffer)/sizeof(WCHAR))-1, &count, NULL)) {
+    while (WCMD_ReadFile(hstdin, buffer, (sizeof(buffer)/sizeof(WCHAR))-1, &count)) {
       if (count == 0) break;	/* ReadFile reports success on EOF! */
       buffer[count] = 0;
       WCMD_output_asis (buffer);
@@ -2559,8 +2552,7 @@ void WCMD_more (WCHAR *command) {
         wsprintfW(moreStrPage, moreFmt2, moreStr, 100);
         WCMD_leave_paged_mode();
         WCMD_output_asis(moreStrPage);
-        WCMD_ReadFile (GetStdHandle(STD_INPUT_HANDLE), buffer,
-                       sizeof(buffer)/sizeof(WCHAR), &count, NULL);
+        WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), buffer, sizeof(buffer)/sizeof(WCHAR), &count);
         WCMD_enter_paged_mode(moreStrPage);
       }
 
@@ -2570,7 +2562,7 @@ void WCMD_more (WCHAR *command) {
 		FILE_ATTRIBUTE_NORMAL, NULL);
       if (h == INVALID_HANDLE_VALUE) {
         WCMD_print_error ();
-        WCMD_output (WCMD_LoadMessage(WCMD_READFAIL), thisArg);
+        WCMD_output_stderr(WCMD_LoadMessage(WCMD_READFAIL), thisArg);
         errorlevel = 1;
       } else {
         ULONG64 curPos  = 0;
@@ -2582,7 +2574,7 @@ void WCMD_more (WCHAR *command) {
         fileLen = (((ULONG64)fileInfo.nFileSizeHigh) << 32) + fileInfo.nFileSizeLow;
 
         needsPause = TRUE;
-        while (WCMD_ReadFile (h, buffer, (sizeof(buffer)/sizeof(WCHAR))-1, &count, NULL)) {
+        while (WCMD_ReadFile(h, buffer, (sizeof(buffer)/sizeof(WCHAR))-1, &count)) {
           if (count == 0) break;	/* ReadFile reports success on EOF! */
           buffer[count] = 0;
           curPos += count;
@@ -2626,7 +2618,7 @@ void WCMD_verify (const WCHAR *command) {
     verify_mode = FALSE;
     return;
   }
-  else WCMD_output (WCMD_LoadMessage(WCMD_VERIFYERR));
+  else WCMD_output_stderr(WCMD_LoadMessage(WCMD_VERIFYERR));
 }
 
 /****************************************************************************
@@ -2667,7 +2659,7 @@ int WCMD_volume(BOOL set_label, const WCHAR *path)
   else {
     static const WCHAR fmt[] = {'%','s','\\','\0'};
     if ((path[1] != ':') || (strlenW(path) != 2)) {
-      WCMD_output (WCMD_LoadMessage(WCMD_SYNTAXERR));
+      WCMD_output_stderr(WCMD_LoadMessage(WCMD_SYNTAXERR));
       return 0;
     }
     wsprintfW (curdir, fmt, path);
@@ -2683,8 +2675,7 @@ int WCMD_volume(BOOL set_label, const WCHAR *path)
     	curdir[0], label, HIWORD(serial), LOWORD(serial));
   if (set_label) {
     WCMD_output (WCMD_LoadMessage(WCMD_VOLUMEPROMPT));
-    WCMD_ReadFile (GetStdHandle(STD_INPUT_HANDLE), string,
-                   sizeof(string)/sizeof(WCHAR), &count, NULL);
+    WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), string, sizeof(string)/sizeof(WCHAR), &count);
     if (count > 1) {
       string[count-1] = '\0';		/* ReadFile output is not null-terminated! */
       if (string[count-2] == '\r') string[count-2] = '\0'; /* Under Windoze we get CRLF! */
@@ -2905,18 +2896,16 @@ void WCMD_assoc (const WCHAR *command, BOOL assoc) {
 /****************************************************************************
  * WCMD_color
  *
- * Clear the terminal screen.
+ * Colors the terminal screen.
  */
 
 void WCMD_color (void) {
 
-  /* Emulate by filling the screen from the top left to bottom right with
-        spaces, then moving the cursor to the top left afterwards */
   CONSOLE_SCREEN_BUFFER_INFO consoleInfo;
   HANDLE hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);
 
   if (param1[0] != 0x00 && strlenW(param1) > 2) {
-    WCMD_output (WCMD_LoadMessage(WCMD_ARGERR));
+    WCMD_output_stderr(WCMD_LoadMessage(WCMD_ARGERR));
     return;
   }
 
