@@ -33,8 +33,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(cmd);
 
-
-extern int echo_mode;
 extern WCHAR quals[MAX_PATH], param1[MAX_PATH], param2[MAX_PATH];
 extern DWORD errorlevel;
 
@@ -53,13 +51,12 @@ typedef enum _DISPLAYORDER
     Date
 } DISPLAYORDER;
 
-static int file_total, dir_total, recurse, wide, bare, max_width, lower;
-static int shortname, usernames;
+static int file_total, dir_total, max_width;
 static ULONGLONG byte_total;
 static DISPLAYTIME dirTime;
 static DISPLAYORDER dirOrder;
 static BOOL orderReverse, orderGroupDirs, orderGroupDirsReverse, orderByCol;
-static BOOL separator;
+static BOOL paged_mode, recurse, wide, bare, lower, shortname, usernames, separator;
 static ULONG showattrs, attrsbits;
 
 static const WCHAR dotW[]    = {'.','\0'};
@@ -633,7 +630,7 @@ static void WCMD_dir_trailer(WCHAR drive) {
 void WCMD_directory (WCHAR *cmd) {
 
   WCHAR path[MAX_PATH], cwd[MAX_PATH];
-  int status, paged_mode;
+  int status;
   CONSOLE_SCREEN_BUFFER_INFO consoleInfo;
   WCHAR *p;
   WCHAR string[MAXSTRING];
@@ -839,7 +836,7 @@ void WCMD_directory (WCHAR *cmd) {
   prevEntry = NULL;
   while (argN) {
     WCHAR fullname[MAXSTRING];
-    WCHAR *thisArg = WCMD_parameter (cmd, argno++, &argN);
+    WCHAR *thisArg = WCMD_parameter(cmd, argno++, &argN, NULL);
     if (argN && argN[0] != '/') {
 
       WINE_TRACE("Found parm '%s'\n", wine_dbgstr_w(thisArg));
