@@ -102,6 +102,19 @@ static char  *output_bufA = NULL;
 #define MAX_WRITECONSOLE_SIZE 65535
 static BOOL unicodePipes = FALSE;
 
+/*
+ * Returns a buffer for reading from/writing to file
+ * Never freed
+ */
+static char *get_file_buffer(void)
+{
+    if (!output_bufA) {
+        output_bufA = HeapAlloc(GetProcessHeap(), 0, MAX_WRITECONSOLE_SIZE);
+        if (!output_bufA)
+            WINE_FIXME("Out of memory - could not allocate ansi 64K buffer\n");
+    }
+    return output_bufA;
+}
 
 /*******************************************************************
  * WCMD_output_asis_len - send output to current standard output
@@ -126,23 +139,18 @@ static void WCMD_output_asis_len(const WCHAR *message, int len, HANDLE device) {
     if (!res) {
       BOOL usedDefaultChar = FALSE;
       DWORD convertedChars;
+      char *buffer;
 
       if (!unicodePipes) {
-        /*
-         * Allocate buffer to use when writing to file. (Not freed, as one off)
-         */
-        if (!output_bufA) output_bufA = HeapAlloc(GetProcessHeap(), 0,
-                                                  MAX_WRITECONSOLE_SIZE);
-        if (!output_bufA) {
-          WINE_FIXME("Out of memory - could not allocate ansi 64K buffer\n");
-          return;
-        }
+
+        if (!(buffer = get_file_buffer()))
+            return;
 
         /* Convert to OEM, then output */
         convertedChars = WideCharToMultiByte(GetConsoleOutputCP(), 0, message,
-                            len, output_bufA, MAX_WRITECONSOLE_SIZE,
+                            len, buffer, MAX_WRITECONSOLE_SIZE,
                             "?", &usedDefaultChar);
-        WriteFile(device, output_bufA, convertedChars,
+        WriteFile(device, buffer, convertedChars,
                   &nOut, FALSE);
       } else {
         WriteFile(device, message, len*sizeof(WCHAR),
@@ -166,7 +174,7 @@ void WCMD_output (const WCHAR *format, ...) {
   va_start(ap,format);
   ret = vsnprintfW(string, sizeof(string)/sizeof(WCHAR), format, ap);
   if( ret >= (sizeof(string)/sizeof(WCHAR))) {
-       WINE_ERR("Output truncated in WCMD_output\n" );
+       WINE_ERR("Output truncated\n" );
        ret = (sizeof(string)/sizeof(WCHAR)) - 1;
        string[ret] = '\0';
   }
@@ -174,6 +182,27 @@ void WCMD_output (const WCHAR *format, ...) {
   WCMD_output_asis_len(string, ret, GetStdHandle(STD_OUTPUT_HANDLE));
 }
 
+/*******************************************************************
+ * WCMD_output_stderr - send output to current standard error device.
+ *
+ */
+
+void WCMD_output_stderr (const WCHAR *format, ...) {
+
+  va_list ap;
+  WCHAR string[1024];
+  int ret;
+
+  va_start(ap,format);
+  ret = vsnprintfW(string, sizeof(string)/sizeof(WCHAR), format, ap);
+  if( ret >= (sizeof(string)/sizeof(WCHAR))) {
+       WINE_ERR("Output truncated\n" );
+       ret = (sizeof(string)/sizeof(WCHAR)) - 1;
+       string[ret] = '\0';
+  }
+  va_end(ap);
+  WCMD_output_asis_len(string, ret, GetStdHandle(STD_ERROR_HANDLE));
+}
 
 static int line_count;
 static int max_height;
@@ -208,40 +237,26 @@ void WCMD_leave_paged_mode(void)
  * WCMD_Readfile
  *
  *	Read characters in from a console/file, returning result in Unicode
- *      with signature identical to ReadFile
  */
-BOOL WCMD_ReadFile(const HANDLE hIn, WCHAR *intoBuf, const DWORD maxChars,
-                          LPDWORD charsRead, const LPOVERLAPPED unused) {
+BOOL WCMD_ReadFile(const HANDLE hIn, WCHAR *intoBuf, const DWORD maxChars, LPDWORD charsRead)
+{
+    DWORD numRead;
+    char *buffer;
 
-    BOOL   res;
+    if (WCMD_is_console_handle(hIn))
+        /* Try to read from console as Unicode */
+        return ReadConsoleW(hIn, intoBuf, maxChars, charsRead, NULL);
 
-    /* Try to read from console as Unicode */
-    res = ReadConsoleW(hIn, intoBuf, maxChars, charsRead, NULL);
+    /* We assume it's a file handle and read then convert from assumed OEM codepage */
+    if (!(buffer = get_file_buffer()))
+        return FALSE;
 
-    /* If reading from console has failed we assume its file
-       i/o so read in and convert from OEM codepage               */
-    if (!res) {
+    if (!ReadFile(hIn, buffer, maxChars, &numRead, NULL))
+        return FALSE;
 
-        DWORD numRead;
-        /*
-         * Allocate buffer to use when reading from file. Not freed
-         */
-        if (!output_bufA) output_bufA = HeapAlloc(GetProcessHeap(), 0,
-                                                  MAX_WRITECONSOLE_SIZE);
-        if (!output_bufA) {
-          WINE_FIXME("Out of memory - could not allocate ansi 64K buffer\n");
-          return 0;
-        }
+    *charsRead = MultiByteToWideChar(GetConsoleCP(), 0, buffer, numRead, intoBuf, maxChars);
 
-        /* Read from file (assume OEM codepage) */
-        res = ReadFile(hIn, output_bufA, maxChars, &numRead, unused);
-
-        /* Convert from OEM */
-        *charsRead = MultiByteToWideChar(GetConsoleCP(), 0, output_bufA, numRead,
-                         intoBuf, maxChars);
-
-    }
-    return res;
+    return TRUE;
 }
 
 /*******************************************************************
@@ -269,8 +284,7 @@ static void WCMD_output_asis_handle (DWORD std_handle, const WCHAR *message) {
         if (++line_count >= max_height - 1) {
           line_count = 0;
           WCMD_output_asis_len(pagedMessage, strlenW(pagedMessage), handle);
-          WCMD_ReadFile (GetStdHandle(STD_INPUT_HANDLE), string,
-                         sizeof(string)/sizeof(WCHAR), &count, NULL);
+          WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), string, sizeof(string)/sizeof(WCHAR), &count);
         }
       }
     } while (((message = ptr) != NULL) && (*ptr));
@@ -1034,7 +1048,7 @@ void WCMD_run_program (WCHAR *command, int called) {
     if (strchrW(param1, '.') != NULL) extensionsupplied = TRUE;
     if (strlenW(param1) >= MAX_PATH)
     {
-        WCMD_output_asis(WCMD_LoadMessage(WCMD_LINETOOLONG));
+        WCMD_output_asis_stderr(WCMD_LoadMessage(WCMD_LINETOOLONG));
         return;
     }
 
@@ -1762,8 +1776,8 @@ static BOOL WCMD_IsEndQuote(const WCHAR *quote, int quoteIndex)
  *     - Anything else gets put into the command string (including
  *            redirects)
  */
-WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE readFrom) {
-
+WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE readFrom)
+{
     WCHAR    *curPos;
     int       inQuotes = 0;
     WCHAR     curString[MAXSTRING];
@@ -1807,15 +1821,16 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
     } else if (readFrom == INVALID_HANDLE_VALUE) {
         WINE_FIXME("No command nor handle supplied\n");
     } else {
-        if (WCMD_fgets(extraSpace, MAXSTRING, readFrom) == NULL) return NULL;
+        if (!WCMD_fgets(extraSpace, MAXSTRING, readFrom))
+          return NULL;
     }
     curPos = extraSpace;
 
     /* Handle truncated input - issue warning */
     if (strlenW(extraSpace) == MAXSTRING -1) {
-        WCMD_output_asis(WCMD_LoadMessage(WCMD_TRUNCATEDLINE));
-        WCMD_output_asis(extraSpace);
-        WCMD_output_asis(newline);
+        WCMD_output_asis_stderr(WCMD_LoadMessage(WCMD_TRUNCATEDLINE));
+        WCMD_output_asis_stderr(extraSpace);
+        WCMD_output_asis_stderr(newline);
     }
 
     /* Replace env vars if in a batch context */
@@ -2167,7 +2182,8 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
         /* Read more, skipping any blank lines */
         while (*extraSpace == 0x00) {
           if (!context) WCMD_output_asis( WCMD_LoadMessage(WCMD_MOREPROMPT));
-          if (WCMD_fgets(extraSpace, MAXSTRING, readFrom) == NULL) break;
+          if (!WCMD_fgets(extraSpace, MAXSTRING, readFrom))
+            break;
         }
         curPos = extraSpace;
         if (context) handleExpansion(extraSpace, FALSE, NULL, NULL);
@@ -2587,8 +2603,7 @@ int wmain (int argc, WCHAR *argvW[])
     /* Read until EOF (which for std input is never, but if redirect
        in place, may occur                                          */
     if (echo_mode) WCMD_show_prompt();
-    if (WCMD_ReadAndParseLine(NULL, &toExecute,
-                              GetStdHandle(STD_INPUT_HANDLE)) == NULL)
+    if (!WCMD_ReadAndParseLine(NULL, &toExecute, GetStdHandle(STD_INPUT_HANDLE)))
       break;
     WCMD_process_commands(toExecute, FALSE, NULL, NULL);
     WCMD_free_commands(toExecute);
