@@ -111,29 +111,6 @@ BOOL stack_get_current_frame(IMAGEHLP_STACK_FRAME* ihsf)
     return stack_get_frame(dbg_curr_thread->curr_frame, ihsf);
 }
 
-BOOL stack_get_register_current_frame(unsigned regno, DWORD_PTR** pval)
-{
-    enum be_cpu_addr            kind;
-
-    if (dbg_curr_thread->frames == NULL) return FALSE;
-
-    if (!be_cpu->get_register_info(regno, &kind)) return FALSE;
-
-    switch (kind)
-    {
-    case be_cpu_addr_pc:
-        *pval = &dbg_curr_thread->frames[dbg_curr_thread->curr_frame].linear_pc;
-        break;
-    case be_cpu_addr_stack:
-        *pval = &dbg_curr_thread->frames[dbg_curr_thread->curr_frame].linear_stack;
-        break;
-    case be_cpu_addr_frame:
-        *pval = &dbg_curr_thread->frames[dbg_curr_thread->curr_frame].linear_frame;
-        break;
-    }
-    return TRUE;
-}
-
 BOOL stack_get_register_frame(const struct dbg_internal_var* div, DWORD_PTR** pval)
 {
     if (dbg_curr_thread->frames == NULL) return FALSE;
@@ -214,7 +191,7 @@ unsigned stack_fetch_frames(const CONTEXT* _ctx)
     /* as native stackwalk can modify the context passed to it, simply copy
      * it to avoid any damage
      */
-    CONTEXT      ctx = *_ctx, prevctx = ctx;
+    CONTEXT      ctx = *_ctx;
 
     HeapFree(GetProcessHeap(), 0, dbg_curr_thread->frames);
     dbg_curr_thread->frames = NULL;
@@ -244,7 +221,7 @@ unsigned stack_fetch_frames(const CONTEXT* _ctx)
         dbg_curr_thread->frames[nf].linear_frame = (DWORD_PTR)memory_to_linear_addr(&sf.AddrFrame);
         dbg_curr_thread->frames[nf].addr_stack   = sf.AddrStack;
         dbg_curr_thread->frames[nf].linear_stack = (DWORD_PTR)memory_to_linear_addr(&sf.AddrStack);
-        dbg_curr_thread->frames[nf].context      = prevctx;
+        dbg_curr_thread->frames[nf].context      = ctx;
         /* FIXME: can this heuristic be improved: we declare first context always valid, and next ones
          * if it has been modified by the call to StackWalk...
          */
@@ -252,7 +229,6 @@ unsigned stack_fetch_frames(const CONTEXT* _ctx)
             (nf == 0 ||
              (dbg_curr_thread->frames[nf - 1].is_ctx_valid &&
               memcmp(&dbg_curr_thread->frames[nf - 1].context, &ctx, sizeof(ctx))));
-        prevctx = ctx;
         nf++;
         /* we've probably gotten ourselves into an infinite loop so bail */
         if (nf > 200) break;
